/**
 * TODO:
 * + launcher files must be prefixed with the artifactId value at build time
 */

'use strict';

// Node.js 7+ workaround for tmpDir deprecation
var os = require('os');
os.tmpDir = os.tmpdir;

// # Globbing
// for performance reasons we're only matching one level down:
// 'test/spec/{,*/}*.js'
// use this if you want to recursively match all subfolders:
// 'test/spec/**/*.js'

module.exports = function (grunt) {

  // Load grunt tasks automatically
  require('load-grunt-tasks')(grunt);

  // Time how long tasks take. Can help when optimizing build times
  require('time-grunt')(grunt);

  var _ = require('lodash');

  var packageVersion = _.find(_.find(require('xml-parser')(require('fs').readFileSync('./pom.xml', 'utf8')).root.children, {name : 'parent'}).children, {name : 'version'}).content;

  var globalConfig = {
      name             : "Akio Web Box",
      version          : packageVersion, // "<%= pkg.version %>
      customerBoxFolder: "akio-webbox",
      customerWebsiteFolder: "example",
      configFolder     : "data",
      srcDir           : 'src',
      sassDir          : '<%= yeoman.app %>/styles',
      buildDir         : 'build',
      tPackageDir      : 'build/package',
      tempDir          : 'build/temp',
      distDir          : 'dist',
      aipCode          : grunt.file.read('src/app/messages/aip.tpl.js'),
      /* Maven repository */
      releaseRepository: 'http://maven.akio.fr:8082/nexus/content/repositories/releases',
      snapshotRepository:'http://maven.akio.fr:8082/nexus/content/repositories/snapshots',
      releaseRepoId    : 'akio-releases',
      snapshotRepoId   : 'akio-snapshots'
    },
    uic = {
      artifactId       : grunt.file.readJSON('package.json').artifactId,
      groupId          : 'com.akio.webbox',
      env: {
        dev :{
          settings: grunt.file.readJSON('src/data/settings-dev.json'),
          dataDir : 'data/generated'
        },
        prod:{
          settings: grunt.file.readJSON('src/data/settings-prod.json'),
          dataDir : 'data'
        }
      },
      destDir          : 'uic'
    };

  globalConfig.uic = uic;

  var coreFiles = [
    'src/bower_components/jquery/dist/jquery.js',
    'src/bower_components/jquery-color/jquery.color.js',
    'src/bower_components/angular/angular.js',
    'src/bower_components/underscore/underscore.js',
    'src/bower_components/backbone/backbone.js',
    'src/bower_components/angular-animate/angular-animate.js',
    'src/bower_components/angular-resource/angular-resource.js',
    'src/bower_components/angular-sanitize/angular-sanitize.js',
    'src/bower_components/ng-content-editable/dist/ng-content-editable.js',
    'src/bower_components/mobile-detect/mobile-detect.js',
    'src/libs/backbone.domStorage.js', /*  the lib was no more reachable on git at a time. It is now available locally. */
    'src/bower_components/lodash/dist/lodash.js',
    'src/app/app.js',
    'build/temp/templates.js', /* html templates compiles as js */
    'src/app/AnchorButton/clickToChatFactory.js',
    'src/app/AnchorButton/clickToWcbFactory.js',
    'src/app/common/languageDetection.js',
    'src/app/common/animates.js',
    'src/app/common/constants.js',
    'src/app/common/filters.js',
    'src/app/common/fix1404.js',
    'src/app/common/fixIeSelect.js',
    'src/app/common/transformTextToSend.js',
    'src/app/common/xmlUtils.js',
    'src/app/common/determineClient.js',
    'src/app/common/emojisConstant.js',
    'src/app/invitation/triggerSvc.js',
    'src/app/invitation/triggerDrct.js',
    'src/app/layout/common/button.js',
    'src/app/layout/common/searchFormDrct.js',
    'src/app/layout/launchbar/launchBarCtrl.js',
    'src/app/layout/launchbar/launchBarDrct.js',
    'src/app/layout/window/westRegionContainerDrct.js',
    'src/app/layout/window/northRegionContainerDrct.js',
    'src/app/layout/window/titlebarDrct.js',
    'src/app/layout/window/windowCtrl.js',
    'src/app/layout/window/windowDrct.js',
    'src/app/layout/window/waitCtrl.js',
    'src/app/layout/window/chatFormCtrl.js',
    'src/app/layout/window/chatFormDrct.js',
    'src/app/layout/common/imageBrokenDrct.js',
    'src/app/media/media.js',
    'src/app/media/MediumModel.js',
    'src/app/media/wcb/wcbCtrl.js',
    'src/app/media/wcb/wcbDrct.js',
    'src/app/media/wcb/calendar/wcbCalendarCtrl.js',
    'src/app/media/wcb/calendar/wcbCalendarDrct.js',
    'src/app/media/wcb/confirm/wcbConfirmCtrl.js',
    'src/app/media/wcb/confirm/wcbConfirmDrct.js',
    'src/app/media/chat/emoji/emojiCtrl.js',
    'src/app/media/chat/emoji/emojiDrct.js',
    'src/app/media/chat/chatCtrl.js',
    'src/app/media/chat/ChatModule.js',
    'src/app/media/form/formCtrl.js',
    'src/app/media/form/FormModule.js',
    'src/app/media/form/formEvent.js',
    'src/app/media/form/surveyEvent.js',
    'src/app/media/chat/chatInvitationEvent.js',
    'src/app/media/form/fieldsetDrct.js',
    'src/app/navigation/navigation.js',
    'src/app/notification/notificationDrct.js',
    'src/app/notification/notificationCtrl.js',
    'src/app/notification/notificationSvc.js',
    'src/app/messages/AIPConnector.js',
    'build/temp/proxy.js', /* dynamically generated */
    'src/app/messages/CORSConnector.js',
    'src/app/messages/EventModel.js',
    'src/app/messages/readerSvc.js',
    'src/app/error/networkConnectionLazyEvent.js',
    'src/app/error/serviceInterruptedEvent.js',
    'src/app/error/errorHandler.js',
    'src/app/error/serviceUnavailableEvent.js',
    'src/app/services/messages.js',
    'src/app/services/configuration.js',
    'src/app/services/externalInterface.js',
    'src/app/session/dataContext.js',
    'src/app/session/session.js',
    'src/app/time-picker/timePickerDrct.js',
    'src/app/time-picker/timePickerCtrl.js',
    'src/app/time-picker/timePickerUtils.js',
    'src/app/widget/widgetDrct.js',
    'src/app/widget/widgetCtrl.js'
  ];

  // Define the configuration for all the tasks
  grunt.initConfig({
    pkg: grunt.file.readJSON('package.json'),
    globalConfig: globalConfig,
    copyright: '\r\n'+'Akio__Web__Box'.replace(/__/g,' ') +' v<%= globalConfig.version %>\r\nCopyright © <%= grunt.template.today("yyyy") %> Akio http://www.akio.com/\r\nAll rights reserved.\r\n',
    banner: '/* <%= copyright %> */\n',

    // Project settings
    yeoman: {
      // configurable paths
      app: require('./bower.json').appPath || 'src',
      dist: 'dist'
    },

    // Watches files for changes and runs tasks based on the changed files
    watch: {
      js: {
        files: ['<%= yeoman.app %>/app/**/{,*/}*.js'],
        tasks: ['newer:jshint:all'],
        options: {
          livereload: true
        }
      },
      jsTest: {
        files: ['test/spec/{,*/}*.js'],
        tasks: ['newer:jshint:test', 'karma']
      },
      compass: {
        files: ['<%= yeoman.app %>/styles/{,*/}*.{scss,sass}'],
        tasks: ['compass:server', 'autoprefixer']
      },
      gruntfile: {
        files: ['Gruntfile.js']
      },
      json: {
        files: ['<%= yeoman.app %>/data/settings-dev.json'],
        tasks: ['clean:generatedFiles', 'includereplace:uic_dev']
      },
      livereload: {
        options: {
          livereload: '<%= connect.options.livereload %>'
        },
        files: [
          '<%= yeoman.app %>/{,*/}*.html',
          '<%= yeoman.app %>/app/**/{,*/}*.tpl.html',
          '.tmp/styles/{,*/}*.css',
          '<%= yeoman.app %>/images/{,*/}*.{png,jpg,jpeg,gif,webp,svg}'
        ]
      }
    },

    // The actual grunt server settings
    connect: {
      options: {
        port: 9000,
        // Change this to '0.0.0.0' to access the server from outside.
        hostname: '0.0.0.0',
        livereload: 35729
      },
      livereload: {
        options: {
          open: true,
          base: [
            '.tmp',
            '<%= yeoman.app %>'
          ]
        }
      },
      dist: {
        options: {
          base: '<%= yeoman.dist %>'
        }
      }
    },

    // Make sure code styles are up to par and there are no obvious mistakes
    jshint: {
      options: {
        jshintrc: '.jshintrc',
        reporter: require('jshint-stylish')
      },
      all: [
        'Gruntfile.js',
        '<%= yeoman.app %>/app/**/*.js'
      ],
      test: {
        options: {
          jshintrc: 'test/.jshintrc'
        },
        src: ['test/spec/**/*.js']
      }
    },

    // Empties folders to start fresh
    clean: {
      //generatedFiles: ['src/data/generated/**'], /* remove generated files */
      generatedFiles: ['src/data/akio-webbox-settings.json'], /* remove generated files */
      dist: {
        files: [{
          dot: true,
          src: [
            '.tmp',
            '<%= globalConfig.buildDir %>/*',
            '<%= globalConfig.distDir %>/*',
            '!<%= globalConfig.distDir %>/.git*'
          ]
        }]
      },
      server: '.tmp'
    },

    // Add vendor prefixed styles
    autoprefixer: {
      options: {
        browsers: ['last 1 version']
      },
      dist: {
        files: [{
          expand: true,
          cwd: '.tmp/styles/',
          src: '{,*/}*.css',
          dest: '.tmp/styles/'
        }]
      }
    },

    // Automatically inject Bower components into the app
    'bower-install': {
      app: {
        html: '<%= yeoman.app %>/index.html',
        ignorePath: '<%= yeoman.app %>/',
        exclude: ['lodash']
      }
    },

    // Compiles Sass to CSS and generates necessary files if requested
    compass: {
      options: {
        // config: '.compass.rb',
        force: true,
        sassDir: '<%= yeoman.app %>/styles',
        cssDir: '.tmp/styles',
        generatedImagesDir: '.tmp/images/generated',
        imagesDir: '<%= yeoman.app %>/styles',
        javascriptsDir: '<%= yeoman.app %>/app',
        fontsDir: '<%= yeoman.app %>/styles/fonts',
        importPath: '<%= yeoman.app %>/bower_components',
        httpImagesPath: '/images',
        httpGeneratedImagesPath: '/images/generated',
        httpFontsPath: '/styles/fonts',
        relativeAssets: false,
        assetCacheBuster: false,
        raw: 'Sass::Script::Number.precision = 10\n'
      },
      dev: { /* build in development env. */
        options: {}
      },
      dist: {
        options: {
          cssDir            : '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>/styles',
          generatedImagesDir: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>/images',
          httpGeneratedImagesPath : '../images', /* images path relative to the dist dir in generated css */
          debugInfo         : false,
          outputStyle       : 'compressed',
          environment       : 'production',
          banner            : '<%= banner %>', /* works with «specify» */
          specify           : '<%= globalConfig.sassDir %>/akio-webbox.scss', /* files under sassDir that you want to compile */
          force             : true      /* give the rights to overwrite existing files */
        }
      },
      server: {
        options: {
          debugInfo: false,
          trace             : true,
          outputStyle       : 'expanded',
          environment       : 'development',
          force             : true
        }
      }
    },

    // Renames files for browser caching purposes
    rev: {
      dist: {
        files: {
          src: [
            '<%= yeoman.dist %>/app/{,*/}*.js',
            '<%= yeoman.dist %>/styles/{,*/}*.css',
            '<%= yeoman.dist %>/images/{,*/}*.{png,jpg,jpeg,gif,webp,svg}',
            '<%= yeoman.dist %>/styles/fonts/*'
          ]
        }
      }
    },

    // Reads HTML for usemin blocks to enable smart builds that automatically
    // concat, minify and revision files. Creates configurations in memory so
    // additional tasks can operate on them
    useminPrepare: {
      html: '<%= yeoman.app %>/index.html',
      options: {
        dest: '<%= yeoman.dist %>'
      }
    },

    // Performs rewrites based on rev and the useminPrepare configuration
    usemin: {
      html: ['<%= yeoman.dist %>/{,*/}*.html'],
      css: ['<%= yeoman.dist %>/styles/{,*/}*.css'],
      options: {
        assetsDirs: ['<%= yeoman.dist %>']
      }
    },

    // The following *-min tasks produce minified files in the dist folder
    imagemin: {
      dist: {
        files: [{
          expand: true,
          cwd: '<%= yeoman.app %>/images',
          src: '{,*/}*.{png,jpg,jpeg,gif}',
          dest: '<%= yeoman.dist %>/images'
        }]
      }
    },
    svgmin: {
      dist: {
        files: [{
          expand: true,
          cwd: '<%= yeoman.app %>/images',
          src: '{,*/}*.svg',
          dest: '<%= yeoman.dist %>/images'
        }]
      }
    },

    processhtml: {
      options: {
        data: {
          version: globalConfig.version
        }
      },
      js: {
        files: {
          '.tmp/js/proxy.js'                                    : 'src/app/messages/proxy.js',
          '.tmp/js/awbWidget-launcher.js'                       : 'src/app/awbWidget-launcher.js'
        }
      },
      jsDist: {
        files: {
          '<%= globalConfig.tempDir %>/proxy.js'                : 'src/app/messages/proxy.js',
          '<%= globalConfig.tempDir %>/akio-webbox-launcher.js' : 'src/app/akio-webbox-launcher.js'
        }
      }
    },

    htmlmin: {
      dist: {
        options: {
          collapseWhitespace: true,
          collapseBooleanAttributes: true,
          removeCommentsFromCDATA: true,
          removeOptionalTags: true
        },
        files: [{
          expand: true,
          cwd: '<%= yeoman.dist %>',
          src: ['*.html', 'views/{,*/}*.html'],
          dest: '<%= yeoman.dist %>'
        }]
      }
    },

    // Allow the use of non-minsafe AngularJS files. Automatically makes it
    // minsafe compatible so Uglify does not destroy the ng references
    ngmin: {
      dist: {
        files: [{
          expand: true,
          cwd: '.tmp/concat/app',
          src: '*.js',
          dest: '.tmp/concat/app'
        }]
      },
      test: {
        files: [{
          expand: true,
          cwd: 'dist/uic/akio-webbox/js/',
          src: 'akio-webbox-core-6.6.min.js',
          dest: 'dist/uic/akio-webbox/js/tmp'
        }]
      }
    },

    // Copies remaining files to places other tasks can use
    copy: {
      dist: {
        files: [{
          expand: true,
          dot: true,
          cwd: '<%= yeoman.app %>',
          dest: '<%= yeoman.dist %>',
          src: [
            '*.{ico,png,txt}',
            '.htaccess',
            '*.html',
            'views/{,*/}*.html',
            'bower_components/**/*',
            'images/{,*/}*.{webp}',
            'fonts/*'
          ]
        }, {
          expand: true,
          cwd: '.tmp/images',
          dest: '<%= yeoman.dist %>/images',
          src: ['generated/*']
        }]
      },

      styles: {
        expand: true,
        cwd: '<%= yeoman.app %>/styles',
        dest: '.tmp/styles/',
        src: '{,*/}*.css'
      },

      assets: {
        flatten: false,
        expand: true,
        files: [
          { /* copy example directory */
            expand: true,
            cwd: '<%= globalConfig.customerWebsiteFolder %>/',
            src: ['**'],
            dest: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerWebsiteFolder %>',
            rename: function(dest, src) {
              return [dest, src.replace(/.tpl/, '')].join('/');
            }
          },
          { /* copy fonts directory*/
            expand: true,
            cwd: '<%= globalConfig.srcDir %>/',
            src: ['fonts/**'],
            dest: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>'
          },
          { /* copy icomoon to fonts directory*/
            expand: true,
            cwd: 'src/fonts/',
            src: ['icomoon.ttf'],
            dest: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>/fonts/'
          },
          { /* copy the settings file */
            expand: true,
            cwd: '<%= globalConfig.srcDir %>',
            src: ['data/settings.tpl.json'],
            dest: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>',
            rename: function(dest, src) {
              var parts = src.split('/');
              parts[parts.length -1 ] = [globalConfig.uic.artifactId, parts[parts.length -1 ].replace(/.tpl/, '')].join('-');
              return ([dest].concat(parts)).join('/');
            }
          },
          { /* copy the 118n files */
            expand: true,
            cwd: '<%= globalConfig.srcDir %>/data/i18n',
            src: ['**'],
            dest: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>/data/i18n'
          },
          { /* copy font awesome & source sans pro to styles*/
            expand: true,
            cwd: '<%= globalConfig.srcDir %>/styles',
            src: [ 'sourceSansPro.css','openSans.css','custom-akio-webbox.css'],
            dest: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>/styles'
          },
          { /* copy google task manager javascript file*/
            expand: true,
            cwd: '<%= globalConfig.srcDir %>/js',
            src: [ 'akio_gtm.js'],
            dest: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>/js'
          },
          { /* copy images to images*/
            expand: true,
            cwd: '<%= globalConfig.srcDir %>/images',
            src: ['avatar-chat.png','spin.gif', 'agent.svg','resize.svg','closeCross.svg','roundedRectangleAgent.svg','resizeUp.svg','roundedRectangleWebUser.svg','helpBubble.svg','arrowdown.gif','loading.gif'],
            dest: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>/images'
          },
          { /* copy launcher file */
            expand: true,
            cwd: '<%= globalConfig.tempDir %>',
            src: ['<%= globalConfig.uic.artifactId %>-launcher.js'],
            dest: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>/js'

          }
        ]
      },

      source: {
        flatten: false,
        expand: true,
        files: [
          {//copy source
            expand:true,
            cwd:'',
            src:['src/**','akioTargeting/**','test/**','test/.jshintrc','example/**','*','.bowerrc','.compass.rb','.jshintrc','pom.xml','!build.sh','!assembly.xml'],
            dest:'<%= globalConfig.tPackageDir %>/source/box'
          },
          {//copy _style.scss for source
            expand:true,
            cwd:'node_modules/uic-common-css/src/main/resources/sass/svg/',
            src:['_style.scss'],
            dest:'<%= globalConfig.tPackageDir %>/source/box/src/styles/default'
          },
          {//copy source
            expand:true,
            cwd:'/js/',
            src:['akioTargeting.js'],
            dest:'<%= globalConfig.tPackageDir %>/source/box/akioTargeting'
          }
        ]
      },

      uic: {
        flatten: false,
        expand: true,
        files: [
          { /* data */
            expand: true,
            cwd: '<%= globalConfig.tPackageDir %>',
            src: ['**'],
            dest: '<%= globalConfig.distDir %>/<%= globalConfig.uic.destDir %>'
          },
          {
            expand: true,
            cwd: '<%= globalConfig.tPackageDir %>/source/box',
            src: ['.bowerrc','.compass.rb','.jshintrc','test/.jshintrc'],
            dest: '<%= globalConfig.distDir %>/<%= globalConfig.uic.destDir %>/source/box'
          }
        ]
      },

      'awb_cors' : {

        src : 'docs/awb_cors.conf',
        dest : '<%= globalConfig.tPackageDir %>/awb_cors.conf'
      }
    },

    // Run some tasks in parallel to speed up the build process
    concurrent: {
      server: [
        'compass:server'
      ],
      test: [
        //'compass'
      ],
      dist: [
        'compass:dist',
        //'imagemin',
        'svgmin'
      ]
    },

    // By default, your `index.html`'s <!-- Usemin block --> will take care of
    // minification. These next options are pre-configured if you do not wish
    // to use the Usemin blocks.
    cssmin: {
      dist: {
        files: {
          '<%= yeoman.dist %>/styles/main.css': [
            '.tmp/styles/{,*/}*.css',
            '<%= yeoman.app %>/styles/{,*/}*.css'
          ]
        }
      }
    },

    /**
     * compiles the html templates files to js
     */
    ngtemplates: {
      'akio.webbox': {
        cwd    :'src/',
        src    :'app/**/{,*/}*.tpl.html',
        dest   :'<%= globalConfig.tempDir %>/templates.js',
        options: {
          htmlmin: {
            collapseBooleanAttributes:      true,
            collapseWhitespace:             true,
            removeAttributeQuotes:          true,
            removeComments:                 true, // Only if you don't use comment directives!
            removeEmptyAttributes:          true,
            removeRedundantAttributes:      true,
            removeScriptTypeAttributes:     true,
            removeStyleLinkTypeAttributes:  true
          }
        }
      }
    },

    concat: {
      dist: {},
      core: {
        options: {
          stripBanners : true,
          banner       : '<%= banner %>'
        },
        src: coreFiles,
        dest: '<%= globalConfig.tempDir %>/<%= globalConfig.uic.artifactId %>-core.js'
      }
    },

    /**
     * compiles templates files - js, json, html
     * TRICKY: we can't inherit variables declared at the root level. We have to redeclare the same variables under each task.
     */
    includereplace: {
      'uic_dev': {
        options: {
          globals: globalConfig.uic.env.dev.settings,
          'project.version'    : '<%= globalConfig.version %>'
        },
        src: 'src/data/settings.tpl.json',
        dest: 'src/data/<%= globalConfig.uic.artifactId %>-settings.json'
      },
      core: {
        options: {
          globals: {
            'project.version'    : '<%= globalConfig.version %>'
          }
        },
        src: '<%= globalConfig.tempDir %>/<%= globalConfig.uic.artifactId %>-core.js',
        dest: '<%= globalConfig.tempDir %>/<%= globalConfig.uic.artifactId %>-core.js'
      },
      'uic_prod':{
        options: {
          globals: _.assign({
            'name'               : '<%= globalConfig.name %>',
            'project.version'    : '<%= globalConfig.version %>',
            'project.artifactId' : '<%= globalConfig.uic.artifactId %>',
            'customer.box.folder': '<%= globalConfig.customerBoxFolder %>',
            'dataDir'            : '<%= globalConfig.uic.env.prod.dataDir %>'
          }, globalConfig.uic.env.prod.settings),
          'project.version'    : '<%= globalConfig.version %>'
        },
        files: {
          './': [
            '<%= globalConfig.distDir %>/<%= globalConfig.uic.destDir %>/**/{,*/}*.html',
            '<%= globalConfig.distDir %>/<%= globalConfig.uic.destDir %>/<%= globalConfig.customerBoxFolder %>/js/<%= globalConfig.uic.artifactId %>-launcher.js',
            '<%= globalConfig.distDir %>/<%= globalConfig.uic.destDir %>/<%= globalConfig.customerBoxFolder %>/data/**/{,*/}*.json'
          ]
        }
      },
      'readme' : {
        options : {
          globals : {
            akioSpecificDeliveryBaseName  : '<%= globalConfig.name %>',
            projectVersion                : '<%= globalConfig.version %>',
            timestamp                     : new Date().toLocaleDateString(),
            customerBoxFolder             : '<%= globalConfig.customerBoxFolder %>',
            customerWebsiteFolder         : '<%= globalConfig.customerWebsiteFolder %>'
          }
        },
        src : 'docs/README.txt',
        dest : '<%= globalConfig.tPackageDir %>/README.txt'
      }
    },

    'string-replace':{
      bower: {
        files: [{
          expand: true,
          cwd: '',
          src: 'bower.json',
          dest: 'build/package/source/box/'
        }],
        options: {
          replacements: [{
            pattern: '../aip/akio-aip-webapp/src/main/webapp/js/akioTargeting.js',
            replacement: './akioTargeting/akioTargeting.js'
          }]
        }
      },
      noSource: {
        files: [{
          expand: true,
          cwd: 'build/package/source/box/',
          src: 'Gruntfile.js',
          dest: 'build/package/source/box/'
        }],
        options: {
          replacements: [{
            pattern: /'copy:source',\/\//g,
            replacement: "//'copy:source',"
          }]
        }
      },
      noUserCommonCss: {
        files: [{
          expand: true,
          cwd: 'build/package/source/box/',
          src: 'package.json',
          dest: 'build/package/source/box/'
        }],
        options: {
          replacements: [{
            pattern: /"uic-common-css.*,/,
            replacement: ""
          }]
        }
      },
      changeFontsRessourcePath: {
        files: [{
          expand: true,
          cwd: 'build/package/source/box/',
          src: 'Gruntfile.js',
          dest: 'build/package/source/box/'
        }],
        options: {
          replacements: [{
            pattern: /node_modules\/uic-common-css\/src\/main\/resources\/sass\/svg/,
            replacement: "src"
          }]
        }
      },

      changeCssRessourcePath: {
        files: [{
          expand: true,
          cwd: 'build/package/source/box/src/styles/',
          src: 'akio-webbox.scss',
          dest: 'build/package/source/box/src/styles/'
        }],
        options: {
          replacements: [{
            pattern: /\.\.\/\.\.\/node_modules\/uic-common-css\/src\/main\/resources\/sass\/svg\/_style/,
            replacement: "default/_style"
          }]
        }
      },

      noBower: {
        files: [{
          expand: true,
          cwd: 'build/package/source/box/',
          src: 'Gruntfile.js',
          dest: 'build/package/source/box/'
        }],
        options: {
          replacements: [{
            pattern: /'string-replace:bower',\/\//g,
            replacement: "//'string-replace:bower',"
          }]
        }
      }
    },

    uglify: {
      options: {
        compress         : {},
        report           : 'min',
        preserveComments : false,
        mangle           : true,
        beautify         : false
      },
      core: { /* minifies and obfuscates the js sources */
        options: {
          sourceMap: false,
          banner   : '<%= banner %>'
        },
        expand: true,
        files: [{
          dest: '<%= globalConfig.tPackageDir %>/<%= globalConfig.customerBoxFolder %>/js/<%= globalConfig.uic.artifactId %>-core-<%= globalConfig.version %>.min.js',
          src : ['<%= globalConfig.tempDir %>/<%= globalConfig.uic.artifactId %>-core.js']
        }]
      }
    },

    /**
     * deploys and releases on nexus
     */
    maven: {
      options: {
        version         : '<%= globalConfig.version %>',
        packaging       : 'zip' /* default */
      },
      deploy: {
        options: {
          artifactId      : '<%= globalConfig.uic.artifactId %>',
          groupId         : '<%= globalConfig.uic.groupId %>',
          goal            : 'deploy',
          url             : '<%= globalConfig.snapshotRepository %>',
          repositoryId    : '<%= globalConfig.snapshotRepoId %>',
          injectDestFolder: ''
        },
        files: [{
          expand: true,
          cwd: '<%= globalConfig.distDir %>/<%= globalConfig.uic.destDir %>/',
          src: '**'
        }]
      },
      release: {
        options: {
          artifactId: '<%= globalConfig.uic.artifactId %>',
          groupId   : '<%= globalConfig.uic.groupId %>',
          goal: 'release',
          url: '<%= globalConfig.releaseRepository %>',
          repositoryId: '<%= globalConfig.releaseRepoId %>',
          injectDestFolder: ''
        },
        files: [{
          expand: true,
          cwd: '<%= globalConfig.distDir %>/<%= globalConfig.uic.destDir %>/',
          src: '**'
        }]
      }
    },

    // Test settings
    karma: {
      options: {
        files: [
          './node_modules/intl/locale-data/jsonp/en-US.js',
          './node_modules/intl/locale-data/jsonp/fr-FR.js',
          'src/bower_components/angular/angular.js',
          'src/bower_components/angular-animate/angular-animate.js',
          'src/bower_components/angular-mocks/angular-mocks.js',
          'src/bower_components/angular-resource/angular-resource.js',
          'src/bower_components/angular-sanitize/angular-sanitize.js',
          'src/bower_components/lodash/dist/lodash.js',
          'src/bower_components/angular-lodash/angular-lodash.js',
          'src/bower_components/backbone/backbone.js',
          'src/js/akioTargeting.js',
          'src/bower_components/ng-content-editable/dist/ng-content-editable.min.js',
          'src/bower_components/mobile-detect/mobile-detect.js',
          'src/libs/jquery-1.6.2.js',
          'src/libs/backbone.domStorage.js',
          'src/app/app.js',
          'src/app/AnchorButton/clickToChatFactory.js',
          'src/app/AnchorButton/clickToWcbFactory.js',
          'src/app/common/animates.js',
          'src/app/common/constants.js',
          'src/app/common/emojisConstant.js',
          'src/app/common/determineClient.js',
          'src/app/common/filters.js',
          'src/app/common/languageDetection.js',
          'src/app/common/transformTextToSend.js',
          'src/app/common/xmlUtils.js',
          'src/app/common/fixIeSelect.js',
          'src/app/error/errorHandler.js',
          'src/app/error/networkConnectionLazyEvent.js',
          'src/app/error/serviceInterruptedEvent.js',
          'src/app/error/serviceUnavailableEvent.js',
          'src/app/invitation/*.js',
          'src/app/invitation/*.html',
          'src/app/layout/window/*js',
          'src/app/layout/window/titlebar.tpl.html',
          'src/app/layout/window/window.tpl.html',
          'src/app/layout/window/waitCtrl.js',
          'src/app/layout/window/chatFormCtrl.js',
          'src/app/layout/window/chatFormDrct.js',
          'src/app/layout/**/*.html',
          'src/app/media/*.js',
          'src/app/media/chat/*.js',
          'src/app/media/chat/emoji/*.js',
          'src/app/media/chat/emoji/emojiPanel.tpl.html',
          'src/app/media/wcb/*.js',
          'src/app/media/wcb/wcb.tpl.html',
          'src/app/media/chat/chat.tpl.html',
          'src/app/media/form/**/*.js',
          'src/app/media/form/form.tpl.html',
          'src/app/media/form/fieldset.tpl.html',
          'src/app/messages/AIPConnector.js',
          'src/app/messages/CORSConnector.js',
          'src/app/messages/EventModel.js',
          'src/app/messages/readerSvc.js',
          '.tmp/js/proxy.js',
          'src/app/navigation/*.js',
          'src/app/notification/*.js',
          'src/app/notification/notification.tpl.html',
          'src/app/services/configuration.js',
          'src/app/services/messages.js',
          'src/app/session/session.js',
          'src/app/session/dataContext.js',
          'src/app/time-picker/timePicker.tpl.html',
          'src/app/time-picker/*.js',
          //'test/mock/**/*.js',
          //'test/spec/**/*.js',
          //json Fixture
          { pattern:  'src/data/i18n/*.json', watched:  true, served:   true, included: false }
        ]
      },
      unit: {
        configFile: 'karma.conf.js',
        singleRun: true,
        files: [
          { src: [

            'test/spec/messages/test_AIPConnector_getInitData.js',
            'test/spec/AnchorButton/*.js',
            'test/spec/common/test_determineClient.js',
            'test/spec/common/test_filters.js',
            'test/spec/common/test_fixIeSelect.js',
            'test/spec/common/test_languageDetection.js',
            'test/spec/common/test_transformText.js',
            'test/spec/common/test_xmlUtils.js',
            'test/spec/contextParameters/*.js',
            'test/spec/invitation/*.js',
            'test/spec/navigation/test_navigation.js',
            'test/spec/layout/*.js',
            'test/spec/media/*.js',
            'test/spec/media/chat/emoji/*.js',
            'test/spec/messages/test_AIPConnector_closeMediaChat.js',
            'test/spec/messages/test_AIPConnector_downloadHistory.js',
            'test/spec/messages/test_AIPConnector_fetch.js',
            'test/spec/error/*.js',
            'test/spec/messages/test_AIPConnector_getCalendar.js',
            'test/spec/messages/test_AIPConnector_getMediaChat.js',
            'test/spec/messages/test_AIPConnector_openMediaChat.js',
            'test/spec/messages/test_AIPConnector_openMediaWcb.js',
            'test/spec/messages/test_AIPConnector_ping.js',
            'test/spec/messages/test_readerSvc.js',
            'test/spec/messages/test_eventModel.js',
            'test/spec/messages/test_AIPConnector_notify.js',
            'test/spec/messages/test_AIPConnector_sendInputEvent.js',
            'test/spec/notification/*.js',
            'test/spec/session/*.js',
            'test/spec/messages/test_AIPConnector_sendMessage.js',
            'test/spec/messages/test_AIPConnector_getInitData2.js',
            'test/spec/service/*.js',
            'test/spec/time-picker/*.js'
          ] }
        ]
      },

      loop: {
        configFile: 'karma.conf.js',
        autoWatch: true
      }
    }
  });

  grunt.registerTask('serve', function (target) {
    if (target === 'dist') {
      return grunt.task.run(['build:dev', 'connect:dist:keepalive']);
    }

    grunt.task.run([
      'clean:server',
      'bower-install',
      'concurrent:server',
      'clean:generatedFiles',
      'includereplace:uic_dev',
      'autoprefixer',
      'connect:livereload',
      'processhtml:js',
      'watch'
    ]);
  });

  grunt.registerTask('server', function () {
    grunt.log.warn('The `server` task has been deprecated. Use `grunt serve` to start a server.');
    grunt.task.run(['serve']);
  });

  grunt.registerTask('test', [
    'jshint',
    'clean:server',
    'autoprefixer',
    'processhtml:js',
    'karma:unit'

  ]);

  grunt.registerTask('build:dev', [
    'clean:dist',
    'bower-install',
    'useminPrepare',
    'concurrent:dist',
    'autoprefixer',
    'concat',
    'ngmin',
    'copy:dist',
    'cssmin',
    'uglify',
    'rev',
    'usemin',
    'htmlmin'
  ]);

  /**
   * build and minify js core
   * convert html templates to js, concat all js files but the launcher, then minify it
   */
  grunt.registerTask('build:corejs', [
    'ngtemplates:akio.webbox',
    'concat:core',
    'includereplace:core',
    'uglify:core'
  ]);

  /**
   * build uic
   */
  grunt.registerTask('build:dist', [
    'test',
    'build:corePackage',
    'build:uic'
  ]);

  /**
   * build the package in the build directory. In this version, templates are not compiled.
   */
  grunt.registerTask('build:corePackage', [
    'clean:dist',
    'compass:dist',
    'processhtml:jsDist',
    'build:corejs',
    'copy:assets',
    //'copy:source',automatically commented in build
    'copy:awb_cors',
    'includereplace:readme',
    'string-replace:bower',//automatically commented in build
    'string-replace:noSource',
    'string-replace:noUserCommonCss',
    'string-replace:changeFontsRessourcePath',
    'string-replace:changeCssRessourcePath'

  ]);

  /**
   * build uic dist from the core package
   */
  grunt.registerTask('build:uic', [
    'copy:uic',
    'includereplace:uic_prod'
  ]);


  grunt.registerTask('default', [
    'newer:jshint',
    'test',
    'build:dev'
  ]);
};
