'use strict';

/**
 * the Webbox object holds a reference on
 * - infos that allow debuggers to check the configuration interpreted by the box (media status...)
 */
window.Akio = {Webbox:{}};

angular.module('akio.webbox', [
    'akio.webbox.filters',
    'akio.webbox.animates',
    'ngResource',
    'ngAnimate',
    'ngSanitize',
    'chat',
    'form',
    'content-editable'
  ])

  /**
   * wraps Backbone in a proper AngularJS service
   * the noConflict method assures that our backbone library will not conflict with another version on the website
   */
  .value('Backbone', Backbone.noConflict())

  /**
   * main function used to kickstart the application
   */
  .run(['session', 'configuration', '$log',  'notification','$window', '$rootScope','boxParams','determineClient',
    function (session, configuration, $log, notification, $window, $rootScope, boxParams, determineClient){

      $rootScope.firstDisplay = true;


      /**
       * start session
       * @param result
       * @returns {*}
       */
      var startSession=/* istanbul ignore next */
          function(result){
          $log.info('session.readConfig.success', result);
          return session.start();
        },

        /**
        *use to set error message position. position change on differents devices
        *
        *
        **/
         setWindowPosition = function() {
          if (determineClient.device === 'smartphone') {
            jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
              width: '100%',
              right: '0',
              bottom: '0',
              boxShadow: 'none',
              borderRadius: '0'
            });
          } else if (determineClient.device === 'tablet' || boxParams.IS_DISPLAY_LAUNCHBAR_AS_BUBBLE) {
            jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
              width: '300px',
              right: '0',
              bottom: '0',
              boxShadow: 'none',
              borderRadius: '0'
            });
          } else {
            jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
              right: '20px',
            });
          }
        },

        /**
         * display error as notification
         * @param error
         */
        displayErrorNotification=/* istanbul ignore next */
          function( error ){
          $log.warn('displayErrorNotification: initialisation failed!', error);
          setWindowPosition();

          //if the box is in launchbar mode => notification otherwise the error push
          session.isNetConnectionDown=true;

          notification.add(
            {
              kind: "error",
              details: error.errors || [],
              states: [
                {text: error.text || boxParams.COMMUNICATION_ERROR_MESSAGE}
              ]
            },
            _.noop
          );
        };

      // set template format
      _.templateSettings.interpolate = /{{([\s\S]+?)}}/g;

      /**
       * fires when the widget interface has loaded successfully.
       * The API connector whether we are in CORS or CWM is ready to be requested.
       */
      $rootScope.$on('widgetUI:loaded', function( event, data ){
        // interprets configuration file
        configuration
          .read( data )
          .then( startSession )
          ["catch"]( displayErrorNotification )
          ["finally"]( function( ){
          delete $window.Akio.Webbox.settings;
        } );
      });

      /**
       * fires when the widget failed to load. NEVER CALLED AT THIS STAGE.
       */
      $rootScope.$on('widgetUI:error', function( event, error ) {
        displayErrorNotification( error );
      });

    }
  ]);
