
'use strict';

// targets a ngIf directive
angular.module('akio.webbox.animates', [])

  .animation('.awb-fade-animation', ['$rootScope', function( $rootScope ) {
    var duration = 400;
    return {
      enter : function(element, done) {
        element = jQuery(element);
        element.css({display:'none'}).fadeIn(duration, 'swing', done);
        // Timeout after 1ms because sometimes the height is wrong at 0 ms
        setTimeout(function() {
          $rootScope.$emit('webbox:animate');
        }, 1);
      },

      leave : function(element, done) {
        element = jQuery(element);
        element.fadeOut(duration, 'swing', done);
        setTimeout(function() {
          // Force the iframe to 0px when the box is hiding
          if(element.hasClass('awb-card-wrapper')) {
            $rootScope.$emit('webbox:animate', 0);
          } else {
            $rootScope.$emit('webbox:animate');
          }
        }, duration);
      }
    };
  }])

  // targets a ngSwitch directive
  .animation('.awb-pane-animation', ['$rootScope',
    function( $rootScope ) {
      var duration = 400;
      return {
        enter : function(element, done) {
          element = jQuery(element);
          element.css({display: 'none'});
          setTimeout(function () {
            $rootScope.$emit('webbox:animate', element.height());
            element.css({bottom: 0}).slideDown(duration, 'swing', done);
          }, duration);
        },

        leave : function(element, done) {
          element = jQuery(element);
          element.slideUp(duration, 'swing', done);
        }
      };
    }
  ])

  // targets a ngSwitch directive
    .animation('.awb-bubble-animation', ['$rootScope',
      function( $rootScope ) {
        var duration = 400;
        return {
          enter : function(element, done) {
            element = jQuery(element);
            element.css({display: 'none'});
            setTimeout(function () {
              $rootScope.$emit('webbox:animate', element.height());
              jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
                right: '20px',
                bottom: '20px',
              });
              element.fadeIn(duration, 'swing', done);
            }, duration);
          },

          leave : function(element, done) {
            jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
              boxShadow: 'none',
            });
            element = jQuery(element);
            element.fadeOut(duration, 'swing', done);
          }
        };
      }
    ])

  /**
   * transition animations attached to the TRIGGER element
   * targets a ngIf directive
   */
  .animation('.awb-trigger-animation', ['$rootScope',
    function( $rootScope ) {
      var duration = 300;
      return {
        enter : function(element, done) {
          element = jQuery(element);
          element
            .css({top:element.parent().height()})
            .animate({top:0}, duration, done);
          $rootScope.$emit('webbox:animate');
        },

        leave : function(element, done) {
          element = jQuery(element);
          element.animate({top:element.parent().height()}, duration, done);
          $rootScope.$emit('webbox:animate');
        }
      };
    }
  ])

  // targets a ngSwitch directive
  .animation('.awb-card-animation', ['$rootScope',
    function( $rootScope ) {
      return {
        enter : function(element, done) {
          element = jQuery(element);
          element.css({
            position:'relative', //@@@ allow card to resize itself
            top: 0,
            left:element.parent().width()-10
          });
          setTimeout(function() {
            element.animate({left:0}, done);
            $rootScope.$emit('webbox:animate');
          }, 400);
        },

        leave : function(element, done) {
          element = jQuery(element);
          element.css({
            position:'absolute',
            top:0,
            left:0
          });
          element.animate({left:-element.parent().width()}, done);
          $rootScope.$emit('webbox:animate');
        }
      };
    }
  ])

  // targets a ngSwitch directive
  .animation('.awb-fly-panel-animation', ['$rootScope',
    function( $rootScope ) {
      var duration = 300;
      return {
        enter : function(element, done) {
          element = jQuery(element);
          element.css({
            position:'absolute',
            left:0
          });
          element.animate({left:-element.parent().width(), duration:duration}, done);
          $rootScope.$emit('webbox:animate');
        },

        leave : function(element, done) {
          element = jQuery(element);
          element.css({
            position:'absolute',
            left:-element.parent().width()
          });
          element.animate({left:0, duration:duration}, done);
          $rootScope.$emit('webbox:animate');
        }
      };
    }
  ])

  // targets a ngRepeat directive
  .animation('.awb-date-animation', ['$rootScope',
    function( $rootScope ) {
      var animationDuration = 200;

      return {
        enter : function(element, done) {
          element.css({
            display: 'none',
            left: element.parent().hasClass('swipe-right') ? -element.parent().width() : element.parent().width()
          });
          setTimeout(function() {
            element.css({
              display: 'inline-block'
            });
            element.animate({left: 0}, animationDuration, done);
            $rootScope.$emit('webbox:animate');
          }, animationDuration);
        },

        leave : function(element, done) {
          element.animate({left: element.parent().hasClass('swipe-right') ? element.parent().width() : -element.parent().width()}, animationDuration, done);
          $rootScope.$emit('webbox: animate');
        }
      };
    }
  ]);
