'use strict';

/**
 * capitalizes a string
 */
angular.module('akio.webbox.filters', [])
  .filter('capitalize', function() {
    return function(string) {
      if (!angular.isDefined(string)){return string;}
      return string.charAt(0).toUpperCase() + string.substring(1).toLowerCase();
    };
  })

  .filter('unsafe', ['$sce', function($sce) {
    return function (html) {
      return $sce.trustAsHtml(html);
    };
  }])

  .filter('reverse', function() {
    return function(items) {
      if (!angular.isArray(items)){return false;}
      return items.slice().reverse();
    };
  })

  .filter('nospace', function () {
    return function (value) {
      return (!value) ? '' : value.replace(/ /g, '');
    };
  })

/**
 * basic filter that do not handle internationalization
 */
  .filter('duration', function ( ) {
    return function (text, duration) {
      var format = function( value ){
        var sec = value%60,
          min   = ( value - sec ) / 60,
          result= [];

        if ( min > 0 ){
          result.push( min + 'min' );
        }

        if (sec > 0){
          result.push( sec + 'sec' );
        }

        return result.join( '&nbsp;' );
      };

      return _.template( text, { duration: format( duration ) } );
    };
  })

/**
 * formats an object into a name/value pairs
 */
  .filter('nameValuePairs', function () {
    return function (value) {
      return _.pluck(value || {}, function(value, key){return {name:key, value:value};});
    };
  })

  .filter('to_trusted', ['$sce', function($sce){
    return function(text) {
      return $sce.trustAsHtml(text);
    };
  }])

  .filter('noHTML', function () {
    return function(text) {
      return text
        .replace(/&/g, '&amp;')
        .replace(/>/g, '&gt;')
        .replace(/</g, '&lt;');
    };
  })

  .filter('transformLink', function() {
    return function(text) {
      var urlReg = /(((https?:\/\/)|(w{2}))[\w-]+((\.|:)[\w-]+)+\.?(:\d+)?(\/\S*)?(?=((<\/)|(<br))))|(((https?:\/\/)|(w{2}))[\w-]+((\.|:)[\w-]+)+\.?(:\d+)?(\/\S*)?)/gi;
      var urlArray = text.match(urlReg);
      if (urlArray) {
        for (var i= 0;i<urlArray.length;i++){
          var currentUrl =urlArray[i];
          if (currentUrl.match(/http/)){
            text = text.replace(urlArray[i],'<a name="awbChatLink" href="'+currentUrl+'" target="_blank">'+currentUrl+'</a>');
          }else{
            text = text.replace(urlArray[i],'<a name="awbChatLink" href="http://'+currentUrl+'" target="_blank">'+currentUrl+'</a>');
          }
        }
      }
      return text;
    };


  })

  .filter('transformLineBreak', function() {
    return function(text) {
      var lineBreakReg = /(&lt;br\/&gt;)/gi;
      var lineBreakArray = text.match(lineBreakReg);
      if (lineBreakArray) {
        for (var i= 0;i<lineBreakArray.length;i++){

          text = text.replace(lineBreakArray[i],'<br/>');

        }
      }
      return text;
    };
  });
