'use strict';

/**
 * this function detect language to use in app.
 * the param i18n is a string "webBrowser" or the javascript command line to get the value of the website's language module selection
 *
 * if the i18nParameter equal "webBrowser" we detect the language from the web browser configuration.
 * Else the language is detected on the website's language module selection using the js command line
 *
 * @param i18nParameter
 * @returns {string} language
 */
/* exported languageDetection */
function languageDetection(i18nParameter){

  /**
   * function to detect language from web browser
   * @returns {*}
   */
  var languageDetectionByBrowser = function(){
    var language;
    if (navigator.browserLanguage) {
      language = navigator.browserLanguage;
    }else{
      language = navigator.language;
    }
    return language;
  };

  /**
   * function to detect language from the website's language module selection
   * @param jsCommandLine
   * @returns {Object}
   */
  var languageDetectionByWebSite = function(jsCommandLine){
    var language;

    try{
      language=eval(jsCommandLine);
    }
    catch(e){
      console.warn("error in I18N parameter : "+e);
      language = languageDetectionByBrowser();
    }

    return  language;

  };

  return i18nParameter==="webBrowser" ? languageDetectionByBrowser() : languageDetectionByWebSite(i18nParameter);
}
