'use strict';

angular.module('akio.webbox')

  .factory('xmlUtils', function(){

    function xmlParse(string, expectedRoot /* optional */) {
      var doc = null;
      if ( window.DOMParser !== undefined ) {
        try {
          doc = new DOMParser().parseFromString(string, "text/xml");
        } catch ( ignore ) {
          // IE 9 and IE 10 throw when there is a parsing error
        }
      } else {
        try {
          doc = new ActiveXObject("Microsoft.XMLDOM");
          doc.async = "false";
          doc.loadXML(string);
        } catch ( e ) {
          console.log(e);
        }
      }
      if ( doc ) {
        var root = doc.documentElement;
        if ( root && (root.nodeName === expectedRoot || root.nodeName !== "parsererror") ) {
          return root;
        }
      }
      return null;
    }

    function xmlSerialize(node) {
      if ( window.XMLSerializer !== undefined ) {
        return new XMLSerializer().serializeToString(node);
      } else {
        try {
          return node.xml;
        } catch ( e ) {
          console.log(e);
          return null;
        }
      }
    }

// ==============
// XML PROCESSING
// ==============

    /*var ReplaceXMLEntities = (function() {
      var replacements = [["&", "38"], ['"', "34"], ["'", "39"], ["<", "60"], [">", "62"]],
        n = replacements.length,
        i, pair;
      for ( i = 0; i < n; i++ ) {
        pair = replacements[i];
        replacements[i] = [new RegExp(pair[0], "g"), "&#"+pair[1]+";"];
      }

      return function(text) {
        var result = text,
          i, pair;
        for ( i = 0; i < n; ++i ) {
          pair = replacements[i];
          result = result.replace(pair[0], pair[1]);
        }
        return result;
      };
    }());*/

    var surrounding = "node";

    function surroundAndXMLParse (str) {
      str = escapeHtml(str); // used to converts the specials characters in string to their corresponding HTML entities.
      return xmlParse("<"+surrounding+">"+str.replace(/<br>/gi, '<br/>')+"</"+surrounding+">", surrounding);
    }

    function hasClass (node, clazz) {
      var haystack = " " + node.getAttribute("class") + " ",
        needle = " " + clazz + " ";
      return haystack.indexOf(needle) >= 0;
    }

    /**
     * This function is used to replace special character by html equivalent
     * @param unsafe
     * @returns {*}
       */
    function escapeHtml(unsafe) {
      return unsafe
        .replace(/&/g, "&amp;")
        .replace(/</g, "&lt;")
       // .replace(/>/g, "&gt;")
       // .replace(/¢/g, "&cent;")
       // .replace(/£/g, "&pound;")
       // .replace(/€/g, "&euro;")
       // .replace(/¥/g, "&yen;")
       // .replace(/°/g, "&deg;")
       // .replace(/¼/g, "&frac14;")
       // .replace(/Œ/g, "&OElig;")
       // .replace(/½/g, "&frac12;")
       // .replace(/œ/g, "&oelig;")
       // .replace(/¾/g, "&frac34;")
       // .replace(/Ÿ/g, "&Yuml;")
       // .replace(/¡/g, "&iexcl;")
       // .replace(/«/g, "&laquo;")
       // .replace(/»/g, "&raquo;")
       // .replace(/¿/g, "&iquest;")
       // .replace(/À/g, "&Agrave;")
       // .replace(/Á/g, "&Aacute;")
       // .replace(/Â/g, "&Acirc;")
       // .replace(/Ã/g, "&Atilde;")
       // .replace(/Ä/g, "&Auml;")
       // .replace(/Å/g, "&Aring;")
       // .replace(/Æ/g, "&AElig;")
       // .replace(/Ç/g, "&Ccedil;")
       // .replace(/È/g, "&Egrave;")
       // .replace(/É/g, "&Eacute;")
       // .replace(/Ê/g, "&Ecirc;")
       // .replace(/Ë/g, "&Euml;")
       // .replace(/Ì/g, "&Igrave;")
       // .replace(/Í/g, "&Iacute;")
       // .replace(/Î/g, "&Icirc;")
       // .replace(/Ï/g, "&Iuml;")
       // .replace(/Ð/g, "&ETH;")
       // .replace(/Ñ/g, "&Ntilde;")
       // .replace(/Ò/g, "&Ograve;")
       // .replace(/Ó/g, "&Oacute;")
       // .replace(/Ô/g, "&Ocirc;")
       // .replace(/Õ/g, "&Otilde;")
       // .replace(/Ö/g, "&Ouml;")
       // .replace(/Ø/g, "&Oslash;")
       // .replace(/Ù/g, "&Ugrave;")
       // .replace(/Ú/g, "&Uacute;")
       // .replace(/Û/g, "&Ucirc;")
       // .replace(/Ü/g, "&Uuml;")
       // .replace(/Ý/g, "&Yacute;")
       // .replace(/Þ/g, "&THORN;")
       // .replace(/ß/g, "&szlig;")
       // .replace(/à/g, "&agrave;")
       // .replace(/á/g, "&aacute;")
       // .replace(/â/g, "&acirc;")
       // .replace(/ã/g, "&atilde;")
       // .replace(/ä/g, "&auml;")
       // .replace(/å/g, "&aring;")
       // .replace(/æ/g, "&aelig;")
       // .replace(/ç/g, "&ccedil;")
       // .replace(/è/g, "&egrave;")
       // .replace(/é/g, "&eacute;")
       // .replace(/ê/g, "&ecirc;")
       // .replace(/ë/g, "&euml;")
       // .replace(/ì/g, "&igrave;")
       // .replace(/í/g, "&iacute;")
       // .replace(/î/g, "&icirc;")
       // .replace(/ï/g, "&iuml;")
       // .replace(/ð/g, "&eth;")
       // .replace(/ñ/g, "&ntilde;")
       // .replace(/ò/g, "&ograve;")
       // .replace(/ó/g, "&oacute;")
       // .replace(/ô/g, "&ocirc;")
       // .replace(/õ/g, "&otilde;")
       // .replace(/ö/g, "&ouml;")
       // .replace(/ø/g, "&oslash;")
       // .replace(/ù/g, "&ugrave;")
       // .replace(/ú/g, "&uacute;")
       // .replace(/û/g, "&ucirc;")
       // .replace(/ü/g, "&uuml;")
       // .replace(/ý/g, "&yacute;")
       // .replace(/þ/g, "&thorn;")
       // .replace(/"/g, "&quot;")
        .replace(/'/g, "&#039;");
    }

    return {
      surroundAndXMLParse: surroundAndXMLParse,
      hasClass: hasClass,
      xmlSerializeAndUnSurround: (function(){
        var regex = new RegExp("^.*<"+surrounding+">(.*)</"+surrounding+">$"); // Opera adds "<?xml ... ?>"

        return function(node)
        {
          return xmlSerialize(node).replace(regex, "$1");
        };
      }())
    };
  });
