'use strict';

/**
 * TRIGGER element's directive definition
 */
angular.module('akio.webbox')
  .directive('awbTrigger', ['$timeout', 'i18nMessages', 'trigger',
    function( $timeout, labels, trigger ){

      function controller( $scope ){
        $scope.trigger = trigger.current;

        // resolves the css class attached to a given event type
        $scope.form = $scope.system = $scope.cob = false;

        $scope.formData = {};
        $scope.fields = [];
        $scope.message  = '';
        $scope.acceptBtnLabel = labels.INVITATION_DEFAULT_ACCEPT_BTN_LABEL;
        $scope.rejectBtnLabel = labels.INVITATION_DEFAULT_REJECT_BTN_LABEL;

        switch ( $scope.trigger.description().eventType ){
          case 'SurveyEvent':
            $scope.message = $scope.trigger.description().eventData.title;
            $scope.form = true;
            $scope.fields = $scope.trigger.description().eventData.fields;
            $scope.acceptBtnLabel = labels.INVITATION_SURVEY_ACCEPT_BTN_LABEL;
            $scope.rejectBtnLabel = labels.INVITATION_SURVEY_REJECT_BTN_LABEL;
            break;
          case 'ChatInvitationEvent':
            $scope.message = labels.INVITATION_CHAT;
            $scope.form = true;
            break;
          case 'FormRequestEvent':
            $scope.message = labels.INVITATION_FORM_MSG;
            $scope.form = true;
            break;

          case 'CobrowsingRequestEventDTO':
            $scope.cob = true;
            break;

          case 'NetworkConnectionLazyEvent':
            $scope.message = labels.ERROR_NETWORK_CONNECTION_LAZY;
            $scope.system  = true;
            $scope.acceptBtnLabel = labels.INVITATION_ERROR_NETWORK_CONNECTION_LAZY_ACCEPT_BTN_LABEL;
            $scope.rejectBtnLabel = labels.INVITATION_ERROR_NETWORK_CONNECTION_LAZY_REJECT_BTN_LABEL;
            break;

          case 'ServiceUnavailableEvent':
            $scope.message = labels.ERROR_SERVICE_UNAVAILABLE;
            $scope.system = true;
            $scope.acceptBtnLabel = labels.INVITATION_ERROR_SERVICE_UNAVAILABLE_ACCEPT_BTN_LABEL;
            $scope.rejectBtnLabel = labels.INVITATION_ERROR_SERVICE_UNAVAILABLE_REJECT_BTN_LABEL;
            break;

          default:
            $scope.message = _.template( "this event type «{{eventType}}» is not implemented yet!", {eventType: trigger.current.description.eventType});
            $scope.system = true;
        }

        $scope.hasEmbebbedForm = function( ){
          return $scope.fields.length>0;
        };

        $scope.getCssClassByType = function(){
          if ($scope.form){
            return 'awb-trigger-form';
          }else if ($scope.cob){
            return 'awb-trigger-cob';
          }else if ($scope.system){
            return 'awb-trigger-system';
          }
        };

        $scope.accept = function( event ){
          event.preventDefault( );
          $scope.trigger.accept( $scope.formData );
        };

        $scope.reject = function( event ){
          event.preventDefault( );
          $scope.trigger.reject( );
        };
      }

      return {
        restrict: 'EA',
        replace: true,
        templateUrl: 'app/invitation/trigger.tpl.html',
        controller: ['$scope', 'trigger', controller],
        link: function link (scope, iElement){
          // adjust sub-elements position & size
          var setFooter=function(){
            var element   = jQuery(iElement),
              $footer     = element.find('.awb-trigger-footer'),
              footerWidth = $footer.find('ul').outerWidth();

            // resize buttons to span all the container width
            if (footerWidth>0){
              $footer.find('li').width(footerWidth*0.5);
              $footer.css('visibility', 'visible');
            }else{
              $timeout(setFooter, 100);
            }
          };

          $timeout(function(){
            var element   = jQuery(iElement),
              footerHeight= element.find('.awb-trigger-footer').outerHeight(),
              parentHeight= element.parent().height();

            element
              .outerWidth( element.parent().width() )
              .find('.awb-trigger-content-wrapper')
              .outerHeight( parentHeight-footerHeight );

            setFooter();
          },0);
        }
      };
    }
  ]);
