'use strict';

/**
 * Trigger Class
 * Does the trigger has an expiration date?
 * Do we handle a queue?
 */

angular.module('akio.webbox')
  .factory('trigger', ['$rootScope', 'state',
    function( $rootScope, state ){

      var service = {
        current: undefined,

        /**
         *
         * @returns {boolean}
         */
        exists: function( ){
          return this.current !== undefined;
        },

        /**
         * register the trigger
         * @param event event object
         * @param eventIndex event index
         * @event accept
         * @event reject
         * @usage trigger.add(triggerEvent);
         * @returns {undefined|*}
           */
        register:function( event, eventIndex ){

          var currentState = state.get(),
            /**
             * un-registers the current trigger
             */
            unregister = function( ){
              service.current = undefined;
              currentState.unset( 'trigger' ).save( );
            };

          // save trigger reference to restore it after a refresh
          // if the value is not valid, the trigger will not be saved and so able to be restored after a refresh
          if (eventIndex > -1){
            currentState.save( 'trigger', eventIndex );
          }

          service.current= {
            /**
             *
             * @returns {{eventType: *, eventId: *, eventData: *}}
             */
            description: function( ){
              return {
                eventType: event.get( 'type' ),        /* event type. REQUIRED TO FIT THE RIGHT CONTENT */
                eventId  : event.get( 'eventId' ),      /* event unique identifier */
                eventData: event.get( 'formData' )
              };
            },

            /**
             *
             * @param formData
             */
            accept: function( formData ){
              unregister( );
              $rootScope.$emit( 'trigger:accept', event, eventIndex, formData );
            },

            /**
             *
             */
            reject: function( ){
              unregister( );
              $rootScope.$emit( 'trigger:reject', event );
            }
          };

          return service.current;
        }

      };

      return service;
    }
  ]);
