'use strict';

/**
 * «awbImgBroken» Directive
 *
 * it hides the image or shows a fallback image if the image fails to load
 *
 * @usage
 * hides the image if the image fails to load:
 * <img ng-src="{{image}}" awb-img-broken />
 *
 * shows the fallback image if the image fails to load:
 * <img ng-src="{{avatar.img}}" awb-img-broken="path/to/fallback/image" />
 */
angular.module('akio.webbox')
  .directive('awbImgBroken', function() {

    var link = function( scope, iElement, attrs ){
      var element     = jQuery( iElement),
        fallbackSrc = attrs.awbImgBroken;

      element
        .error(function(){
          var hideIfBroken= _.isUndefined( fallbackSrc ) || _.isEmpty( fallbackSrc ) || attrs.src === fallbackSrc;
          if ( hideIfBroken ){
            jQuery( this ).hide( );
          }else{
            element.attr('src', fallbackSrc);
          }
        })
        .attr( "src",  attrs.src ); // forces the image to reload to be certain to catch the error if fired

      scope.$on('$destroy', function(){
        element.off( );
      });
    };

    return {
      restrict: 'A',
      replace : false,
      link: link
    };
  });
