'use strict';

/**
 * launchbar controller
 * any changes in one of the medium's attributes will be taken into account and the view automatically updated (media[0].accessible = false)
 */
angular.module('akio.webbox')
  .controller('launchBarCtrl', ['$scope', '$rootScope', 'navigation', 'aip', 'state', '$q', '$injector', 'boxParams', 'determineClient',
    function ($scope, $rootScope, navigation, aip, state, $q, $injector, boxParams, determineClient) {

      var setLaunchBarPosition = function () {
        if (determineClient.device !== 'smartphone' && determineClient.device !== 'tablet' && !boxParams.IS_DISPLAY_LAUNCHBAR_AS_BUBBLE) {
          jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
            right: '20px',
            width: '300px',
            bottom: '0'
          });
        } else if (boxParams.IS_DISPLAY_LAUNCHBAR_AS_BUBBLE) {
          jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
            'right': '20px',
            'width': '60px',
            'bottom': '20px',
            'border-radius': '50%'
          });
        } else {
          jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
            'right': '20px',
            'width': '60px',
            'bottom': '20px',
            'box-shadow': '0 0 4px rgba(0,0,0,.14), 0 4px 8px rgba(0,0,0,.28)',
            'border-radius': '50%'
          });
        }
      };

      if ($rootScope.firstDisplay) {
        setLaunchBarPosition();
        $rootScope.firstDisplay = false;
      } else {
        setTimeout(setLaunchBarPosition, 400);
      }

      /**
       * media list send by the server
       * @type {mediaList|service.mediaList}
       */
      $scope.media = state.mediaList;


      /**
       * display a medium
       * @param {Object} medium the media description object
       */
      $scope.displayMedium = function (medium) {
        state.events().reset();
        if (medium.get('type') === 'VOICE') {
          aip.getMedia()
            .then(function () {
              navigation.goto(medium);
            });
        } else if (!state.openMediaChatRequestSent) {
          // we lock open media chat request to not send more than 1 request
          state.openMediaChatRequestSent = true;
          //after the polling delay,in case of we unlock the open media chat request
          setTimeout(function () {
            state.openMediaChatRequestSent = false;
          }, boxParams.POLLING_DELAY_EVENT);

          //display form before starting chat
          var askQuestion = boxParams.ASK_QUESTION_BEFORE_CHAT;
          if (askQuestion && _.find(boxParams.CHAT_FORM, function (o) {
            return o.type === 'button';
          })) {
            aip.getMedia()
              .then(function () {
                navigation.goto(medium);
              });
          } else {
            state.setCurrentMediaType(state.mediaList.findWhere({'type': 'CHAT'}).get('type'));
            state.setCurrentMediaCondition(state.mediaList.findWhere({'type': 'CHAT'}).get('conditionUuid'));
            aip.openMediaChat()
              .then(function () {
                // request has been accepted
                navigation.goto(medium);
              });
          }
        }
        state.data.save({ui: {collapse: false}});
      };

      /**
       * search a question
       * @param text
       */
      $scope.search = function (text) {
        $scope.textSearch = ''; // reset input field
        navigation.search(text); // not handled anymore at this stage of the BOX
        return $q.when('true');
      };

    }
  ]);
