'use strict';

/**
 * @usage
 * <awb-launchbar></awb-launchbar>
 */
angular.module('akio.webbox')
  .directive('awbLaunchbar', ['MediumStatus', 'i18nMessages', 'boxParams', 'state', 'determineClient',
    function (MediumStatus, labels, boxParams, state, determineClient) {

      function link(scope) {
        scope.labels = labels;
        scope.textSearch = "";

        scope.doDisplayMedium = function (medium) {
          scope.displayMedium({medium: medium});
        };

        scope.prioritySort = function (medium) {
          return -1 * medium.get('priority');
        };

        scope.statusSort = function (medium) {
          switch (medium.get('status')) {
            case MediumStatus.MEDIUM_STATUS_OPEN_AVAILABLE:
            case MediumStatus.MEDIUM_STATUS_DELAYED_WEBCALLBACK:
              return 1;
            case MediumStatus.MEDIUM_STATUS_OPEN_NOT_AVAILABLE:
              return 0;
            case MediumStatus.MEDIUM_STATUS_CLOSED:
              return -1;
          }
        };
        scope.displayLaunchbarAsBubble = boxParams.IS_DISPLAY_LAUNCHBAR_AS_BUBBLE;

        scope.HelpBubbleVisible = function () {
          return boxParams.HELP_BUBBLE_VISIBLE;
        };

        scope.addLaunchBarClass = function () {
          if (boxParams.IS_DISPLAY_LAUNCHBAR_AS_BUBBLE) {
            return 'awb-launchbar-wrapper-bubble';
          } else {
            return 'awb-launchbar-wrapper';
          }
        };

        scope.NotDisplayAsButton = function () {
          return !state.getDisplayAsButton();
        };

        scope.isMultipleMediaAvailable = function () {
          return scope.mediaList.length > 1;
        };

        scope.isDisplayAsBubble = function () {
          return (determineClient.device === 'tablet' || determineClient.device === 'smartphone' || scope.displayLaunchbarAsBubble);
        };

        scope.isGenericNlSearchFormInBubble = function () {
          return boxParams.RULES.NL_AGGREGATION_ENABLED &&
            scope.mediaList.hasMultipleNlMediaAvailable() &&
            !scope.mediaList.isPriorityMediumNl();
        };

        scope.isGenericNlSearchForm = function () {
          return boxParams.RULES.NL_AGGREGATION_ENABLED &&
            scope.mediaList.hasMultipleNlMediaAvailable() &&
            scope.mediaList.isPriorityMediumNl();
        };

        scope.medium = function () {
          return state.mediaList.find(function(media) {
            return media.get('status') === MediumStatus.MEDIUM_STATUS_OPEN_AVAILABLE ||
              media.get('status') === MediumStatus.MEDIUM_STATUS_DELAYED_WEBCALLBACK;
          });
        };

        scope.$on('$destroy', function () {
          jQuery(document).off();
        });
      }

      return {
        restrict: 'EA',
        link: link,
        replace: true,
        templateUrl: 'app/layout/launchbar/launchBar.tpl.html',
        scope: {
          mediaList: '=',
          displayMedium: '&',
          hideHelpBubble: '&',
          helpBubbleVisible: '@',
          search: '&'
        }
      };
    }]);
