'use strict';

angular.module('akio.webbox')

  .controller('chatFormCtrl', ['$scope', '$filter', '$window', 'navigation', 'aip', 'boxParams', 'i18nMessages', 'notification', 'state','session','MediumViewStatus',
  function($scope, $filter, $window, navigation, aip, boxParams, labels, notification, state,session,MediumViewStatus) {

    state.setTitle($scope.labels.WAITBOX_TITLE);
    state.setCollapseIcon(false);

    $scope.profileParameters = $filter( 'nameValuePairs' )( $window.parent.profileParameters );


    /** function used to find label
    *
    * @input : form array define in webbox settings file
    * @output : form array with label define
    **/

    function addLabelToForm(formArray) {
      var newFormArray=[];
      for (var i = 0; i < formArray.length; i++) {
        newFormArray.push({});
        newFormArray[i].type = formArray[i].type;
        newFormArray[i].textKey =eval("labels." + formArray[i].textKey);
        newFormArray[i].mapping = formArray[i].mapping;
      }
      newFormArray = checkFormWithProfileParameters(newFormArray);
      return newFormArray;
    }

    function checkFormWithProfileParameters(formArray){
      for(var i=0; i<formArray.length;i++){
        if(formArray[i].mapping!==undefined){
          for(var j=0; j<$scope.profileParameters.length;j++){
            formArray[i].value="";
            if(formArray[i].mapping===$scope.profileParameters[j].name){
              formArray[i].value = $scope.profileParameters[j].value;
              break;
            }
          }
        }
      }
      return formArray;
    }

    $scope.emailTester = function(email){
      var regex = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
      return regex.test(email);
    };

    /** function used to find form element text
    *
    * @input : string element type ('button' or 'title')
    * @output : string label
    **/
    function searchFormElementText(elementType) {
      for (var i = 0; i < $scope.chatForm.length; i++) {
        if ($scope.chatForm[i].type === elementType) {
          return $scope.chatForm[i].textKey;
        }
      }
    }

    $scope.checkFormValue=function(){
      for(var i=0;i<$scope.chatForm.length;i++){
        if($scope.chatForm[i].mapping==="frm_email" || $scope.chatForm[i].mapping==="frm_adresse_email_1"){
          return $scope.emailTester($scope.chatForm[i].value);
        }else if($scope.chatForm[i].type ==='field'){
          return ($scope.chatForm[i].value && $scope.chatForm[i].value.length>0);
        }
      }
      return true;
    };

    $scope.checkValidField=function(fieldValue, fieldMapping){
      if(fieldValue && (fieldMapping==="frm_email" ||fieldMapping==="frm_adresse_email_1" )){
        return !(fieldValue.length > 0 && !$scope.emailTester(fieldValue));
      }else{
        return true;
      }
    };

    $scope.chatForm = addLabelToForm(boxParams.CHAT_FORM);

    // define labels for UI
    $scope.order = function(){
      if(!session.isMediaChatOpenAndAvailable() && !session.isMediaWcbOpenAndAvailable()){
        return labels.NOTIFICATION_OPEN_CHAT_RESULT_NO_AGENT;
      }else{
        return searchFormElementText("title");
      }
    };
    $scope.chatButton = searchFormElementText("button");
    $scope.wcbButton = searchFormElementText("buttonWCB");
    $scope.boxIsCollapse = function () {
      return state.data.get('ui').collapse;
    };

    $scope.canStartChat=function(){
      return $scope.checkFormValue() && session.isMediaChatOpenAndAvailable();
    };

    $scope.canStartWcb=function(){
      return $scope.checkFormValue() && session.isMediaWcbOpenAndAvailable();
    };

    /** function used to start conversation
    *
    **/
    $scope.startChat = function() {
      updateProfileParameters();
      state.setMediaStatus(state.mediaList.findWhere({'type': 'CHAT'}).get('uuid'), MediumViewStatus.MEDIUM_STATUS_PENDING);
      if(session.isMediaChatOpenAndAvailable() &&$scope.checkFormValue()){
        state.setCollapseIcon(true);
        state.setCurrentMediaType(state.mediaList.findWhere({'type': 'CHAT'}).get('type'));
        state.setCurrentMediaCondition(state.mediaList.findWhere({'type': 'CHAT'}).get('conditionUuid'));
        navigation.goto(state.mediaList.findWhere({'type': 'CHAT'}));

        //jQuery('#akioWebBoxIFrame', window.parent.document).parent().height(0);
        aip.openMediaChat()
          .then(function() {
            //state.setIsFirstChat(false);
            // request has been accepted
        });
      }

    };

    $scope.startWcb = function() {
      updateProfileParameters();
      if(session.isMediaWcbOpenAndAvailable() && $scope.checkFormValue()){
        state.setCollapseIcon(true);
        //jQuery('#akioWebBoxIFrame', window.parent.document).parent().height(0);
        navigation.goto(state.mediaList.findWhere({'type': 'VOICE'}));

        aip.getMedia().then(function(){
          state.setCurrentMediaType('VOICE');
        });
        // request has been accepted

      }
    };

    $scope.getPlaceHolder = function(mapping){
      if(mapping === "frm_email"||mapping ==="frm_adresse_email_1"){
        return "mon.email@example.com";
      }else{
        return "";
      }
    };

    function updateProfileParameters(){
      if($scope.chatForm){
        for(var i=0; i<$scope.chatForm.length;i++){
          if($window.parent.profileParameters){
            $window.parent.profileParameters[$scope.chatForm[i].mapping]=$scope.chatForm[i].value;
          }
        }
      }
    }

  }]);
