'use strict';

/**
 * Title bar
 */
angular.module('akio.webbox')
  .directive('awbTitlebar', ['$log', 'aip', 'state', 'navigation', 'trigger','session' ,'MediumViewStatus', 'boxParams', '$rootScope','i18nMessages',
    function( $log, aip, state, navigation, trigger,session, MediumViewStatus, boxParams, $rootScope,labels ) {

      function controller ($scope) {
        var isTerminalPanel = false;
        $scope.closeButtonLabel =function(){
          var mediumUuid = state.getCenterRegion( ['mediumUuid'] );
          if(mediumUuid==='8') {
            return labels.WCB_CLOSE_FORM;
          }else{
            return labels.CHATBOX_STOP_CONVERSATION;
          }
        };

        $scope.contractButtonLabel =function(){
          var mediumUuid = state.getCenterRegion( ['mediumUuid'] );
          if(mediumUuid==='8') {
            return labels.BOX_BUTTON_CONTRACT_WCB;
          }else{
            return labels.BOX_BUTTON_CONTRACT;
          }
        };

        $scope.expandButtonLabel =function(){
          var mediumUuid = state.getCenterRegion( ['mediumUuid'] );
          if(mediumUuid==='8') {
            return labels.BOX_BUTTON_EXPAND_WCB;
          }else{
            return labels.BOX_BUTTON_EXPAND;
          }
        };


        $scope.logout = function(){
          var mediumUuid = state.getCenterRegion( ['mediumUuid'] );
          if(mediumUuid!=='8' && !session.fetchMediaIsRunning) {
            if (!state.closeMediaChatRequestSent) {
              // we block close media chat request for time equal to poling delay
              state.closeMediaChatRequestSent = true;
              setTimeout(function () {
                state.closeMediaChatRequestSent = false;
              }, boxParams.POLLING_DELAY_EVENT);
              aip
                .closeMediaChat()
                .then(function () {
                  // request has been accepted
                  var mediumUuid = state.getCenterRegion(['mediumUuid']);
                  state.setMediaStatus(mediumUuid, MediumViewStatus.MEDIUM_STATUS_CLOSED);
                })
                ["catch"](function (err) {
                $log.error("logout failure", err);
              });
            }
          }else{
            state.events().reset();
            state.data.save({ 'openMediaWcbRequestSent' : false });
            state.setCurrentMediaType('');
            session.fetchEventsIsRunning=false;
            if (session.fetchMediaIsRunning===false){
              session.fetchMediaIsRunning=true;
              session.fetchMedia();
            }


            if ( angular.isDefined( trigger.current ) ){
              trigger.current.reject( );
            }
            navigation.quit();
          }
        };

        $scope.canQuit = function(){
          var mediumUuid = state.getCenterRegion( ['mediumUuid'] );
          if(mediumUuid==='8'||(boxParams.ASK_QUESTION_BEFORE_CHAT && session.fetchMediaIsRunning)){
            return true;
          }
          return state.events().length>0;
        };

        $scope.titleBar = {
          title: function(){
            var mediumUuid = state.getCenterRegion( ['mediumUuid'] );
            if(mediumUuid==='8'){
              return labels.WCB_TITLE;
            }
            return state.getTitle();
          },

          /**
           * holds the collapse state. it reads it directly from the storage
           */
          collapse: $scope.state.data.get('ui').collapse || false,

          /**
           * expand/collapse alternatively the window
           */
          toggleCollapse: function( event ){
            event.stopPropagation( );
            if ( jQuery( event.target ).hasClass( 'awb-titlebar-can-contract' )){
              $scope.titleBar.collapse = !$scope.titleBar.collapse;
              $scope.state.data.save({ui:{collapse:$scope.titleBar.collapse}});
              // Refresh IFrame size
              setTimeout(function() {
                $rootScope.$emit('webbox:animate');
              }, 1);
            }
          },

          isCollapsed: function(){
            return $scope.titleBar.collapse;
          },

          canCollapse:function(){
            return state.getCollapseIcon();
          },

          isClosable: function(){
            var mediumStatus = state.getCenterRegion( ['mediumStatus'] );
            return !angular.isDefined( mediumStatus ) || mediumStatus === MediumViewStatus.MEDIUM_STATUS_CLOSED || isTerminalPanel;
          },

          setPanelAsClosable: function( value ){
            isTerminalPanel = value;
          },

          /**
           * close the window
           */
          close: function( event ){
            var mediumUuid = state.getCenterRegion( ['mediumUuid'] );
            if(mediumUuid!=='8'||boxParams.ASK_QUESTION_BEFORE_CHAT){
              event.preventDefault( );
              state.openMediaChatRequestSent=false;
              state.setCurrentMediaType('');
            }
            navigation.leaveMedia( mediumUuid );
            session.fetchEventsIsRunning=false;
            navigation.quit();


            if ( angular.isDefined( trigger.current ) ){
              trigger.current.reject( );
            }
          },

          expand: function(){
            $scope.titleBar.collapse = false;
            $scope.state.data.save({ui:{collapse:$scope.titleBar.collapse}});
            setTimeout(function() {
              $rootScope.$emit('webbox:animate');
            }, 1);
          }
        };
      }

      return {
        restrict: 'EA',
        replace: true,
        templateUrl: 'app/layout/window/titlebar.tpl.html',
        controller: ['$scope', controller]
      };
    }
  ]);
