'use strict';

angular.module('akio.webbox')
  .controller('windowCtrl', ['$scope', '$rootScope', '$element', 'state', 'trigger', 'navigation', 'Layout', 'ViewState', 'MediumType',
    'i18nMessages', 'session', 'boxParams', 'determineClient',
    function ($scope, $rootScope, $element, state, trigger, navigation, Layout, ViewState, MediumType, labels, session, boxParams, determineClient) {
      $scope.askQuestion = boxParams.ASK_QUESTION_BEFORE_CHAT;
      $scope.question = labels.CHATBOX_ASK_QUESTION;
      $scope.Layout = Layout;

      $scope.loader = window.AkioBaseUrl + 'images/spin.gif';

      $scope.state = state;

      $scope.isDisplayOnSmartphone = determineClient.device === 'smartphone';

      /* resize the iframeWrapper */
      var setWindowPosition = function () {
        if (determineClient.device === 'smartphone') {
          jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
            width: '100%',
            right: '0',
            bottom: '0',
            boxShadow: 'none',
            borderRadius: '0'
          });
        } else if (determineClient.device === 'tablet' || boxParams.IS_DISPLAY_LAUNCHBAR_AS_BUBBLE) {
          jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
            width: '300px',
            right: '0',
            bottom: '0',
            boxShadow: 'none',
            borderRadius: '0'
          });
        } else {
          jQuery('#akioWebBoxIFrameWrapper', window.parent.document).css({
            right: '20px',
          });
          $rootScope.$emit('webbox:animate', $element.height());
        }
      };

      var hasNoRunningMediaAndMustAskQuestion = function(buttonType) {
        return $scope.askQuestion === true &&
               !session.fetchEventsIsRunning &&
               state.getCurrentMediaType() === '' &&
               _.find(boxParams.CHAT_FORM, { type: buttonType });
      };


      if ($rootScope.firstDisplay) {
        setWindowPosition();
        $rootScope.firstDisplay = false;
      } else {
        setTimeout(setWindowPosition, 400);
      }

      $scope.win = {
        /**
         * this method returns the current VIEW STATE for each state or individual medium.
         * this method is also called from the «westRegionContainer» template through the «regionWestContainerController» controller
         * @param constraint
         * @returns {string|undefined}
         */
        getViewState: function (constraint) {
          var region = state.getByRegion(constraint),
            regionMediumType = region.mediumType,
            isMediaPending = state.isMediumPending(region),
            viewState;
          switch (regionMediumType) {
            case MediumType.MEDIUM_CHAT:
              if (hasNoRunningMediaAndMustAskQuestion('button')) {
                viewState = ViewState.STATE_CHATFORM;
              } else {
                viewState = isMediaPending ? ViewState.STATE_WAIT : ViewState.STATE_CHATBOX;
              }
              break;

            case MediumType.MEDIUM_WCB:
              if (hasNoRunningMediaAndMustAskQuestion('buttonWCB')) {
                viewState = ViewState.STATE_CHATFORM;
              } else {
                state.setCurrentMediaType(state.mediaList.findWhere({'type': 'VOICE'}).get('type'));
                if(state.isMediumCalendar(region)){
                  viewState = ViewState.STATE_WCB_CALENDAR;
                } else if (state.isMediumConfirm(region)){
                  viewState = ViewState.STATE_WCB_CONFIRM;
                } else {
                  viewState = ViewState.STATE_WCB;
                }
              }
              break;

            case 'NLS':
              viewState = 'SEARCH';
              break;

            default:
              console.log("no viewState define for : " + viewState);
          }
          return viewState;
        },

        /**
         * triggers the visibility of the North Region.
         * At the moment, it is reserved to the static avatar image in chat mode
         * @returns Boolean
         */
        hasRegionNorth: function () {
          var region = state.getCenterRegion(),
            inConversation = state.isMediumOpened(region);
          if ($scope.titleBar) {//wait for scope
            return (region.mediumType === MediumType.MEDIUM_CHAT) && inConversation && !$scope.titleBar.collapse;
          }
        },

        /**
         * triggers the visibility of the West Region. This region is included in a sliding panel
         * @returns Boolean
         */
        hasRegionWest: function () {
          var regions = state.get().attributes.regions;
          return _.findIndex(regions, {constraint: $scope.Layout.BORDER_WEST}) > -1 || false;
        },

        /**
         * triggers the visibility of the invitation trigger
         * @returns Boolean
         */
        hasTrigger: function () {
          return trigger.exists();
        },

        /**
         * return information about conversation ending
         * @returns Boolean
         */
        isConversationOver: function () {
          var events = state.events(),
            lastEvent = events.at(events.length - 1) || {},
            lastEventId = (lastEvent.attributes || {}).eventId || 0;
          return lastEventId <= 0 || lastEvent.isMediumClosedEvent() || lastEvent.isDissuadedMessage();
        },

        /**
         * return information about conversation currently running
         * @returns Boolean
         */
        isConversationRunning: function () {
          return !this.isConversationOver();
        },

        /**
         * the application is processing. No entry is allowed.
         * the lightbox is displayed to keep the user from interacting
         * @returns Boolean
         */
        isNavigationLocked: function () {
          return navigation.isLocked();
        }
      };

    }
  ]);
