'use strict';

angular.module('akio.webbox')
  .factory('MediumModel', ['Backbone', 'MediumStatus', 'MediumType',
    function (Backbone, MediumStatus, MediumType) {
      return Backbone.Model.extend({
        idAttribute: 'conditionUuid',
        defaults: {
          uuid: undefined,                         /* unique identifier */
          type: undefined,                         /* MEDIA_CHAT | MEDIA_WCB */
          status: MediumStatus.MEDIUM_STATUS_CLOSED, /* CLOSED | OPEN_AVAILABLE | OPEN_NOT_AVAILABLE */
          title: '',
          estimatedWaitTime: 0,                                 /* estimated wait time in secs */
          description: 'no description',                  /* current description - future opening - */
          textInput: false,                             /* text entry */
          exposed: false,                             /* exposed through the launchbar */
          message: '',                                /* display in the launch bar */
          icon: '',
          conditionUuid: ''
        },

        /**
         * returns TRUE when the medium is opened and some agent are available ( meaning all the queues are not full )
         * @returns {boolean}
         */
        isOpenedAndAgentAvailable: function () {
          return this.get('status') === MediumStatus.MEDIUM_STATUS_OPEN_AVAILABLE;
        },

        isDelayedWebCallback: function() {
          return this.get('status') === MediumStatus.MEDIUM_STATUS_DELAYED_WEBCALLBACK;
        },

        /**
         * returns TRUE when the medium is opened but there is no agent available ( agent in chat but no more room in the queues )
         * @returns {boolean}
         */
        // isOpenedAndAgentNotAvailable: function () {
        //   var status = this.get('status');
        //   return status === MediumStatus.MEDIUM_STATUS_OPEN_NOT_AVAILABLE;
        // },

        /**
         * returns TRUE when medium is closed
         * @returns {boolean}
         */
        isClosed: function () {
          return this.get('status') === MediumStatus.MEDIUM_STATUS_CLOSED;
        },

        getClassByStatus: function () {
          switch (this.get('status')) {
            case MediumStatus.MEDIUM_STATUS_OPEN_AVAILABLE:
            case MediumStatus.MEDIUM_STATUS_DELAYED_WEBCALLBACK:
              return 'medium-open-available';
            case MediumStatus.MEDIUM_STATUS_OPEN_NOT_AVAILABLE:
              return 'medium-open-not-available';
            case MediumStatus.MEDIUM_STATUS_CLOSED:
              return 'medium-closed';
          }
        },

        // isExposed: function () {
        //   return this.get('exposed');
        // },

        hasTextEntry: function () {
          return this.get('textInput');
        },


        /**
         * return TRU if the medium is CHAT
         * @returns {boolean}
         */
        isChat: function () {
          return this.get('type') === MediumType.MEDIUM_CHAT;
        }
      });
    }])

  .factory('MediaCollection', ['Backbone', 'MediumModel',
    function (Backbone, MediumModel) {
      return Backbone.Collection.extend({
        model: MediumModel,

        getMediumByUuid: function (uuid) {
          return _.find(this.models, function (item) {
            return item.get('uuid') === uuid;
          });
        },

        hasSomeAvailable: function () {
          return _.any(this.models, function (medium) {
            return medium.isOpenedAndAgentAvailable() || medium.isDelayedWebCallback();
          });
        },

        isPriorityMediumNl: function () {
          var priorityMedium = _.min(this.models, function (medium) {
            return medium.get('priority');
          });
          return priorityMedium.hasTextEntry();
        },

        hasMultipleNlMediaAvailable: function () {
          var count = 0;
          _.each(this.models, function (medium) {
            if (medium.hasTextEntry() && medium.isOpenedAndAgentAvailable()) {
              count++;
            }
          });
          return count > 1;
        }
      });
    }
  ]);
