'use strict';


//---- 4.3.3 fonctionnality *NOT* restored  ----
// (+) remove avatar in message item
// (+) messages layout
// (+) info message look&feel

//---- new fonctionnality  ----
// (+) add icon to info message description

/**
 * (REQUIRED jQuery-color plugin)
 *
 * chat module features
 * - handles synchronous as asynchronous mode
 * - balloon theme through the «theme» property
 * - links removal cycle (apply to non hyperlink tags)
 *  + removePermanentLinks = TRUE => all non hyperlink tags are removed at start and after each new message arrival
 *  + removePermanentLinks = FALSE => all non hyperlink tags that are not permanent are removed at start only
 * - scrolling automatically to the bottom of the list after the addition of each new message
 * - handling user inactivity
 *   + the process is reseted each time the search text changes
 *   + the process is launched after each message arrival
 * - enabling nickname display
 * - (static avatar image display is controlled outside the component - north region)
 * - keep from sending messages when an answer is pending.
 *  => The user can still type his message. The input bg color is colouring red to warn the user of the current status.
 *  (this is the behavior adopted in the 4.3.3 version). Colouring red when the user failed to send his message plus displaying an explicit info message
 *  may do the trick.
 *  - permanent as temporary messages are displayed via the «info» attribute
 *  - the input field content is erased if the message is posted successfully. during that time, the input field is not accessible. the error is handled by the box.
 *  - the *operatorLeftEvent* is logged if the operator is not the last one connected
 *  - when the conversation is stopped. The input field is hidden and all the links removed.
 *  - when the conversation is stopped, the restart button is displayed if the media is available.
 */
angular.module('chat', [])
  .directive('awbChat', ['$log', '$timeout', 'i18nMessages','boxParams','determineClient',
    /* istanbul ignore next */
    function ($log, $timeout, labels,boxParams, determineClient) {
      /**
      *remove url autodetect for ie11
      */
      var isIe11 = /Trident.*rv[ :]*11\./.test(navigator.userAgent);
      if(isIe11){
        document.execCommand("AutoUrlDetect", false, false);
      }

      /**
       * Remove NL-links from the conversation history except on the last utterance.
       * @param removePermanentLinks
       */
      var removeNLLinks = function (removePermanentLinks, $$li) {
        var $$a = jQuery('a', $$li).filter(".event-link, .nl-link");

        if (removePermanentLinks === 'false') {
          $$a = $$a.filter(function () {
            return !jQuery(this).hasClass("permanent");
          });
        }
        $$a.replaceWith(function () {
          return jQuery(this).html();
        });
      },

      /**
       * @inheritDoc
       */
      link = function (scope, iElement) {
        // fires when the component is destroyed
        var ready                 = true,
          $awbTabs                 = $('.awb-tabs'),
          $widget                 = $('#awbWidget'),
          $titlebar               = $('.awb-titlebar'),
          $titleBarActionButton   = $('.awb-icon'),
          element                 = jQuery(iElement),
          $inputEl                = element.find('#awb-inputTextArea'),
          $input                  = element.find('.awb-chat-input'),
          $output                 = element.find('.awb-chat-output'),
          $historyEl              = element.find('.awb-chat-history'),
          $awbChatInfoBar         = element.find('.awb-chat-infobar'),
          isStarting              = true,
          originalInputBgColor    = $inputEl.css("background-color"),
          textareaMaxHeight       = parseFloat($inputEl.css("max-height"), 10),
          isTyping = false,
          isGrey = false,         //memorise the state of the input text area
          lastRequalIds = ['a'],  //memorise eventIds of all conversationRequalificationEvent
          isReady = function () {
            return (scope.syncMode === 'false') || ready;
          },

          /**
           * fires when a link is clicked
           * @param e
           */
          linkClickedHandler = function (e) {
            var $target   = jQuery(e.target),
              cls         = ($target.attr('class') || "").split(' '),
              isNlLink    = _.contains(cls, 'nl-link'),
              isEventLink = _.contains(cls, 'event-link');

            if (isNlLink) {
              e.preventDefault();
              var val = $target.attr('data-link').replace(/'/g, "\\'");
              scope.link({type:"nl", data: encodeURIComponent(val).replace("%C2%A0"," ")});
            } else if (isEventLink) {
              e.preventDefault();
              var link= $target.attr('data-link'),
                data= link.split("|", 2),
                value = data.length === 2 ? encodeURIComponent(data[1]) : "";
              scope.link({type:"event", data:[data[0], value]});
            } else {
              // url link is processed as normal
            }
          },

          startTyping=function( ){
            scope.notify( { type:'typing', status:true } );
          },

          stopTyping=function( ){
            isTyping = false;
            if(determineClient.device === 'desktop'){
              scope.notify( { type:'typing', status:false } );
            }else{
              setTimeout(function(){scope.notify( { type:'typing', status:false } );},300);
            }
          },


          /**
           * scroll to the bottom of the messages list
           */
          scrollToBottom = function () {
            if ($output.length === 1) {
              $output.scrollTop($output[0].scrollHeight );
            }
          },

          /**
           *resize textarea when emoji or attachment is active
           */
          resizeTextarea=function () {
            var size;
            var $awbChatButtonPanel = element.find('.awb-chat-panel-button');
            size = $input.outerWidth() - $awbChatButtonPanel.width() - $input.css('padding-left');
            $inputEl.width(size);
          },

          /**
           * scroll to bottom of the textarea.
           */
          scrollToBottomTextArea = function () {
            var textArea = document.getElementsByClassName('awb-textArea')[0];
            setTimeout(function () {
              textArea.scrollTop = textArea.scrollHeight;
            }, 100);
          },

          unregisterOngoingWatcher = scope.$watch(
            function () {
              return scope.ongoing;
            },
            function (newValue) {
              if (newValue === false) { // the conversation has stopped
                $inputEl.css('height', $inputEl.css('min-height'));
                $input.hide();
                $awbChatInfoBar.hide();
                setTimeout(function (){
                  if(determineClient.device==="desktop" || determineClient.device==="tablet"){
                    $output.height($output.height());
                  }else if(determineClient.device === "smartphone"){

                    if (determineClient.viewport.height > 600) {
                      $output.height("70vh");
                    }else{
                      $output.height("60vh");
                    }
                  }
                }, 400);
                // remove links
                $timeout(function () {
                  var $$li= jQuery('li', $historyEl);
                  removeNLLinks('true', $$li);
                }, 0);
              }
            }),
            unregisterMediumAvailabilityWatcher = scope.$watch(
              function () {
                return scope.mediumAvailability;
              },
              function () {
                scope.logoutButtonList = scope.logoutButtons();
              }),
              unregisterMessagesWatcher = scope.$watchCollection('messages', function (newValues) {
                scope.agentNum = 0;
                _.forEach(newValues, function (event) {
                  var isANewId = true;
                  for( var i = 0; i < lastRequalIds.length; i++) {
                    if (lastRequalIds[i] === event.attributes.eventId && event.isConversationRequalificationEvent()) {
                      isANewId = false;
                      break;
                    }
                  }
                  if (event.isDissuadedEvent()) {
                    scope.logout();
                  }

                  if (event.isConversationRequalificationEvent()) {
                    if (isANewId) {
                      lastRequalIds.push(event.attributes.eventId);
                      //memorise the text in input text area
                      scope.memoriseInputText = scope.text;
                      scope.inputLocked = true;
                      $inputEl.attr("contenteditable","false");
                      //change color of input text area : use grey
                      if (!isGrey) {
                        $inputEl.animate({"background-color" : '#e6e6e6'}, 800);
                        jQuery(".awb-chat-input-border-in").animate({"background-color" : '#e6e6e6'}, 800);
                        //stop focus on input element
                        //empty the text input area
                        scope.text = "";
                        isGrey = true;
                        $inputEl.attr("placeholder", '');
                      }
                      //we display the event message
                      scope.infoText = event.get('message');
                      scope.operatorIsTyping = false;
                    }
                    scope.agentNum = scope.agentNum - 1;
                  } else if (event.isOperatorJoinedEvent()) {
                    $inputEl.attr("placeholder",labels.CHATBOX_ASK_QUESTION);
                    scope.agentNum = scope.agentNum + 1;
                    if (scope.inputLocked) {
                      //unlock the text area
                      scope.inputLocked = false;
                      $inputEl.attr("contenteditable", "true");

                      //display the  text memorise before requalififctaion
                      scope.text = scope.memoriseInputText;
                      //erase the event message
                      scope.infoText = event.get('');
                      //set default color of input text area
                      if (isGrey) {
                        $inputEl.animate({"background-color": originalInputBgColor}, 800);
                        jQuery(".awb-chat-input-border-in").animate({"background-color": 'white'}, 800);
                        $inputEl.contentEdit = true;
                        isGrey = false;
                        //focus on input element
                        $inputEl.focus();
                      }
                    }
                  } else if (event.isOperatorLeftEvent()) {
                    scope.agentNum = scope.agentNum - 1;
                  }
                  if (event.isTypingStartedEvent()) {
                    // notify user the agent is typing
                    scope.operatorIsTyping = true;
                    scope.infoText = '';
                  } else if (event.isTypingStoppedEvent() || event.isDissuadedMessage()) {
                    // hide agent typing notification
                    scope.infoText = '';
                    scope.operatorIsTyping = false;
                    $inputEl.attr("placeholder", labels.CHATBOX_ASK_QUESTION);
                  }
                });

                if (scope.removePermanentLinks === "false") {
                  // At start and after each new message
                  var $$li= jQuery('li', $historyEl);

                  if (!isStarting) { // remove the last utterance from the selection
                    $$li = $$li.slice(0, $$li.length - 1);
                  }
                  // trick to process codes when template has been loaded, cloned and transformed and rendered
                  $timeout(function () {
                    removeNLLinks(scope.removePermanentLinks, $$li);
                  }, 0);
                } else if (isStarting) {
                  // only at start
                  $timeout(function () {
                    removeNLLinks(scope.removePermanentLinks, $$li);
                  }, 0);
                }

                // scroll to the bottom after each new message
                $timeout(function () {
                  scrollToBottom();
                }, 0);

                isStarting = false;
                ready = true;
              });

        var isIE11 = !!navigator.userAgent.match(/Trident.*rv[ :]*11\./);

        // textarea scrollTobottom for IE 11 only
        if (isIE11) {
          var unregisterCollapseWatcher2 = scope.$watch('collapsed', function (collapsed) {
            if (!collapsed) {
              scrollToBottomTextArea();
              scrollToBottom();
            }
          });
        }

        scope.memoriseInputText = '';//used to memorise the text before a requalification. After the requalification the text is display in the input text area
        scope.buttons = [];
        scope.deltaSinceLastEnter = 0;//used to avoid double tap on enter key to send the same message twice.
        scope.timeBetweenTwoPost = 500;// time in ms
        scope.emojiVisible = boxParams.EMOJI_VISIBLE;
        scope.attachmentVisible = boxParams.ATTACHMENT_VISIBLE;

        scope.isDisplayOnDesktop = function () {
          return determineClient.device === 'desktop';
        };

        /**
         * @param event
         * @returns {boolean}
         */
        scope.isMessageDisplayed = function (event) {
          var isDisplayed = true;
          if (event.isTypingStartedEvent() || event.isTypingStoppedEvent()) {
            isDisplayed = false;
          } else if (event.isOperatorLeftEvent() && scope.agentNum === 0) {
            isDisplayed = false;
            scope.operatorIsTyping = false;
          }
          return isDisplayed;
        };

        scope.agentNum = 0;

        /**
         * input locked until the message has been sent successfully.
         * @type {boolean}
         */
        scope.inputLocked = false;


        scope.chatButtonVisible = function () {
          if ((boxParams.EMOJI_VISIBLE || boxParams.ATTACHMENT_VISIBLE) && !isGrey) {
            resizeTextarea();
            return true;
          } else {
            resizeTextarea();
            return false;
          }
        };

        scope.isDisplayOnMobile = function () {
          return determineClient.device === 'smartphone';
        };

        scope.fitOutputToOriginalSize = function () {
          if(determineClient.device ==='smartphone') {
            $output.height('70vh');
            $awbTabs.css('height', '93vh');
            $widget.css('bottom', '0px');
            window.scrollTo(0, 1);
          }
        };

        scope.resizeScreenHeight=function () {
          if (determineClient.device ==='smartphone') {
            if (determineClient.os === 'iOS') {
              if (determineClient.viewport.height > 600) {
                $output.height('25vh');
                $awbTabs.css('height', '52vh');
                $widget.css('bottom', '40%');
              } else {
                $output.height('25vh');
                $awbTabs.css('height', '42vh');
                $widget.css('bottom', '50%');
              }
            } else if (determineClient.os === 'AndroidOS') {
              $output.height('30vh');
              $awbTabs.css('height', '47vh');
            }
            window.scrollTo(0, 1);
          }
        };

        /**
         * Emoji Code Start Here
         */
        scope.selectionCarret = 0;
        scope.previousElementSelectionCarret="";

        scope.updateSelectionCarret = function () {
          if (document.getSelection().focusNode &&
            document.getSelection().focusNode.previousSibling &&
            document.getSelection().focusNode.previousSibling.nodeName !== "#comment") {
            scope.previousElementSelectionCarret = document.getSelection().focusNode.previousSibling;
            scope.selectionCarret = document.getSelection().focusOffset;
          }
          if (document.getSelection().focusNode && document.getSelection().focusNode.parentNode.id === "awb-inputTextArea") {
            scope.selectionCarret = document.getSelection().focusOffset;
            scope.previousElementSelectionCarret = document.getSelection().focusNode.previousSibling;
          }
        };

        //used to stock selection carret before display emohji panel
        scope.setSelectionCarret = function (position) {
          scope.selectionCarret = position;
        };

        scope.emojiPanelActive = false;
        scope.displayEmojiPanel = function () {
          return scope.emojiPanelActive;
        };

        scope.toggleEmojiPanel = function () {
          scope.emojiPanelActive=!scope.emojiPanelActive;
          var emojiPanelHeight = 100;

          if (scope.emojiPanelActive) {
            $output.height($output.height() - emojiPanelHeight);
            $(".awb-chat-button-action")[0].classList.add('awb-chat-button-active');
          } else {
            $output.height($output.height() + emojiPanelHeight);
            $(".awb-chat-button-action")[0].classList.remove('awb-chat-button-active');
          }
        };

        scope.closeEmojiPanel = function () {
          if (scope.emojiPanelActive) {
            scope.toggleEmojiPanel();
            scope.$apply();
          }
        };

        //used to close emoji panel when click on inputEl
        $inputEl.click(function () {
          if (scope.emojiPanelActive) {
            scope.closeEmojiPanel();
          }
        });

        //used to close emoji panel when click on outputEl
        $output.click(function () {
          if(scope.emojiPanelActive) {
            scope.closeEmojiPanel();
          }
        });

        //used to close emoji panel when click outside the window
        $(window).blur(function () {
          if (scope.emojiPanelActive) {
            scope.closeEmojiPanel();
          }
        });

        //used to close emoji panel when click on titlebar
        $titlebar.click(function () {
          if (scope.emojiPanelActive) {
            scope.closeEmojiPanel();
          }
        });

        $titleBarActionButton.click(function () {
          if (scope.emojiPanelActive) {
            scope.closeEmojiPanel();
          }
        });

        //Emoji code end here

        /**
         * scrolls to the bottom of the list when a message is displayed to show it
         */
        scope.onInfoShow = function () {
          scrollToBottom();
        };

        /**
         * Is the textarea is scrollable
         * @type {boolean}
         */
        scope.scrollable = false;

          /**
           * fires when the input text value changes
           */
        scope.textChange = function( ){
          if (isReady()){
            if (!isTyping){
              $inputEl.attr("placeholder",labels.CHATBOX_ASK_QUESTION);
              if(scope.text!==null && (scope.text.length >= 1 && scope.text !=="<br>")){
                isTyping = true;
                startTyping();
              }
            }else if (isTyping){
              if(scope.text!==null && (scope.text.length === 0 ||scope.text ==="<br>" )){
                stopTyping();
              }
            }
          } else { /* waiting an answer */
            $inputEl
              .clearQueue().stop()
              .css("background-color", "#F08080")
              .animate({"background-color" : originalInputBgColor}, 800);
          }

          if (!scope.scrollable) {
            $inputEl.css("height", "0px");
            var textarea = $inputEl.get(0);
            var scrollHeight = textarea.scrollHeight;

            if (scrollHeight > textareaMaxHeight) {
              scope.scrollable = true;
            }
          }
        };

        scope.enter = function (event) {
          if (event.keyCode === 13) { // ENTER key
            scope.text = $inputEl.html();
            var enterIsValid = Date.now() - scope.deltaSinceLastEnter > scope.timeBetweenTwoPost; //used to avoid double tap on enter key to send the same message twice. if last input key 13 was <900 then we block the input key 13
            event.preventDefault();
            if (enterIsValid) {
              scope.deltaSinceLastEnter = Date.now();
              if (isReady() && scope.ongoing && scope.text.length > 0 &&scope.text !=='<br>'){ //ie automatically add br in contentEdit div
                scope.inputLocked = true;
                scope.submit({text : scope.text})
                  .then(function clearInputField() {
                    // message posted successfully
                    scope.inputLocked = false;
                    scope.text = "";
                    stopTyping();

                    if (scope.emojiPanelActive) {
                      scope.toggleEmojiPanel();
                    }
                    scope.setSelectionCarret(0);
                    scope.resizeScreenHeight();

                    //resize text input
                    $timeout(function () {
                      $inputEl.css("height", "0px").css("height", $inputEl.get(0).scrollHeight + "px");
                      scope.scrollable = false;
                    }, 10);
                  })
                  ["catch"](function () {
                    $log.error("- ChatModule.message.sent failed");
                    scope.inputLocked = false;
                  });
                ready = false;
              } else {
                $log.info('chat is not ready!');
              }
            }
          }
        };

        /**
         * fires when the user logouts
         * @param event
         */
        scope.doLogout = function (event) {
          event.preventDefault();
          scope.logout();
        };

        scope.getMessageEventSenderClass = function (message) {
          if (message.isOperatorType()) {
            return 'awb-chat-speaker-operator';
          } else if (message.isAssistantType()) {
            return 'awb-chat-speaker-assistant';
          } else if (message.isUserType()) {
            return 'awb-chat-speaker-user';
          } else if (message.isSystemType()) {
            return 'awb-chat-speaker-system';
          } else if (message.isNetConnectionLazy()) {
            $inputEl.blur();
          }
        };

        scope.getLogMessageEventTypeClass = function (message) {
          if (message.isLogoutEvent() || message.isDissuadedMessage()) {
            return 'awb-chat-info-logout';
          } else if (message.isOperatorLeftEvent()) {
            return 'awb-chat-info-agent-left';
          } else if (message.isNetConnectionLazy()) {
            $inputEl.blur();
          }
        };

        /**
         * fires when the directive is destroyed
         */
        scope.$on('$destroy', function () {
          // unregisters watchers
          unregisterOngoingWatcher();
          unregisterMediumAvailabilityWatcher();
          unregisterMessagesWatcher();

          //unregister IE scroll handlers
          if (isIE11) {
            unregisterCollapseWatcher2();
          }

          // unregisters DOM event handlers
          $output.off();
          $inputEl.off();
          $input.off();
        });

        // registers listeners

        $input.hover(function () {
          $input.toggleClass("awb-chat-hover"); // ":hover" does not work on IE Quirks
        });

        $inputEl.focus(function () {
          $input.addClass("awb-chat-selected");
          scope.resizeScreenHeight();
        }).blur(function () {
          $input.removeClass("awb-chat-selected");
          scope.fitOutputToOriginalSize();
        });

        /**
         * listens click on link element
         */
        $output.on('click', linkClickedHandler);

        // Debug function designed for automatic testing
        window.Akio.Webbox.debug = {
          'chat': {
            'fillTextarea': function (text) {
              scope.text=text;
              scope.$digest();
            },
            'isInputAreaLocked': function () {
              return scope.inputLocked;
            }
          }
        };
      };

      return {
        restrict  : "EA",
        replace   : true,
        scope     : {
          buttons             : '=',    /* list of buttons to display in the control bar */
          ongoing             : '=',   /* tells whether the conversation is ongoing */
          displayNickname     : '@',    /* activate nickname display */
          inactivityDelay     : '@',    /* inactivity delay value in ms */
          messages            : '=',    /* list of messages */
          syncMode            : '@',    /* synchronous mode */
          placeholder         : '@',
          removePermanentLinks: '@',
          text                : '=',    /* input text */
          theme               : '@',    /* css theme */
          inactivity          : '&',    /* fires after a given time of inactivity */
          submit              : '&',    /* post a message */
          logout              : '&',    /* logout */
          link                : '&',     /* link event */
          notify              : '&',
          labels              : '=',
          logoutButtons       : '=',
          mediumAvailability  : '=',
          collapsed           : '=',
          infoText            :'@infoText',
          infoDelay           :'@infoDelay'
        },
        templateUrl: "app/media/chat/chat.tpl.html",
        link       : link
      };
    }
  ])


  /**
   * info bar directive
   * it attaches a permanent|temporary message
   */
  .directive('awbChatInfobar', ['$timeout', function ($timeout) {
    /**
     * @inheritDoc
     */
    function link(scope) {
      // invalid | relevant | expired | upToDate | consumed
      var unregisterTextWatcher,
        timer;

      function isPermanent() {
        var delay = Number(scope.delay);
        return delay === 0 || isNaN(delay);
      }

      scope.isValid = function () {
        return scope.text !== '';
      };

      unregisterTextWatcher = scope.$watch(
        function () { /* watch expression */
          return scope.text; // return the value to be watched
        }, function (newValue) { /* change listener */
          if (timer) {
            $timeout.cancel(timer);
          }

          if (newValue !== '') {
            if (!isPermanent()) {
              timer = $timeout(function () { /* info relevance has expired */
                scope.text = '';
              }, scope.delay);
            }

            $timeout(function () {
              scope.infoShow();
            }, 0);
          }
        }
      );

      scope.$on('$destroy', function () {
        unregisterTextWatcher();
        $timeout.cancel(timer); // for safety
      });
    }

    return {
      restrict: 'EA', /* declaring it as an Element is better to clarify the code. */
      replace: true,
      scope: {
        text: '@infoText',
        delay: '@infoDelay',
        infoShow: '&'
      },
      template: '<div class="awb-chat-infobar" ng-show="isValid()">{{text}}</div>',
      link: link
    };
  }]);
