'use strict';

angular.module('akio.webbox').controller('chatCtrl', ['$scope', '$log', 'state', 'session','aip', '$q', 'MediumViewStatus', 'notification', 'MediumType','i18nMessages','$timeout','boxParams','emojis',
  function( $scope, $log, state,session, aip, $q, MediumViewStatus, notification, MediumType, labels,$timeout,boxParams,emojis ){
    var region      = state.getCenterRegion( ),
      mediumType  = region.mediumType,
      mediumUuid  = region.mediumUuid,
      chatRestarting = false,
      unregisterMediumStatusChange = $scope.$watch( function(){
        return state.getMediaStatus();
      }, function( newValue){
        switch ( newValue ){
          case MediumViewStatus.MEDIUM_STATUS_OPENED:
            state.setTitle( $scope.labels.CHATBOX_TITLE_ONGOING_CONVERSATION );
            break;
          case MediumViewStatus.MEDIUM_STATUS_CLOSED:
            state.setTitle( $scope.labels.BOX_TITLE_DEFAULT );
            $timeout(function(){state.events( ).off( 'add' );},boxParams.POLLING_DELAY_EVENT);
            break;
        }
      });

    // expand chat on init

    if ($scope.titleBar){ //wait for scope
      if($scope.titleBar.isCollapsed( )) {
        $scope.titleBar.expand( );
      }
    }

    //---------
    // inputs
    //---------

    $scope.chat = {

      /**
       * set the collection of events
       */
      events: state.events( ).getMediumEvents( ), //models,

      /**
       * list of buttons to display in actions bar
       * @type {Array}
       */
      buttons: [ ],

      /**
       * activates the nickname display on the UI
       * @type {boolean}
       */
      displayNickName : true,


      /**
       * synchronous mode
       * @type {boolean}
       */
      syncMode : false,

      /**
       * css theme
       * @type {string}
       */
      theme : "awb-chat-theme-balloon",

      /**
       * info message
       * @type {{text: string, delay: number}}
       */
      info: {
        text: '',
        delay: 0
      },

      isSendingMessage : false,

      /**
       * tells the kind of links we make eligible for removal.
       * if TRUE, all links (nl-link, event-link) are removed - permanent or not.
       * if FALSE, only non-permanent links are removed
       * @type {boolean}
       */
      removePermanentLinks : ( mediumType === MediumType.MEDIUM_CHAT ),


      text : '',


      /**
       * tells whether a conversation is ongoing
       * @returns {boolean}
       */
      ongoing : function( ){
        return state.isMediumOpened( mediumType );
      },




      isMediumAvailable : function( ){
        var medium          = state.mediaList.findWhere({uuid: mediumUuid});

        return _.isUndefined( medium )||session.isNetConnectionDown? false :medium.isOpenedAndAgentAvailable( );
      },
      displayRestartButton : function( ){
        return session.isNetConnectionDown? false :true;
      },

      logoutButtons : function( ){

        var items                = [ ],
          isMediumAvailable    = $scope.chat.isMediumAvailable(),
          displayButtons = $scope.chat.displayRestartButton();
        if(displayButtons){
          if (isMediumAvailable) {
            items.push({
              label: $scope.labels.CHATBOX_RESTART_CONVERSATION,
              title: $scope.labels.CHATBOX_RESTART_CONVERSATION,
              action:"restart",
              iconCls: 'icon-refresh',
              handler: $scope.chat.restart,
              name:"akioWebBoxRestartButton"
            });
          }
          items.push({
            label: $scope.labels.CHATBOX_DOWNLOAD_CONVERSATION,
            title: $scope.labels.CHATBOX_DOWNLOAD_CONVERSATION,
            action:"download",
            handler: $scope.chat.downloadHistory,
            iconCls: 'icon-download',
            name:"akioWebBoxDownloadButton"

          });
        }
        return items;
      },


      //---------
      // outputs
      //---------


      /**
       * restart media
       */
      restart : function(){
        if(!chatRestarting){
          chatRestarting=true;
          $log.info('chat:restart');
          state.resetChatSessionUri();
          state.setMediaStatus(mediumUuid, MediumViewStatus.MEDIUM_STATUS_PENDING);
          state.events().reset();
          aip.openMediaChat().then(function(){chatRestarting=false;});
        }

      },

      /**
       * download history as pdf
       */

      downloadHistory : function(){
        $log.info('chat:download History');
        var language = languageDetection(boxParams.I18N);
        aip.downloadHistory(language);
      },



      /**
       * fires when the user LOGOUT by itself
       */
      logout : function(){
        aip
          .closeMediaChat()
          .then(function(){
            state.setMediaStatus( mediumUuid, MediumViewStatus.MEDIUM_STATUS_CLOSING );
          })
          ["catch"](function(err){
          $log.error("logout failure", err);
        });
      },

      notify : function( type, status ){
        aip.notify( type, status );
      },

      /**
       * fires when a message is send to the server
       * @param text
       */
      sendMessage : function( text ){
        if ( angular.isDefined( text ) && !$scope.chat.isSendingMessage){
          $scope.chat.isSendingMessage = true;

          text = $("#awb-inputTextArea").html();

          //add emoji keyboard - keyboard emoji is something like ":-)" or ":D"
          if(text){
            emojis.forEach(function(emoji){
              if(emoji.keyboard){
                //we create a regex with the keyboard emoji
                var emojiKeyboard = emoji.keyboard.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, '\\$&');
                var emojiRegex = new RegExp(emojiKeyboard,"g");
                //check if there is a keyboard emoji in the text
                if( text.match(emojiRegex)){
                  text = text.replace(emojiRegex,"<span name='"+emoji.unicode.twitter+"' class='emoji emoji-"+emoji.shortcode+"'></span>");
                }
              }
            });
          }

          return aip
            .sendMessage( text )
            .then(function logMessage( result ){
            $scope.chat.isSendingMessage = false;
            return result;
          }, function( rejection ){
              notification.error(labels.CHAT_SENDMESSAGE_ERROR);
              return $q.reject( rejection );
            });
        }else{
          return $q.when(true);
        }
      }
    };

    $scope.$on('$destroy', function(){
      unregisterMediumStatusChange( );

      state.events( ).off( 'add' );
    });


    //-------
    // main
    //-------

    state.events( ).on( 'add', function( model ){
      $scope.chat.events.push( model );
      var isCollapsed = $scope.titleBar.isCollapsed( ); // declared in titleBar
      if ( isCollapsed && ( model.isMessage( ) || model.isLogMessage( ) || model.isMediumClosedEvent( ) || model.isOperatorLeftEvent( ) ) ){
        $scope.titleBar.expand( );
      }
    } );


  }
]);
