'use strict';



angular.module('akio.webbox')

  .controller('emojiCtrl', ['$scope','emojis',function($scope,emojis) {

    //IE11 specific
    //use It to display emoji correctly in ie11
    function controlselectHandler(evt) {
      evt.preventDefault();
    }
    document.body.addEventListener('mscontrolselect', controlselectHandler);
    //end IE11 specific

    //FireFox specific
    // fix backspace bug in FF for content editable
    // https://bugzilla.mozilla.org/show_bug.cgi?id=685445

    /* istanbul ignore next */
    if(navigator.userAgent.toLowerCase().indexOf('firefox') > -1) {
      $('#awb-inputTextArea').on('keydown', function (event) {
        if (window.getSelection && event.which === 8) { // backspace

          var selection = window.getSelection();
          if (!selection.isCollapsed || !selection.rangeCount) {
            return;
          }

          var curRange = selection.getRangeAt(selection.rangeCount - 1);
          if (curRange.commonAncestorContainer.nodeType === 3 && curRange.startOffset > 0) {
            // we are in child selection. The characters of the text node is being deleted
            return;
          }

          var range = document.createRange();
          if (selection.anchorNode !== this) {
            // selection is in character mode. expand it to the whole editable field
            range.selectNodeContents(this);
            range.setEndBefore(selection.anchorNode);
          } else if (selection.anchorOffset > 0) {
            range.setEnd(this, selection.anchorOffset);
          } else {
            // reached the beginning of editable field
            return;
          }
          range.setStart(this, range.endOffset - 1);


          var previousNode = range.cloneContents().lastChild;
          if (previousNode && (previousNode.nodeType !== 3 || previousNode.data === " ")) {
            // this is some rich content, e.g. smile. We should help the user to delete it
            range.deleteContents();
            event.preventDefault();
          }
        }
      });
    }
    //end firefox specific


    $scope.emojisCollection = emojis;

    var textScope = angular.element(document.getElementById('awb-chat')).scope();
    var selectionPosition ;
    var previousHtmlElement ;


    $scope.emojiClicked = function(emojiUnicode,emojiShortCode){
      if (textScope) {
        selectionPosition = textScope.$$childHead.selectionCarret;
        previousHtmlElement = textScope.$$childHead.previousElementSelectionCarret;
      }
      var $inputElement= $('.awb-textArea');
      insertAtCaret($inputElement,emojiUnicode,emojiShortCode);
    };




    /**
     * used to replace '&nbsp;'
     *
     * @param codeHtml
     * @returns codeHtml Without '&nbsp;'
     */
    function replaceSpace(codeHtml){
      var spaceRegex =/&nbsp;/g;
      if(codeHtml!==null){
        codeHtml=codeHtml.replace(spaceRegex, ' ');
      }
      return codeHtml;
    }
    /**
     * used to delete '<br>;' <br> are automatically add by ie11 when we add an emoji
     *
     * @param codeHtml
     * @returns codeHtml Without '<br>;'
     */
    function replaceBr(codeHtml){
      var brRegex =/<br>/g;
      if(codeHtml!==null) {
        codeHtml = codeHtml.replace(brRegex, '');
      }
      return codeHtml;
    }

    /**
     * used to adapt focus index to html code
     *
     * @param focusindex in an input
     * @returns focus index in the content editable
     */

    $scope.adaptFocusIndex=function(codeHtml,focusIndex,previousHtmlElement){
      var spanRegex = /<span[^>]*><\/span>/g;
      var position={};
      position.originalFocusIndex = focusIndex;
      var m;
      while( (m = spanRegex.exec(codeHtml)) !==null){
        if (m.index === spanRegex.lastIndex) {
          spanRegex.lastIndex++;
        }
        if(previousHtmlElement!==null && m[0].match(previousHtmlElement.id)){
          focusIndex= m.index +m[0].length +focusIndex;
          position.previousElementId = previousHtmlElement.id;
          break;
        }
      }
      position.focusIndex=focusIndex;
      return position;
    };
    /**
     * used to position the focus after adding an emoji
     * @param spanId
     */
    function focusPosition(spanId){
      var focusIndex = 1;
      var previousId = spanId;
      var node = document.getElementById("awb-inputTextArea");
      node.focus();
      var textNodes = node.childNodes;
      var textNode;
      for(var i=0;i<textNodes.length;i++){
        if(textNodes[i].previousSibling!==null){
          if(textNodes[i].previousSibling.id!==null){
            if( parseInt(textNodes[i].previousSibling.id) === previousId){
              textNode = textNodes[i];
            }
          }
        }
      }
      if (textNode===undefined){
        textNode = node;
        focusIndex =0;
      }

      var caret = focusIndex;
      var range = document.createRange();
      range.setStart(textNode, caret);
      range.setEnd(textNode, caret);
      var sel = window.getSelection();
      sel.removeAllRanges();
      sel.addRange(range);
    }


    /**
     * used to insert an emoji in the text
     *
     * @param the jquery input element, emoji value, emojishortCode
     */
    function insertAtCaret(inputField, myValue,emojiShortCode) {//todo : check if all if are used
      textScope = angular.element(document.getElementById('awb-chat')).scope();
      var htmlCode = replaceSpace(inputField.html());
      htmlCode = replaceBr(htmlCode);
      var position = $scope.adaptFocusIndex(htmlCode,selectionPosition,previousHtmlElement);
      var startPos = position.focusIndex;
      var endPos = startPos;
      if(htmlCode!==null){
        var spanId= Date.now();
        inputField.html(htmlCode.substring(0, startPos)+'<span unselectable="on" contenteditable="false" name="'+myValue+'" id="'+spanId+'"class="emoji emoji-'+emojiShortCode+'"></span>&nbsp'+htmlCode.substring(endPos,htmlCode.length));
        textScope.$$childHead.textChange();
        focusPosition(spanId);
        if((navigator.userAgent.indexOf('Trident/')>0)||(navigator.userAgent.toLowerCase().indexOf('firefox') > -1)){
          textScope.$$childHead.updateSelectionCarret();
        }
      }
    }

  }]);
