'use strict';

angular.module('akio.webbox')

  .factory('formHelper', ['state', function(state){
    return {
      get: function(region){
        var events = state.events(),
          event  = events.at( region.data /* === event index */ );

        return event.get('formData');
      }
    };
  }])

  /**
   * controller attached to the form parent container when the form is seen as a primary media
   */
  .controller('formPanelCtrl',['$scope', 'formHelper', 'aip', 'navigation', '$log', 'state', 'notification','i18nMessages',
    function($scope, formHelper, aip, navigation, $log, state, notification, labels){

      $scope.formItem = formHelper.get( state.getCenterRegion() );

      state.setTitle( labels.TITLE_MEDIA_WCB );

      $scope.submit = function(formData){
        // send results to aip
        aip.send( angular.toJson(formData) );

        navigation.quit();

        notification.add(
          {
            kind: "formResult",
            states: [
              {text: labels.WCB_CONFIRMATION}
            ]
          },
          null,
          10,
          false
        );
      };

      $scope.cancel = function(){
        navigation.quit();
      };
    }])
  /**
   * controller attached to the form parent container when the form is seen as a secondary media (pushed by an agent)
   * TODO: extend formPanelController
   */
  .controller('pushedFormCtrl', ['$scope', 'formHelper', 'aip', 'state', 'navigation', 'Layout', 'i18nMessages', '$injector',
    function($scope, formHelper, aip, state, navigation, Layout, labels, $injector){

      var region = state.getWestRegion( );

      function close( ){
        navigation.leaveMedia( region.mediumUuid, Layout.BORDER_WEST );
      }

      $scope.formItem = formHelper.get( region );
      $scope.submit = function( formData ){
        // send results to aip
        aip.send( angular.toJson( formData ) );

        $injector.get('messageReader').injectLogMessage( labels.FORM_SENT, region.mediumUuid, 'FormSubmitEvent' );

        close( );
      };

      $scope.cancel = function( ){
        $injector.get('messageReader').injectLogMessage( labels.FORM_CANCEL, region.mediumUuid, 'FormSubmitCancelEvent' );

        close( );
      };

    }
  ]);
