'use strict';

angular.module('akio.webbox')

  .controller('wcbCtrl', ['$scope', 'aip', 'i18nMessages', 'state', 'session', 'navigation', '$window', 'trigger', 'boxParams', 'MediumViewStatus', 'MediumStatus', '$log',
    function ($scope, aip, labels, state, session, navigation, $window, trigger, boxParams, MediumViewStatus, MediumStatus, $log) {

      var mediumUuid = state.getCenterRegion().mediumUuid;

      state.setCollapseIcon(true);

      // TODO supprimer lorsque la fonctionnalité sera gérée par la règle provenant d'Admin
      $scope.delayedWebCallBack = boxParams.DELAYED_WCB;

      //define labels for UI
      $scope.order = labels.MEDIUM_WCB_ORDER;
      $scope.pleaseWait = labels.WCB_ONGOING_CALL;
      $scope.wrongNumber = labels.WCB_WRONG_NUMBER;
      $scope.callNowButtonLabel = $scope.delayedWebCallBack ? labels.MEDIUM_CALL_NOW_WCB_BUTTON : labels.MEDIUM_WCB_BUTTON;
      $scope.callLaterButtonLabel = labels.MEDIUM_CALL_LATER_WCB_BUTTON;
      $scope.phoneNumberPlaceHolder = labels.MEDIUM_WCB_PHONE_NUMBER_PLACEHOLDER;


      $scope.phoneNumber = state.getPhoneNumber() !== null ? state.getPhoneNumber() : '';

      $scope.baseUrl = $window.AkioBaseUrl;

      /**
       * function use to get default country
       * @param countriesList : array of country object
       * @returns default country object if it is define. if no default country, the default country is the first of the array
       */
      $scope.getDefaultCountry = function (countriesList) {
        var defaultCountry = countriesList[0];
        _.forEach(countriesList, function (value) {
          if (value.isDefault) {
            defaultCountry = value;
          }
        });
        return defaultCountry;
      };

      //define the country list
      $scope.countryList = state.getCountriesList();

      //define the default country as selected country
      $scope.selectedCountry = (typeof state.getSelectedCountry() === 'undefined' || state.getSelectedCountry() === null) ? $scope.getDefaultCountry($scope.countryList) : state.getSelectedCountry();
      //define the style of selected country
      // $scope.selectedStyle={'background-image':'url("'+$scope.baseUrl+'images/country/'+$scope.selectedCountry.alphaCode+'.png")'};

      /**
       * used to change the selected country
       * @param country
       */
      $scope.changeCountry = function (country) {
        $scope.selectedCountry = country;
      };

      /**
       * used to valid form on press enter key
       * @param event
       * @param alphaCode
       */
      $scope.sendFormOnPressEnter = function (event, alphaCode) {
        if (event.keyCode === 13 && $scope.validatePhone($scope.phoneNumber)) {
          $scope.callNow(alphaCode, $scope.phoneNumber);
        }
      };


      /**
       * used to check if number phone is a valid number (no letter but sip are authorized if ALLOW_SIP_NUMBER parameter is set to true)
       * note : there is an other verification on server side
       * @param phone
       * @returns {boolean}
       */
      $scope.validatePhone = function (phone) {
        var regexSip = /(^(\d){7,16}$)|(^(sip:))/;
        var regex = /^(\d){7,16}$/;
        if (boxParams.ALLOW_SIP_NUMBER) {
          regex = regexSip;
        }

        return regex.test(phone);
      };

      $scope.isInstantWcbAvailable = function () {
        var medium = state.mediaList.where({uuid: "8", status: MediumStatus.MEDIUM_STATUS_OPEN_AVAILABLE});
        return medium.length > 0;
      };

      $scope.isDelayedWcbAvailable = function () {
        var medium = state.mediaList.where({uuid: "8", status: MediumStatus.MEDIUM_STATUS_DELAYED_WEBCALLBACK});
        return medium.length > 0;
      };

      $scope.openMediaWcb = function(alphaCode, phoneNumber) {
        var phoneNumberToSend = phoneNumber;
        //check if it's a SIP number
        var regexSip = /^(sip:)/;
        if (boxParams.ALLOW_SIP_NUMBER && regexSip.test(phoneNumber)) {
          alphaCode = null;
        }

        state.data.save({'openMediaWcbRequestSent': true});
        $scope.wcbState = 'onGoingCall';
        $scope.information = labels.WCB_ONGOING_CALL;

        aip.openMediaWcb(alphaCode, phoneNumberToSend)
          .then(function () {
              $scope.updateUI();
            },
            function (error) {
              $log.error('error: ' + error.data.responseText);
              $scope.wcbState = 'wrongNumber';
              if (error.data.status === 422) {
                $scope.information = labels.WCB_WRONG_NUMBER;
              } else {
                $scope.information = labels.WCB_BLACKLISTED_NUMBER;
              }
              session.fetchMediaIsRunning = true;
            });
      };

      /**
       * function used to send phone number to AIP and update UI with WCB state
       *
       * @param alphaCode
       * @param phoneNumber
       */
      $scope.callNow = function (alphaCode, phoneNumber) {
        aip.getMedia().then(function () {
          state.setCurrentMediaCondition(state.mediaList.findWhere({
            'type': 'VOICE',
            'status': 'OPEN_AVAILABLE'
          }).get('conditionUuid'));

          $scope.openMediaWcb(alphaCode, phoneNumber);
        });
      };

      $scope.callLater = function (alphaCode, phoneNumber) {
        aip.getMedia().then(function () {
          state.setCurrentMediaCondition(state.mediaList.findWhere({
            'type': 'VOICE',
            'status': 'OPEN_DELAYED_WEBCALLBACK'
          }).get('conditionUuid'));

          state.setMediaStatus(mediumUuid, MediumViewStatus.MEDIUM_STATUS_CALENDAR);
          state.setSelectedCountry($scope.selectedCountry);
          state.setPhoneNumber(phoneNumber);
        });
      };

      //this watcher look at the net connection and display an information if the net connection is down
      var netConnectionWatcher = $scope.$watch(function () {
        return session.isNetConnectionDown;
      }, function (newValue, oldValue) {
        if (newValue !== oldValue) {
          if (newValue === true) {
            $scope.wcbState = 'onGoingCall';
            $scope.information = labels.WCB_NETWORK_CONNECTION_ERROR;
          } else if (newValue === false && session.fetchMediaIsRunning) {
            $scope.wcbState = 'emptyForm';
            $scope.information = '';
          } else if (newValue === false && session.fetchEventsIsRunning) {
            $scope.updateWcbState(state.getWcbState());
          }
        }
      });

      //we initiate a watcher to look at the wcb state and update the ui
      var wcbStateWatcher;
      $scope.updateUI = function () {
        wcbStateWatcher = $scope.$watch(function () {
          return state.getWcbState();
        }, function (newValue, oldValue) {
          if (newValue !== oldValue) {
            $scope.updateWcbState(newValue);
          }
        });
      };


      /**
       * function used to set the UI
       * @param updatedWcbState
       */
      $scope.updateWcbState = function (updatedWcbState) {
        switch (updatedWcbState) {
          case 'WRONG_NUMBER':
            $scope.wcbState = 'wrongNumber';
            $scope.information = labels.WCB_WRONG_NUMBER;
            break;
          case 'INTERNAL_ERROR_DIALING_IMPOSSIBLE':
          case 'CONNECTION_ERROR':
            $scope.wcbState = 'onGoingCall';
            $scope.information = labels.WCB_NETWORK_CONNECTION_ERROR;
            break;
          case 'CUSTOMER_PHONE_RINGING':
          case 'WCB_IN_PREPARATION' :
          case 'DIALING_IN_PROGRESS' :
            $scope.wcbState = 'onGoingCall';
            $scope.information = labels.WCB_ONGOING_CALL;
            break;

          case 'CALL_FAILED_NEW_ATTEMPT_IN_PROGRESS' :
            $scope.wcbState = 'onGoingCall';
            $scope.information = labels.WCB_CALL_FAIL_TRY_AGAIN;
            break;

          case 'CALL_FAILED_LAST_ATTEMPT' :
            $scope.wcbState = 'emptyForm';
            $scope.information = labels.WCB_CALL_FAIL_NO_TRY;
            break;

          case 'CALL_PICKED_UP_BY_CUSTOMER' :
            $scope.wcbState = 'emptyForm';
            $scope.information = '';
            state.events().reset();
            state.data.save({'openMediaWcbRequestSent': false});
            if (angular.isDefined(trigger.current)) {//todo: look here if we can do better
              trigger.current.reject();
            }
            navigation.quit();
            state.setCurrentMediaType('');
            break;

          default :
            $scope.wcbState = 'emptyform';
            $scope.information = '';
            break;
        }
      };

      //when we destroy the controller we destroy additional watchers. (Call a watcher for a second time kill him)
      $scope.$on('$destroy', function () {
        $scope.destroyWatcher();
      });

      $scope.destroyWatcher = function () {
        netConnectionWatcher();
        if (wcbStateWatcher) {
          wcbStateWatcher();
        }
      };

    }
  ]);
