'use strict';

/**
 * the AIP API
 *
 * services bonds
 *
 *  AIPConnector
 *       |
 *       |
 *       |
 *  CORSConnector
 *       |
 *       | ( callback success & error)
 *       |
 *     Proxy
 *  ( aip.tpl.js injected at build time in proxy)
 *
 *
 */
angular.module('akio.webbox')
  .factory('aip', ['state', 'session', '$q', '$injector', 'ErrorCode', 'xmlUtils', 'media' /* DEBUG */, '$log','clickToChatButton','clickToWcbButton','boxParams','i18nMessages','transformText','ServiceUnavailableEvent',
    function( state, session, $q, $injector, ErrorCode, xmlUtils, media, $log,clickToChatButton,clickToWcbButton,boxParams,labels,transformText,ServiceUnavailableEvent) {
      var messageReader = $injector.get( 'messageReader' ),
        requestId     = 0,
        _connector,

        /**
         * returns the connector that fits the current configuration
         * @returns {*}
         */
        getConnector=function( ){
          if ( !angular.isDefined( _connector ) ){
            var connectorName = 'CORSConnector';
            _connector =  $injector.get( connectorName );
          }
          return _connector;
        },

        /**
         * ping the AIP connector
         */
        ping=function( ){
          return getConnector( ).ping( ++requestId );
        },

        /**
         * used to  play targeting rules
         * @param jsonData
         */
        runWebboxRules=function (jsonData) {
          window.parent.akioTargetingRun(jsonData.webboxRule, function (stats) {
            // Save targeting rules into the box config
            state.data.save({
              targetingRules: {
                targetingConditionUuid: stats,
                display: true
              }
            });
            if (!session.fetchMediaIsRunning && !session.fetchEventsIsRunning) {
              session.startPolling();
            }
          }, function (stats) {
            if (!session.fetchEventsIsRunning && state.isLaunchBarMode()) {
              session.stopPolling();
              state.data.save({
                targetingRules: {
                  targetingConditionUuid: stats,
                  display: false
                }
              });
            } else if (!state.isLaunchBarMode()) {
              session.startPolling();
              state.data.save({
                targetingRules: {
                  targetingConditionUuid: stats,
                  display: true
                }
              });
            }
          });
        },

      /**
         * will get the i18n, data & css
         * process API answer : {"Configuration":{"webboxRule":{"uuid":"40e7419f-f62a-4937-a393-722d4e66f2eb","validation":"ALL","conditions":[{"uuid":"813a6a95-cf1b-4ef7-bc69-ef01162ea978","name":"URL-http://uicdev/tenant1/example/","script":"function(callback) { callback(document.URL == \"http://0.0.0.0:9000/\"); }"},{"uuid":"264a9991-4d32-44ed-a0d9-8c1ea17334ce","name":"Javascript-DOM","script":"function(callback) { callback(eval(\"document.getElementById(\\\"userSelectedLanguage\\\") != null;\")); }"},{"uuid":"6c4bed66-27db-4153-b358-a667c4b7d88a","name":"Temps-10","script":"function(callback) { window.setTimeout(function() { callback(true); }, 10 * 1000); }"}],"name":"1"}}}
         *
         * send a request to AIP
         *
         * responses status
         * 201 @return header-location => token context
         * 401
         */
      getInitData = function(){
          var transformResponse=function (data) {
            try{
                var jsonData = angular.fromJson(data);
                if(jsonData) {
                  if (!state.isLaunchBarMode() && state.data.get("targetingRules").display) {
                    session.startPolling();
                    runWebboxRules(jsonData);
                  } else {
                    if(eval(boxParams.IS_DISPLAY_AS_BUTTON)) {
                      state.setDisplayAsButton(true);
                    } else {
                      state.setDisplayAsButton(false);
                    }
                    state.setCountriesList(jsonData.countries);
                    runWebboxRules(jsonData);
                  }
                }else {
                  throw new Error('No JSON data from AIP');
                }
              } catch(err) {
              $log.error(err.message);
            }
            return data;
          };

          return getConnector().getInitData(++requestId).then(function(result) {
            return transformResponse( result );
          });
        },


        /**
         * Get media availability
         * process API answer : {"Medium":{"uuid":"2","estimatedWaitTime":0,"status":"OPEN_AVAILABLE","name":"Chat","type":"CHAT"}}
         *
         * issue a request to
         * GET /chat/availability/
         *
         * responses status
         * 201 @return chat availability
         * 401
         */
        getMedia=function(){
          var transformResponse=function (data) {
            try{

              var mediaList =[];

              var response = angular.fromJson(eval(data));

              for(var i=0;i<response.length;i++){
                mediaList.push(response[i].media);
                mediaList[i].conditionUuid = response[i].conditionUuid;
              }


              var mediumObject = {media :  _.map(mediaList, function(medium) { return _.assign(medium, media.getMediaExtrasDescription(medium.type) || {}); })};
              return mediumObject;


            }catch(err){
              return data;
            }
          };
          return getConnector( )
            .getMedia( ++requestId )
            .then ( function( response ){
              return transformResponse( response, [] );
            });
        },


        /**
         * FETCH EVENTS and MEDIA LIST
         * @response {"MediasAndEvents":{"events":{"event":[]},"medias":{"media":[{"type":"CHAT","status":"OPEN_AVAILABLE","uuid":"2"}]}}}
         *
         * {"MediumAndEvents":{"events":{"event":[{"type":"OpenMediumResultEvent","status":"ALERTING","eventId":1,"mediumUuid":"2"},
         * {"type":"OpenMediumResultEvent","status":"OK","sender":{"nickName":"Segard","userType":"OPERATOR"},"eventId":2,"mediumUuid":"2"},
         * {"type":"MessageEvent","messageId":0,"message":"Bonjour, quel est votre problème ? ","sender":{"nickName":"ese","userType":"OPERATOR"},"eventId":3,"mediumUuid":"2"}]},
         * "medium":{"uuid":"2","estimatedWaitTime":0,"status":"OPEN_AVAILABLE","name":"Chat","type":"CHAT"}}}
         *
         * issue a request to
         * GET /sessions/:sessionUuid/events
         *
         * responses status
         * 200 204 - method successfully executed but no event found -
         * 400 401 403
         *
         * fetch
         *  - the list of events whom id is higher than *lastEventId*
         *  - the list of medias by token of context
         */
        fetch=function ( lastEventId ){

          /**
           * debug for sessionUuid
           *
           */
          window.Akio.Webbox.SessionUuid =
            {
              'getSessionUuid':function(){var uuid =  state.get( ).get( 'sessionUri' ); uuid= uuid.split('/chat/');return uuid[1];}
            }
          ;

          /**
           * format link
           * @param text
           * @returns {*}
           */
          var formatLink=function( text ){

              var message  = xmlUtils.surroundAndXMLParse( text ),
                aElements= message.getElementsByTagName("a"),
                href, a, i;

              for ( i = aElements.length-1; i >= 0; --i ) { // start from end since this loop may remove elements
                a = aElements[i];
                href = a.getAttribute("href");
                if ( href ) {
                  if ( xmlUtils.hasClass(a, "nl-link") ) {
                    a.setAttribute("data-link", href);
                    a.setAttribute("href", ""); // after using "href"
                  } else if ( xmlUtils.hasClass(a, "event-link") ) {
                    a.setAttribute("data-link", href);
                    a.setAttribute("href", ""); // after using "href"
                  }
                }
                if ( !a.hasChildNodes() ) {
                  a.parentNode.removeChild(a);
                }
              }

              return  xmlUtils.xmlSerializeAndUnSurround( message );
            },

            /**
             * transform response
             * @param data
             * @returns {*}
             */
            transformResponse=function (data) {
              /* HACKS */
              try{

                var jsonData  = angular.fromJson(data),
                  medium = jsonData.medium,
                  events    = [],
                  openMediaResultEvent;
                medium.conditionUuid = state.getCurrentMediaCondition();

                /* events */
                _.map(jsonData.events, function(event){


                  if ( event.type === 'MessageEvent' ){
                    if ( !angular.isDefined( event.sender.userType ) ){
                      event.sender.userType = 'OPERATOR';
                    }

                    event.message = formatLink( event.message );
                  }

                  if ( event.type === 'MediumClosedEvent'){
                    if (event.mediumUuid === '4'){
                      event.sender.userType = 'WEBUSER';
                    }
                  }

                  events.push( event );

                  // OperatorJoinedEvent is not fired at the moment so we add it by hand if OpenMediaEventResult is detected and its status is 'OK'
                  if ( event.type === 'OpenMediumResultEvent' && event.status === 'OK' ){
                    openMediaResultEvent = _.clone( event );
                    events.push( _.assign( openMediaResultEvent, {type: 'OperatorJoinedEvent'} ) );
                  }

                  if ( (event.type === 'OpenMediumResultEvent' && event.status === 'DISSUADED')||(medium.status ==='CLOSED') ){
                    session.fetchEventsIsRunning= false;
                    session.fetchMediaIsRunning = true;
                    $log.info('Dissuaded');
                    session.fetchMedia();
                  }

                  return event;
                });
                if ( medium.status ==='CLOSED' ){
                  messageReader.injectEvent( ServiceUnavailableEvent );
                }

                return {
                  //media : _.sortBy( _.map(mediaList, function(medium) { return _.assign(medium, media.getMediaExtrasDescription(medium.type) || {}); }), _.property('uuid') ),
                  media : _.assign(medium, media.getMediaExtrasDescription(medium.type) || {}),
                  events: events
                };

              }catch(err){
                // response is invalid!
                return data;
              }
            };

          return getConnector( )
            .fetch( ++requestId, lastEventId )
            .then ( function( response ){
              return transformResponse( response, [] );
            });
        },

        fetchWcbState=function (){

          /**
           * transform response
           * @param data
           * @returns {*}
           */
          var transformResponse=function (data) {
            /* HACKS */
            try{

              var jsonData  = angular.fromJson(data),
                wcbState = jsonData.status;

              return wcbState;

            }catch(err){
              // response is invalid!
              return data;
            }
          };

          return getConnector( )
            .fetchWcbState()
            .then ( function( response ){
              return transformResponse(response);
            });
        },




        /**
         * open WCB
         */
        openMediaWcb=function(alphaCode,phoneNumber, callbackTime, callbackTimeEnd){
          //stop the Media fetch
          session.fetchMediaIsRunning = false;//stop the fetchMedia

          return getConnector( ).openMediaWcb(++requestId,alphaCode, phoneNumber, callbackTime, callbackTimeEnd)
            .then(function(sessionUri){
              if (angular.isDefined(sessionUri)){ /* new session */
                state.data.save({'sessionUri': boxParams.AIP_SERVICE_BASE_URL+'/webcallback/status/'+JSON.parse(sessionUri).sessionUuid});
              }
            })
            .then(function(){
              session.fetchEventsIsRunning = true;
            })
            .then(session.fetchWcbState);
        },

        /**
         * open WCBD
         */
        openMediaWcbd=function(alphaCode,phoneNumber, callbackTime, callbackTimeEnd){
          //stop the Media fetch
          session.fetchMediaIsRunning = false;//stop the fetchMedia

          return getConnector( ).openMediaWcb(++requestId,alphaCode, phoneNumber, callbackTime, callbackTimeEnd)
            .then(function(){
              session.fetchEventsIsRunning = true;
            });
        },

        /**
         * get WCB  calendar
         */
        getWebcallbackCalendar=function(fromDate){
          return getConnector().getWebcallbackCalendar(++requestId,fromDate);
        },


        /**
         * open a media
         */
        openMediaChat=function(  ){
          //stop the Media fetch
          session.fetchMediaIsRunning = false;

          return getConnector( ).openMediaChat( ++requestId )
            .then(function(sessionUri){
              if (angular.isDefined(sessionUri)){ /* new session */
                state.data.save({'sessionUri': sessionUri});
              }
            })
            .then(function(){
              session.fetchEventsIsRunning = true; //stop the fetchMedia
            })
            .then(session.fetch); // start the fetchEvent
        },



        /**
         * close a media
         */
        closeMediaChat=function(  ){
          return getConnector( )
            .closeMediaChat( ++requestId )
            .then(function(){
              //start the media fetch
              session.fetchMediaIsRunning = true;
            });
        },



        /**
         * send a message in natural language
         * No specific ERROR scenario to attached to.
         * @param text the text of th message
         * @return
         *  status: 204 (no content)
         *  see specification
         */
        sendMessage=function( text ){
          var textToSend = transformText.deleteSpanAndAddEmoticonCharacter(text);
          var id = state.getMessageId( );

          return getConnector( ).sendMessage( ++requestId, textToSend, id )
            .then(function(result){
              messageReader.injectMessage( 'WEBUSER', text );
              return result;
            });
        },

        /**
         * send an input event
         * No specific ERROR scenario attached to
         * @param name
         * @param value
         */
        sendInputEvent=function( name, value ){
          return getConnector( ).sendInputEvent( ++requestId, name, value );
        },

        /**
         * notifies the agent of an event on the box
         * - starts/stops typing message
         * @return
         *  status 204 ( no content )
         */
        notify=function( type, status ){
          getConnector( ).notify( ++requestId, type, status );
        },


        /**
         * try to get history
         *
         * if history available (200) open the history url in a new window to download the pdf
         * if history  unavailable alert the web user
         */
        downloadHistory=function(language){
          return getConnector().downloadHistory(++requestId,language)
            .then(function(result){
              if (result && !result.historyAvailable){
                window.alert(labels.HISTORY_NOT_AVAILABLE);
              }else{
                var isSafari = navigator.userAgent.indexOf("Safari") > -1;
                //we have two differents way to download history depending webbrowser
                if(isSafari){
                  var link=document.createElement('a');
                  link.href=state.getChatSessionUri()+"/history?language="+language;
                  link.download="Awb_" + new Date() + ".pdf";
                  link.click();
                }else{
                  window.open(state.getChatSessionUri()+"/history?language="+language);
                }
              }
            });
        },

        /**
         * send FormData
         * @param formData
         */
        send=function( formData ){
          $log.info('send form data', formData);
        };

      return {
        ping                    : ping, /* NOT PART OF AIP api */
        getInitData             : getInitData,
        getMedia                : getMedia,
        fetch                   : fetch,
        openMediaChat           : openMediaChat,
        closeMediaChat          : closeMediaChat,
        sendMessage             : sendMessage,
        sendInputEvent          : sendInputEvent,
        notify                  : notify,
        openMediaWcb            : openMediaWcb,
        openMediaWcbd            : openMediaWcbd,
        getWebcallbackCalendar  : getWebcallbackCalendar,
        fetchWcbState           : fetchWcbState,
        send                    : send,
        downloadHistory         : downloadHistory/* sending form data */
      };
    }
  ]);
