'use strict';

/**
 * the AIP API
 */
angular.module('akio.webbox')

  .factory('CORSConnector', [ 'state', 'boxParams', '$q', '$filter', 'proxy', '$log', '$window', 'determineClient',
    function( state, boxParams, $q, $filter, proxy, $log, $window, determineClient) {

      /**
       * ping
       * @returns {*}
       */
      var ping = function( requestId ){
          var deferred = $q.defer();

          proxy.ping( requestId,
            function success( response ){
              deferred.resolve( response.data );
            }
          );
          return deferred.promise;
        },

        /**
         *
         * @param requestId
         * @returns {*}
         */


        getInitData = function (requestId){
          var deferred = $q.defer();
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;
          proxy.getInitData( requestId,
            boxParams.AIP_SERVICE_BASE_URL,
            boxParams.TOKEN_AUTHENTICATION,
            boxParams.SERVER_URL,
            isSafari,
            originUrl,
            boxParams.COMMUNICATION_ERROR_MESSAGE,
            function success( response ){
              deferred.resolve( response.data );
            },
            function error( rejection ){
              deferred.reject( rejection );
            }
          );
          return deferred.promise;
        },

        /**
         *
         * @param requestId
         * @returns {deferred.promise|{then, always}|{then, catch, finally}}
         */
        getMedia = function(requestId){
          var targetingConditionUuid = state.get().get('targetingRules').targetingConditionUuid;
          var deferred = $q.defer();
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;
          proxy.getMedia( requestId,
            boxParams.AIP_SERVICE_BASE_URL,
            boxParams.TOKEN_AUTHENTICATION,
            boxParams.SERVER_URL,
            isSafari,
            originUrl,
            boxParams.COMMUNICATION_ERROR_MESSAGE,
            targetingConditionUuid,
            function success( response ){
              deferred.resolve( response.data );
            },
            function error( rejection ){
              deferred.reject( rejection );
            },
            false
          );

          return deferred.promise;

        },



        /**
         * FETCH EVENTS and MEDIA LIST
         * @response {"MediasAndEvents":{"events":{"event":[]},"medias":{"media":[{"type":"CHAT","status":"OPEN_AVAILABLE","uuid":"2"}]}}}
         *
         * issue a request to
         * GET /sessions/:sessionUuid/events
         *
         * responses status
         * 200 204 - method successfully executed but no event found -
         * 400 401 403
         *
         * fetch
         *  - the list of events whom id is higher than *lastEventId*
         *  - the list of medias by token of context
         */
        fetch=function( requestId, lastEventId ){
          var deferred = $q.defer();
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;
          var sessionUri = state.getChatSessionUri();
          if(sessionUri !== null){
            proxy.fetch( requestId,
              sessionUri,
              boxParams.TOKEN_AUTHENTICATION,
              boxParams.SERVER_URL,
              lastEventId,
              isSafari,
              originUrl,
              function success( response ){
                deferred.resolve( response.data );
              },
              function error( rejection ){
                deferred.reject( rejection );
              }
            );
            return deferred.promise;
          } else {
            return $q.reject('no session uuid found');
          }

        },


        fetchWcbState=function(){
          var deferred = $q.defer();
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;

          var sessionUri = state.get( ).get( 'sessionUri' );

          proxy.fetchWcbState(
            sessionUri,
            boxParams.TOKEN_AUTHENTICATION,
            boxParams.SERVER_URL,
            isSafari,
            originUrl,
            function success( response ){
              deferred.resolve( response.data );
            },
            function error( rejection ){
              deferred.reject( rejection );
            }
          );
          return deferred.promise;
        },

        /**
         * OPENS a medium
         * @param requestId
         */
        openMediaWcb=function( requestId, alphaCode, phoneNumber, callbackTime, callbackTimeEnd ){
          var deferred = $q.defer( );
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;
          var targetingConditionUuid = state.getCurrentMediaCondition();




          proxy.openMediaWcb( requestId,
            boxParams.AIP_SERVICE_BASE_URL,
            boxParams.TOKEN_AUTHENTICATION,
            boxParams.SERVER_URL,
            isSafari,
            originUrl,
            $filter( 'nameValuePairs' )( $window.parent.profileParameters ),
            targetingConditionUuid,
            alphaCode,
            phoneNumber,
            callbackTime,
            callbackTimeEnd,

            function success( response ){
              deferred.resolve( response.data );
            },
            function error( rejection ){
              deferred.reject( rejection );
            }
          );

          return deferred.promise;
        },

        /**
         * Get the webcallback calendar
         * @param requestId
         */
        getWebcallbackCalendar=function( requestId, fromDate ){
          var deferred = $q.defer( );
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;
          var targetingConditionUuid = state.getCurrentMediaCondition();

          proxy.getWebcallbackCalendar( requestId,
            boxParams.AIP_SERVICE_BASE_URL,
            boxParams.TOKEN_AUTHENTICATION,
            boxParams.SERVER_URL,
            isSafari,
            originUrl,
            targetingConditionUuid,
            fromDate,

            function success( response ){
              deferred.resolve( response.data );
            },
            function error( rejection ){
              deferred.reject( rejection );
            }
          );

          return deferred.promise;
        },


        /**
         * OPENS a medium
         * @param requestId
         */
        openMediaChat=function( requestId ){
          var deferred = $q.defer( );
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;
          var targetingConditionUuid = state.getCurrentMediaCondition();

          proxy.openMediaChat( requestId,
            boxParams.AIP_SERVICE_BASE_URL,
            boxParams.TOKEN_AUTHENTICATION,
            boxParams.SERVER_URL,
            isSafari,
            originUrl,
            $filter( 'nameValuePairs' )( $window.parent.profileParameters ),
            targetingConditionUuid,
            function success( response ){
              deferred.resolve( response.data );
            },
            function error( rejection ){
              deferred.reject( rejection );
            }
          );

          return deferred.promise;
        },

        /**
         * CLOSES a medium
         * @param requestId
         */
        closeMediaChat=function( requestId ){
          var deferred = $q.defer( );
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;

          var sessionUri = state.get( ).get( 'sessionUri' );
          proxy.closeMediaChat( requestId,
            sessionUri,
            boxParams.TOKEN_AUTHENTICATION,
            boxParams.SERVER_URL,
            isSafari,
            originUrl,
            function success( response ){
              deferred.resolve( response.data );

            },
            function error( rejection ){
              deferred.reject( rejection );
            }
          );

          return deferred.promise;
        },

        /**
         * download history
         *
         * @param requestId
         * @param language
         * @returns {*|promise}
         */
        downloadHistory=function(requestId, language) {
          var deferred = $q.defer( );
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;
          var sessionUri = state.getChatSessionUri();
          proxy.downloadHistory(
            requestId,
            language,
            sessionUri,
            isSafari,
            originUrl,
            function success( response){
              deferred.resolve( response.data);

            });
          return deferred.promise;
        },



        sendInputEvent=function( requestId, name, value ){
          var deferred = $q.defer( );
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;

          var sessionUri = state.get( ).get( 'sessionUri' );

          proxy.sendInputEvent( requestId,
            sessionUri,
            boxParams.TOKEN_AUTHENTICATION,
            boxParams.SERVER_URL,
            isSafari,
            originUrl,
            name,
            value,
            function success( response ){
              deferred.resolve( response.data );
            },
            function error( rejection ){
              deferred.reject( rejection );
            }
          );

          return deferred.promise;
        },

        /**
         * SENDS a message in natural language
         * @param requestId
         * @param text
         * @param id
         */
        sendMessage=function( requestId, text, id ){
          var deferred = $q.defer( );
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;

          var sessionUri = state.get( ).get( 'sessionUri' );

          proxy.sendMessage( requestId,
            sessionUri,
            boxParams.TOKEN_AUTHENTICATION,
            boxParams.SERVER_URL,
            isSafari,
            originUrl,
            text,
            id,
            function success( response ){
              deferred.resolve( response.data );
            },
            function error( rejection ){
              deferred.reject( rejection );
            }
          );

          return deferred.promise;
        },

        /**
         * NOTIFIES the agent of an some kind event
         *
         * @param requestId
         * @param type      the action type
         * @param status    the action status
         * @returns {$promise|*|A.$promise}
         */
        notify=function( requestId, type, status ){
          var deferred = $q.defer( );
          var isSafari = navigator.userAgent.indexOf("Safari") > -1;
          var originUrl = determineClient.url.href;

          var sessionUri = state.get( ).get( 'sessionUri' );

          switch (type){
            case 'typing':
              if (status){
                proxy.userStartTyping( requestId,
                  sessionUri,
                  boxParams.TOKEN_AUTHENTICATION,
                  boxParams.SERVER_URL,
                  isSafari,
                  originUrl,
                  function success( response ){
                    deferred.resolve( response.data );
                  },
                  function error( rejection ){
                    deferred.reject( rejection );
                  }
                );
              }else{
                proxy.userStopTyping( requestId,
                  sessionUri,
                  boxParams.TOKEN_AUTHENTICATION,
                  boxParams.SERVER_URL,
                  isSafari,
                  originUrl,
                  function success( response ){
                    deferred.resolve( response.data );
                  },
                  function error( rejection ){
                    deferred.reject( rejection );
                  }
                );
              }
              break;
          }

          return deferred.promise;
        };

      return {
        ping                    : ping,
        closeMediaChat          : closeMediaChat,
        getInitData             : getInitData,
        getMedia                : getMedia,
        fetch                   : fetch,
        notify                  : notify,
        openMediaWcb            : openMediaWcb,
        getWebcallbackCalendar  : getWebcallbackCalendar,
        openMediaChat           : openMediaChat,
        sendMessage             : sendMessage,
        sendInputEvent          : sendInputEvent,
        downloadHistory         : downloadHistory,
        fetchWcbState           : fetchWcbState
        //send                : send /* sending form data */
      };

    }
  ]);
