'use strict';

angular.module('akio.webbox')
  .factory('Event', ['Backbone',
    function(Backbone){

      return Backbone.Model.extend({
        defaults:{
          eventId    : undefined,
          type       : undefined, //  MESSAGE | _ASSISTANT_MESSAGE | INTERNAUT_INACTIVITY | USER_MESSAGE | _COB_SESSION_BEGIN | _COB_SESSION_END | _COB_CTRL_DENIED | _COB_CTRL_STARTED | _COB_CTRL_STOPPED | _COB_SESSION_ACCEPTED | _COB_SESSION_REJECTED | _COB_CTRL_REQUEST_ACCEPTED | _COB_CTRL_REQUEST_REJECTED | _COB_SESSION_INIT_FAILURE | _COB_NOT_ACTIVATED | _COB_ACTIVATED
          message    : '',        // MessageEvent ONLY
          sender     : undefined, // { nickName : xx, userType: xxx }
          logoutDelay: undefined  // inactivityEvent ONLY
        },

        isMessage: function(){
          return this.get('type') === "MessageEvent";
        },

        isLogMessage: function(){
          return this.get('type') === "LogMessageEvent";
        },

        // isInfoMessage: function(){
        //   return this.get('type') === "InfoMessageEvent";
        // },

        isUserType: function(){
          return (this.get("sender") || {}).userType === "WEBUSER";
        },

        isAssistantType: function(){
          return (this.get("sender") || {}).userType === "VIRTUAL_ASSISTANT";
        },

        isOperatorType: function(){
          return (this.get("sender") || {}).userType === "OPERATOR";
        },

        isSystemType: function(){
          return (this.get("sender") || {}).userType === "SYSTEM";
        },

        isSurvey: function(){
          return this.get("type") === "SurveyEvent";
        },

        isFormSent: function(){
          return this.get("original") && this.get("original").type === "FormSubmitEvent";
        },

        isInactivityEvent: function(){
          return this.get("original") && this.get("original").type === "ConversationInactivityEvent";
        },

        isChatInvitation: function(){
          return this.get("type") === "ChatInvitationEvent";
        },

        isNetConnectionLazy: function(){
          return this.get("type") === "NetworkConnectionLazyEvent";
        },

        isServiceUnavailable: function(){
          return this.get("type") === "ServiceUnavailableEvent";
        },

        isInvitationAccepted: function(){
          return this.get("original") && this.get("original").accepted === true;
        },

        isInvitationRejected: function(){
          return this.get("original") && this.get("original").accepted === false;
        },

        isTypingStartedEvent: function(){
          return this.get("original") && this.get("original").type === "TypingStartedEvent";
        },

        isTypingStoppedEvent: function(){
          return this.get("original") && this.get("original").type === "TypingStoppedEvent";
        },

        isConversationRequalificationEvent:function(){
          return this.get("original") && this.get("original").type === "ConversationRequalificationEvent";
        },

        isDissuadedEvent:function(){
          return this.get("type") === "OpenMediumResultEvent" && this.get("status") === "DISSUADED";
        },

        isDissuadedMessage:function(){
          return this.get("original") && this.get("original").type === "OpenMediumResultEvent" && this.get("original").status === "DISSUADED";
        },

        /**
         * the event is a form event
         * @returns {boolean}
         */
        isForm: function(){
          return this.get("type") === "FormRequestEvent";
        },

        isLogoutEvent: function(){
          return this.get("original") && this.get("original").type === "MediumClosedEvent";
        },

        isOperatorJoinedEvent: function(){
          return this.get("original") && this.get("original").type === "OperatorJoinedEvent";
        },

        isOpenMediumResultEvent: function(){
          return this.get("type") === "OpenMediumResultEvent";
        },

        isOperatorLeftEvent: function(){
          return this.get("original") && this.get("original").type === "OperatorLeftEvent";
        },

        isMediumClosedEvent: function(){
          return this.get("original") && this.get("original").type === "MediumClosedEvent";
        }

        /**
         * TODO: implements it
         * @returns {boolean}
         */
        // isInvitation: function(){
        //   return true;
        // }
      });
    }
  ])

  .factory('EventCollection', ['Backbone', 'Event',
    function( Backbone, Event ) {
      return Backbone.Collection.extend({
        model: Event,

        /**
         * returns the list of messages for the active media
         * @returns {Array}
         */
        getMediumEvents: function( ){
          var index = this.models.length,
            event = this.models[ index-1 ],
            count     = 0;

          while ( index-- ) {
            count++;
            if ( event.isOpenMediumResultEvent( ) ){
              break;
            }
            event = this.models[ index-1 ];
          }

          return _.last( this.models, count );
        },

        /**
         * tells whether this the first message received
         * @returns {boolean}
         */
        isFirstIncomingMessage: function( ){
          var count = 0;
          _.forEachRight(this.models, function(event){
            if ( event.isMessage( ) ){
              count++;
            }
            return count === 0 && !event.isOpenMediumResultEvent( );
          });

          return count === 0;
        }
      });
    }
  ]);
