"use strict";

var /*xmlHttp, */result,

  /**
   *
   * @param err
   */
  onErrorHandler = function( err ){
    console.error( 'xmlHttp --> onErrorhandler', err );
  },


/**
 * ECHOES the request
 * this method is not part of AIP
 * it is used to check that the AIP connector is ready in a CWM mode
 */
/* exported ping */
  ping=function( requestId, success ){
    ( success || result )( {
      id: requestId,
      success: true,
      method : 'ping'
    } );
  },

/**
 * returns the box version
 * version @@boxVersion is dynamically replaced during the grunt build
 */
/* exported box version */
  getBoxVersion=function(){
    return "@@project.version";
  },

/**
 * send a Request to AIP with authentication Token and URL to get box's configuration and target rule
 *
 * issue a request to
 * GET /webbox/configuration
 *
 * responses status
 * 201 @return webbox configuration and target rules
 * 401
 *
 */
/* exported getInitData */
  getInitData=function( requestId, serviceBaseUrl, tokenAuthentication, serverUrl, isSafari, originUrl, errorMessage, success, error ){
    var url  = serviceBaseUrl + '/webbox/configuration/';
    if(serverUrl !== "") {
      url += '?serverUrl='+serverUrl;
      if(isSafari) {
        url += "&url="+originUrl;
      }
    } else if(isSafari) {
      url += "?url="+originUrl;
    }
    // * communication the box version while first call fetching the configuration
    if(url.indexOf('/webbox/configuration/?') > -1) {
      url += "&boxVersion="+getBoxVersion();
    } else {
      url += "?boxVersion="+getBoxVersion();
    }
    var xmlHttp;
    if (window.XMLHttpRequest) {
      // If IE8 & 9
      xmlHttp = new XMLHttpRequest();
      xmlHttp.open( "GET", url );
      xmlHttp.withCredentials = true;
      xmlHttp.onerror = onErrorHandler;

      xmlHttp.setRequestHeader("Accept", "application/json; charset=utf-8");
      xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);

      xmlHttp.onreadystatechange = function(){
        if( xmlHttp.readyState === 4 /* DONE */ ) {
          xmlHttp.onreadystatechange = null;
          xmlHttp.onerror = null;
          var errorMessages = errorMessage;

          if ( xmlHttp.status === 200 || xmlHttp.status === 204) { /* success */
            ( success || result )( {
              id: requestId,
              success: true,
              method : 'getInitData',
              data   : xmlHttp.responseText
            } );

          } else if (errorMessages !== ""){
            if (xmlHttp.status === 404){ //not found
              ( error || result )( {
                id: requestId,
                success: false,
                method : 'getInitData',
                data   : errorMessages/*xmlHttp.statusText*/,
                code   : 0 /* CRITICAL */
              } );

            }else if (xmlHttp.status === 403){ //forbidden
              ( error || result )( {
                id: requestId,
                success: false,
                method : 'getInitData',
                data   : errorMessages/*xmlHttp.statusText*/,
                code   : 0 /* CRITICAL */
              } );

            }else if (xmlHttp.status === 503){ //forbidden
              ( error || result )( {
                id: requestId,
                success: false,
                method : 'getInitData',
                data   : errorMessages/*xmlHttp.statusText*/,
                code   : 0 /* CRITICAL */
              } );

            }else { /* error 401 */
              ( error || result )( {
                id: requestId,
                success: false,
                method : 'getInitData',
                data   : errorMessages,
                code   : 0 /* CRITICAL */
              } );
            }
          }
          xmlHttp = null;
        }
      };
      xmlHttp.send();
    }else{
      throw new Error( 'XMLHttpRequest is not supported!');
    }
  },

/**
 *send a Request to AIP with authentication Token and server URL to get media chat's availability
 *
 * issue a request to
 * GET /chat/availability/
 *
 * responses status
 * 201 @return chat availability
 * 401
 */
/* exported getMedia */
  getMedia = function(requestId, serviceBaseUrl, tokenAuthentication, serverUrl, isSafari, originUrl, errorMessage,targetingConditionUuid, success, error, retry){
    var conditionUuids='';

    for(var i=0;i<targetingConditionUuid.length;i++){
      if(i===0){
        conditionUuids+='&conditionUuids='+targetingConditionUuid[i].conditionUuid;

      }else{
        conditionUuids+=','+targetingConditionUuid[i].conditionUuid;
      }
    }

    var url = serviceBaseUrl + '/media/availability/?timestamp='+new Date().getTime()+conditionUuids;
    if(isSafari){
      url += "&url="+originUrl;
    }
    var errorMessages = errorMessage;
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open( "GET", url, true );
    xmlHttp.withCredentials = true;
    xmlHttp.onerror = onErrorHandler;
    xmlHttp.timeout = 3000;
    xmlHttp.setRequestHeader("Accept", "application/json; charset=utf-8");
    xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);
    xmlHttp.onreadystatechange = function(){
      if( xmlHttp.readyState === 4 /* DONE */ ) {
        xmlHttp.onreadystatechange = null;
        xmlHttp.onerror = null;

        if ( xmlHttp.status === 200 || xmlHttp.status === 204 /* no events & no media */ || xmlHttp.status === 1223 ) { /* success */
          (success || result)( {
            id: requestId,
            success: true,
            method : 'getMedia',
            data   : xmlHttp.responseText
          } );

        } else if(errorMessages !== ""){ /* error */
          var errCode;
          console.error( "-------getMediaChat.error", xmlHttp, xmlHttp.status );

          switch ( xmlHttp.status ){
            case 0: /* NETCONNECTION LAZY */
              errCode =  2 /* RESUME */;
              break;
            case 400 /* BAD REQUEST */:
            case 401 /* UNAUTHORIZED */:
            case 403 /* FORBIDDEN */:
              //$log.warn("@AWB - AIP.getMediaAndEvents --> HTTP request failed ( "+ rejection.status +" )" , rejection.data);
              errCode = 0 /* CRITICAL */;
              break;

            default:
              errCode = 0 /* CRITICAL */;
            //$log.warn("@AWB - AIP.getMediaAndEvents --> HTTP request failed. Unexpected status ( " + status + ")" );
          }

          // Case where there is no technical problem on XMLHttpRequest but request answer is error code. It avoid fail on node switching in a clustered environment.
          if (!retry) {
            getMedia(requestId, serviceBaseUrl, tokenAuthentication, serverUrl, isSafari, originUrl, errorMessage,targetingConditionUuid, success, error, true);
          } else {
            (error || result)( {
              id: requestId,
              success: false,
              method : 'getMedia',
              data   : errorMessages,
              code   : errCode
            } );
          }
        }
        xmlHttp = null;
      }
    };
    xmlHttp.send( null );
  },


/**
 * FETCH EVENTS and MEDIA LIST
 * @response {"MediasAndEvents":{"events":{"event":[]},"medias":{"media":[{"type":"CHAT","status":"OPEN_AVAILABLE","uuid":"2"}]}}}
 *
 * issue a request to
 * GET /chat/:sessionUuid/events
 *
 * responses status
 * 200 204 - method successfully executed but no event found -
 * 400 401 403
 *
 * fetch
 *  - the list of events whom id is higher than *lastEventId*
 *  - the list of medias by token of context
 */
/* exported fetch */
  fetch=function( requestId, sessionUri, tokenAuthentication, serverUrl, lastEventId, isSafari, originUrl, success, error, retry ){
    var url = sessionUri+ '/events/?timestamp='+new Date().getTime()+'&lastEventId=' + (lastEventId || 0);
    if(isSafari){
      url += "&url="+originUrl;
    }
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open( "GET", url );
    xmlHttp.withCredentials = true;
    xmlHttp.onerror = onErrorHandler;
    xmlHttp.timeout = 3000;
    xmlHttp.setRequestHeader("Accept", "application/json; charset=utf-8");
    xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);

    xmlHttp.onreadystatechange = function(){
      if( xmlHttp.readyState === 4 /* DONE */ ) {
        xmlHttp.onreadystatechange = null;
        xmlHttp.onerror = null;
        if ( xmlHttp.status === 200 || xmlHttp.status === 204 /* no events & no media */ || xmlHttp.status === 1223) { /* success */
          (success || result)( {
            id: requestId,
            success: true,
            method : 'fetch',
            data   : xmlHttp.responseText
          } );

        } else { /* error */
          var errCode;
          switch ( xmlHttp.status ){
            case 12029:
            case 12007:
            case 0: /* NETCONNECTION LAZY */
            case 404 :
            case 503 :
            case 500 :
              errCode =  2 /* RESUME */;
              break;

            case 400 /* BAD REQUEST */:
            case 401 /* UNAUTHORIZED */:
            case 403 /* FORBIDDEN */:
              //$log.warn("@AWB - AIP.getMediaAndEvents --> HTTP request failed ( "+ rejection.status +" )" , rejection.data);
              errCode = 0 /* CRITICAL */;
              break;

            default:
              errCode = 0 /* CRITICAL */;
            //$log.warn("@AWB - AIP.getMediaAndEvents --> HTTP request failed. Unexpected status ( " + status + ")" );
          }

          // We allow one retry on request error, to avoid fail on node switching in a clustered environment.
          if (!retry) {
            fetch(requestId, sessionUri, tokenAuthentication, serverUrl, lastEventId, isSafari, originUrl, success, error, true);
          } else {
            (error || result)( {
              id: requestId,
              success: false,
              method : 'fetch',
              data   : xmlHttp.statusText,
              code   : errCode
            } );
          }
        }
        xmlHttp = null;
      }
    };
    xmlHttp.send( null );
  },


/* exported fetchWcbState */
  fetchWcbState=function(  sessionUri, tokenAuthentication, serverUrl,  isSafari, originUrl, success, error ){
    var url = sessionUri+ '/?timestamp='+new Date().getTime();

    if(isSafari){
      url += "&url="+originUrl;
    }
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open( "GET", url );
    xmlHttp.withCredentials = true;
    xmlHttp.onerror = onErrorHandler;
    xmlHttp.timeout = 3000;
    xmlHttp.setRequestHeader("Accept", "application/json; charset=utf-8");
    xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);

    xmlHttp.onreadystatechange = function(){
      if( xmlHttp.readyState === 4 /* DONE */ ) {
        xmlHttp.onreadystatechange = null;
        xmlHttp.onerror = null;

        if ( xmlHttp.status === 200 || xmlHttp.status === 204 /* no events & no media */ || xmlHttp.status === 1223) { /* success */
          (success || result)( {
            success: true,
            method : 'fetchWcbState',
            data   : xmlHttp.responseText
          } );

        } else { /* error */
          var errCode;
          switch ( xmlHttp.status ){
            case 12029:
            case 12007:
            case 503 :
            case 500 :
              //errCode =  2 /* RESUME */;
              //break;
            case 0: /* NETCONNECTION LAZY */

            case 400 /* BAD REQUEST */:
            case 401 /* UNAUTHORIZED */:
            case 403 /* FORBIDDEN */:
              errCode = 6 /* CRITICAL */;
              break;

            default:
              errCode = 0 /* CRITICAL */;
          }

          (error || result)( {
            success: false,
            method : 'fetchWcbState',
            data   : xmlHttp.statusText,
            code   : errCode
          } );
        }
        xmlHttp = null;
      }
    };
    xmlHttp.send( null );
  },


/* exported openMediaWcb */
  openMediaWcb=function(requestId, serviceBaseUrl, tokenAuthentication, serverUrl, isSafari, originUrl, profileParameters, targetingConditionUuid, alphaCode, phoneNumber, callbackTime, callbackTimeEnd, success, error){
    var url = serviceBaseUrl + '/webcallback';
    if(isSafari){
      url += '?url='+originUrl;
    }


    if(targetingConditionUuid===''){
      targetingConditionUuid=null;
    }
    var payload =JSON.stringify( {
      countryCode : alphaCode,
      phoneNumber:phoneNumber,
      callbackTime:callbackTime,
      callbackTimeEnd: callbackTimeEnd,
      profileParameters:profileParameters,
      targetingConditionUuid:targetingConditionUuid
    } );

    var xmlHttp;

    if (window.XMLHttpRequest) {
      // If IE8 & 9
      xmlHttp = new XMLHttpRequest();
      xmlHttp.open( "POST", url );
      xmlHttp.withCredentials = true;
      xmlHttp.onerror = onErrorHandler;
      xmlHttp.setRequestHeader("Content-Type", "application/json");
      xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);
      xmlHttp.onreadystatechange = function(){
        if( xmlHttp.readyState === 4  ) {

          xmlHttp.onreadystatechange = null;
          xmlHttp.onerror = null;

          if ( xmlHttp.status === 200 /*&& isSessionUriValid*/ ) {
            ( success || result )( {
              id: requestId,
              success: true,
              method : 'openMediaWcb',
              data   : xmlHttp.responseText//sessionUri
            } );

          } else {
            ( error || result )( {
              id: requestId,
              success: false,
              method : 'openMediaWcb',
              data   : xmlHttp,
              code   : 0
            } );
          }
          xmlHttp = null;
        }
      };
      xmlHttp.send( payload );
    }else{
      throw new Error( 'XMLHttpRequest is not supported!');
    }
  },

/* exported getWebcallbackCalendar */
  getWebcallbackCalendar=function(requestId, serviceBaseUrl, tokenAuthentication, serverUrl, isSafari, originUrl, targetingConditionUuid,fromDate,success, error){
    var url = serviceBaseUrl + '/webcallback/calendar';
    if(isSafari){
      url += '?url='+originUrl;
    }

    if(targetingConditionUuid===''){
      targetingConditionUuid=null;
    }
    var payload =JSON.stringify( {
      fromDate : fromDate,
      targetingConditionUuid:targetingConditionUuid
    } );

    var xmlHttp;

    if (window.XMLHttpRequest) {
      // If IE8 & 9
      xmlHttp = new XMLHttpRequest();
      xmlHttp.open( "POST", url );
      xmlHttp.withCredentials = true;
      xmlHttp.onerror = onErrorHandler;
      xmlHttp.setRequestHeader("Content-Type", "application/json");
      xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);
      xmlHttp.onreadystatechange = function(){
        if( xmlHttp.readyState === 4  ) {

          xmlHttp.onreadystatechange = null;
          xmlHttp.onerror = null;

          if ( xmlHttp.status === 200 /*&& isSessionUriValid*/ ) {
            ( success || result )( {
              id: requestId,
              success: true,
              method : 'getWebcallbackCalendar',
              data   : xmlHttp.responseText//sessionUri
            } );

          } else {
            ( error || result )( {
              id: requestId,
              success: false,
              method : 'getWebcallbackCalendar',
              data   : xmlHttp,
              code   : 0
            } );
          }
          xmlHttp = null;
        }
      };
      xmlHttp.send( payload );
    }else{
      throw new Error( 'XMLHttpRequest is not supported!');
    }
  },





/**
 * opens a chat
 *
 * issues a request to
 * POST v1/chat/
 *
 * responses status
 * 201 @return chat sessionUuid
 * 400 401 403
 */
/* exported openMediaChat */
  openMediaChat=function(requestId, serviceBaseUrl, tokenAuthentication, serverUrl, isSafari, originUrl, profileParameters, targetingConditionUuid,success, error){
    var url = serviceBaseUrl + '/chat';
    if(isSafari){
      url += '?url='+originUrl;
    }

    var payload =JSON.stringify( {
      profileParameters : profileParameters,
      targetingConditionUuid : targetingConditionUuid
    } );

    var xmlHttp;

    if (window.XMLHttpRequest) {
      // If IE8 & 9
      xmlHttp = new XMLHttpRequest();
      xmlHttp.open( "POST", url );
      xmlHttp.withCredentials = true;
      xmlHttp.onerror = onErrorHandler;
      xmlHttp.setRequestHeader("Content-Type", "application/json");
      xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);
      xmlHttp.onreadystatechange = function(){
        if( xmlHttp.readyState === 4  ) {
          var sessionUri = xmlHttp.getResponseHeader('Location'),
            isSessionUriValid = !( sessionUri === undefined || sessionUri === null || sessionUri === '' );
          xmlHttp.onreadystatechange = null;
          xmlHttp.onerror = null;

          if ( xmlHttp.status === 201 && isSessionUriValid ) {
            ( success || result )( {
              id: requestId,
              success: true,
              method : 'openMediaChat',
              data   : sessionUri
            } );

          } else {
            ( error || result )( {
              id: requestId,
              success: false,
              method : 'openMediaChat',
              data   : xmlHttp.statusText,
              code   : 0
            } );
          }
          xmlHttp = null;
        }
      };
      xmlHttp.send( payload );
    }else{
      throw new Error( 'XMLHttpRequest is not supported!');
    }
  },

/**
 * closes a medium
 *
 * issues a request to
 * DELETE v1/chat/sessionUuid
 *
 * responses status
 * 204 (no content)
 * 400 401 403
 */
/* exported closeMediaChat */
  closeMediaChat=function( requestId, sessionUri, tokenAuthentication, serverUrl, isSafari, originUrl, success, error ){
    var url = sessionUri;
    if(isSafari){
      url += "?url="+originUrl;
    }
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open( "DELETE", url );
    xmlHttp.withCredentials = true;
    xmlHttp.onerror = onErrorHandler;

    xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);

    xmlHttp.onreadystatechange = function() {
      if ( xmlHttp.readyState === 4 /* DONE */ ) {
        xmlHttp.onreadystatechange = null;
        xmlHttp.onerror = null;

        if ( xmlHttp.status === 204 || xmlHttp.status === 1223 ||  xmlHttp.status === 404 || xmlHttp.status === 0 ) { /* success */
          (success || result)({
            id: requestId,
            success: true,
            method: 'closeMediaChat',
            data: null
          });

        } else { /* error */
          var errCode;

          switch ( xmlHttp.status ){
            case 400 /* BAD REQUEST */:
              errCode = 1; /* RESTART */
              break;
            case 401 /* UNAUTHORIZED */:
            case 403 /* FORBIDDEN */:
              //$log.warn("@AWB - AIP.getMediaAndEvents --> HTTP request failed ( "+ rejection.status +" )" , rejection.data);
              errCode = 1 /* RESTART */;
              break;
          }

          (error || result)({
            id: requestId,
            success: false,
            method: 'closeMediaChat',
            data: xmlHttp.statusText,
            code: errCode
          });
        }
        xmlHttp = null;
      }
    };
    xmlHttp.send( null );
  },

/**
 * send an input event
 * No specific ERROR scenario attached to
 * @param requestId
 * @param sessionUri
 * @param tokenAuthentication
 * @param serverUrl
 * @param isSafari
 * @param originUrl
 * @param name
 * @param value
 * @param success
 * @param error
 */
/* exported sendInputEvent */
  sendInputEvent=function( requestId, sessionUri, tokenAuthentication, serverUrl, isSafari, originUrl, name, value, success, error ){
    var url = sessionUri + '/inputevents?authen_token='+tokenAuthentication+'&serverUrl='+serverUrl;
    if(isSafari){
      url += "&url="+originUrl;
    }
    var payload= {name: name, value:value};
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open("POST", url);
    xmlHttp.withCredentials = true;
    xmlHttp.onerror = onErrorHandler;

    /**
     * issue a request to
     * POST {sessionUri}/inputevents
     *
     * responses status
     * 204
     * 400 401 403
     *
     * send an input event
     */
    xmlHttp.setRequestHeader("Content-Type", "application/json");
    xmlHttp.onreadystatechange = function(){
      if( xmlHttp.readyState === 4 /* DONE */ ) {
        xmlHttp.onreadystatechange = null;
        xmlHttp.onerror = null;

        if ( xmlHttp.status === 204 || xmlHttp.status === 1223 ) { /* success */
          (success || result)( {
            id: requestId,
            success: true,
            method : 'sendInputEvent',
            data   : sessionUri
          } );

        } else {
          (error || result)( {
            id: requestId,
            success: false,
            method : 'sendInputEvent',
            data   : xmlHttp.statusText,
            code   : 5 /* WARNING */
          } );
        }
        xmlHttp = null;
      }
    };
    xmlHttp.send( payload );
  },

/**
 * send a message in natural language
 *
 * issue a request to
 * POST V1/chat/{sessionUuid}/messages/{messageId}
 *
 * responses status
 * 204
 * 400 401 403
 *
 * send a message in natural language
 */
/* exported sendMessage */
  sendMessage=function(requestId, sessionUri, tokenAuthentication, serverUrl, isSafari, originUrl, text, id, success, error){
    var url = sessionUri+'/messages/'+id;
    if (isSafari){
      url += "?url="+originUrl;
    }
    var payload= text;
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open("POST", url);
    xmlHttp.withCredentials = true;
    xmlHttp.onerror = onErrorHandler;

    xmlHttp.setRequestHeader("Content-Type", "text/html");
    xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);
    xmlHttp.onreadystatechange = function(){
      if( xmlHttp.readyState === 4 /* DONE */ ) {
        xmlHttp.onreadystatechange = null;
        xmlHttp.onerror = null;

        if ( xmlHttp.status === 204 || xmlHttp.status === 1223 ) { /* success */
          (success || result)( {
            id: requestId,
            success: true,
            method : 'sendMessage',
            data   : sessionUri
          } );

        } else {
          console.error( "aip.sendMessage.failure", xmlHttp.status, xmlHttp );
          (error || result)( {
            id: requestId,
            success: false,
            method : 'sendMessage',
            data   : xmlHttp.statusText,
            code   : 5 /* WARNING */
          } );
        }
        xmlHttp = null;
      }
    };
    xmlHttp.send( payload );
  },

/**
 * notifies agent when the IUser starts typing
 *
 * issues a request to
 * POST {sessionUri}/typing
 *
 * responses status
 * 201
 * 400 401 403
 *
 * send a message in natural language
 */
/* exported userStartTyping */
  userStartTyping=function( requestId, sessionUri, tokenAuthentication, serverUrl, isSafari, originUrl, success, error ){
    var url = sessionUri + '/typing';
    if(isSafari){
      url += '?url='+originUrl;
    }
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open("POST", url);
    xmlHttp.withCredentials = true;
    xmlHttp.onerror = onErrorHandler;

    xmlHttp.setRequestHeader("Content-Type", "application/json");
    xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);
    xmlHttp.onreadystatechange = function(){
      if( xmlHttp.readyState === 4 /* DONE */ ) {
        xmlHttp.onreadystatechange = null;
        xmlHttp.onerror = null;

        if ( xmlHttp.status === 201 ) { /* success */
          (success || result)( {
            id: requestId,
            success: true,
            method : 'userStartTyping',
            data   : sessionUri
          } );

        } else if ( xmlHttp.status === 204 ) { /* success */
          (success || result)({
            id: requestId,
            success: true,
            method : 'userStartTyping',
            data   : sessionUri
          });

        }
        else {
          (error || result)( {
            id: requestId,
            success: false,
            method : 'userStartTyping',
            data   : xmlHttp.statusText,
            code   : 4 /* INFORMATION */
          } );
        }
        xmlHttp = null;
      }
    };
    xmlHttp.send( null );
  },

/**
 * notifies agent when the IUser stops typing
 *
 * issues a request to
 * DELETE {sessionUri}/typing
 *
 * responses status
 * 204
 * 400 401 403
 *
 * send a message in natural language
 */
/* exported userStopTyping */
  userStopTyping=function( requestId, sessionUri,tokenAuthentication, serverUrl,isSafari,originUrl, success, error ){
    var url = sessionUri + '/typing';
    if(isSafari){
      url += '?url='+originUrl;
    }
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open("DELETE", url);
    xmlHttp.withCredentials = true;
    xmlHttp.onerror = onErrorHandler;

    xmlHttp.setRequestHeader("Auth-Token", tokenAuthentication);
    xmlHttp.onreadystatechange = function(){
      if( xmlHttp.readyState === 4 /* DONE */ ) {
        xmlHttp.onreadystatechange = null;
        xmlHttp.onerror = null;

        if ( xmlHttp.status === 204 || xmlHttp.status === 1223 ) { /* success */
          (success || result)( {
            id: requestId,
            success: true,
            method : 'userStopTyping',
            data   : null
          } );

        } else {
          (error || result)( {
            id: requestId,
            success: false,
            method : 'userStopTyping',
            data   : xmlHttp.statusText,
            code   : 4 /* INFORMATION */
          } );
        }
        xmlHttp = null;
      }
    };
    xmlHttp.send( null );
  },

/**
 * get the history and return the history availability
 *
 * response status 200 => history available
 */


/* exported downloadHistory */
  downloadHistory=function(  requestId, language, sessionUri,  isSafari, originUrl ,success){
    var url = sessionUri + '/history';
    if(isSafari){
      url += '?url='+originUrl;
    }
    var xmlHttp = new XMLHttpRequest();
    xmlHttp.open("get", url);
    xmlHttp.withCredentials = true;
    xmlHttp.onerror = onErrorHandler;
    xmlHttp.setRequestHeader("Accept","application/pdf");
    xmlHttp.setRequestHeader("Accept-Language",language);

    xmlHttp.onreadystatechange = function(){
      if( xmlHttp.readyState === 4 /* DONE */ ) {
        xmlHttp.onreadystatechange = null;

        if ( xmlHttp.status === 200 || xmlHttp.status === 0) {
          (success || result)( {
            id:requestId,
            success: true,
            method : 'downloadHistory',
            data   : {historyAvailable : true}
          } );

        } else {
          (success || result)( {/* success but historic no more available*/
            id:requestId,
            success: true,
            method : 'downloadHistory',
            data   : {historyAvailable : false}
          } );
        }
        xmlHttp = null;
      }
    };
    xmlHttp.send( null );
  };
