'use strict';

/**
 * the AIP API
 */
angular.module('akio.webbox')

  .factory('messageReader', ['state', 'i18nMessages', 'navigation', 'Event', '$log', '$timeout', '$injector', '$q',
    function( state, labels, navigation, Event, $log, $timeout, $injector, $q ) {

      var firstMessageEvent       = true,
        OpenMediaResultEventStatus= $injector.get( 'OpenMediaResultEventStatus'),

        /**
         * Handle media list and events
         * @param result
         * @returns {Promise}
         */
        handleResponse=function( result ){

          // updates media list
          state.mediaList.set( result.media );

          // process each incoming event
          _.forEach( result.events, processEvent );

          return $q.when ( state.get( ).save( ) );
        },

        /**
         * @param accept {Boolean}
         * @param event
         */
        injectInvitationEvent=function(accept, event){
          var eventData = {
            type: "InvitationEvent",
            accepted: accept,
            subType: event.get('type'),
            mediaType: event.get('formData').type,
            "eventId": getLastEventId()
          };

          processEvent( eventData );
          state.get().save( );
        },

        /**
         * inject a message event by hand
         * @param {string} userType Possible values : 'WEBUSER' or 'ASSISTANT'
         * @param text
         * @param nickname
         */
        injectMessage=function( userType, text, nickname ) {

          var lastEventId = getLastEventId( ), //  copy the id of the last event in the list
            nick        = getNickname(userType, nickname),
            message     = {
              "type"   : "MessageEvent",
              "message": text,
              "sender" : {
                "userType": userType,
                "nickName": nick
              },
              "eventId": lastEventId,
              "mediumUuid": state.getCenterRegion( ['mediumUuid'] )
            };

          processEvent( message );
          state.get().save( );
        },

        injectLogMessage=function( text, mediumUuid, type ){
          var lastEventId = getLastEventId( ), //  copy the id of the last event in the list
            message     = {
              "type"   : "LogMessageEvent",
              "message": text,
              "original": {
                "type": type
              },
              //"sender" : {
              //    "userType": userType,
              //    "nickName": ''
              //},
              "eventId": lastEventId,
              "mediumUuid": mediumUuid
            };

          processEvent( message );
          state.get().save( );
        },

        /**
         * inject an Log message event by hand
         * @param event
         * @param text
         */
        wrapIntoLogMessage=function( event, text ){
          return {
            "type"     : "LogMessageEvent",
            "original" : event,
            "message"  : text,
            "sender"   : {userType: 'SYSTEM', nickName:''},
            "eventId"  : event.eventId,
            "mediumUuid": event.mediumUuid
          };
        },

        wrapIntoInfoMessage=function( event, text ){
          return {
            "type"     : "InfoMessageEvent",
            "original" : event,
            "message"  : text,
            "show"     : !_.isEmpty( text ),
            "sender"   : {userType: 'SYSTEM', nickName:''},
            "eventId"  : event.eventId,
            "mediumUuid": event.mediumUuid
          };
        },

        /**
         * inject an event - entirely described -
         * @param event
         */
        injectEvent=function( event ){
          _.assign( event, {eventId: getLastEventId()});

          $timeout(function(){
            processEvent( event );
            state.get().save( );
          }, 1000);
        },

        /**
         * @private
         */
        processEvent=function( eventData ){

          var eventType   = eventData.type,
            mediumUuid  = eventData.mediumUuid || state.getCenterRegion( ['mediumUuid'] ),
            medium      = state.mediaList.findWhere( {uuid: mediumUuid }),
            isSenderUser= eventData.sender && eventData.sender.userType === 'WEBUSER',
            message;

          switch ( eventType ){
            // @extends Event{eventId, mediumUuid, sender{name, type}}

            case 'OpenMediumResultEvent':

              switch ( eventData.status ){

                case OpenMediaResultEventStatus.OK: /* success */
                  if ( !medium.isChat( ) ){
                    navigation.enterMedia( mediumUuid );
                  }
                  break;

                case OpenMediaResultEventStatus.BAD_SERVICE:
                  // notify user
                  var notification = $injector.get( 'notification' );
                  notification.error('The media failed to open!');

                  navigation.quit( );
                  break;

                case OpenMediaResultEventStatus.DISSUADED: /* dissuasion */
                  message = labels.NOTIFICATION_OPEN_CHAT_RESULT_NO_AGENT;
                  eventData = wrapIntoLogMessage(eventData, message);
                  navigation.leaveMedia( mediumUuid );
                  break;

                default:
                  // all the others status are handled on the view
                  break;
              }
              break;

            case 'OperatorJoinedEvent':

              message = labels.CHATBOX_OPERATOR_JOIN;
              eventData = wrapIntoLogMessage( eventData, message );

              firstMessageEvent = true;

              break;

            case 'MessageEvent':
              // message
              firstMessageEvent = firstMessageEvent && state.events( ).isFirstIncomingMessage( );

              if ( medium.isChat( )  && firstMessageEvent ){
                navigation.enterMedia( mediumUuid );
              }

              break;

            case 'OperatorLeftEvent':
              message = labels.OPERATOR_TITLE + getNickname(eventData.sender.userType, eventData.sender.nickName) + labels.OPERATOR_ACTION_QUIT;
              eventData = wrapIntoLogMessage(eventData, message);
              break;

            case 'MediumClosedEvent':
              $log.debug('MediumClosedEvent.sender: ', eventData.sender.userType);

              message = (isSenderUser) ? labels.CHATBOX_USER_LOGOUT : labels.CHATBOX_OPERATOR_LOGOUT;

              eventData = wrapIntoLogMessage(eventData, message);

              // DO NOT USE FOR NOW
              navigation.leaveMedia( mediumUuid );
              break;

            case 'ConversationInactivityEvent':
              // logoutDelay

              message = _.template(labels.CHATBOX_INACTIVITY, {delay: eventData.logoutDelay});
              eventData = wrapIntoLogMessage(eventData, message);
              break;

            case 'TypingStartedEvent':
              message = _.template( labels.CHATBOX_IS_TYPING, {nickname: eventData.sender.nickName} );
              eventData = wrapIntoInfoMessage( eventData, message );
              break;

            case 'TypingStoppedEvent':
              eventData = wrapIntoLogMessage( eventData, '' );
              break;

            // généré localement
            case 'LogMessageEvent':
              break;

            case 'NetworkConnectionLazyEvent':
//                    eventData.message = labels.ERROR_NETWORK_CONNECTION_LAZY;
              navigation.interpretEvent( new Event( eventData ), state.events().length );
              break;

            case 'ServiceUnavailableEvent':
              navigation.interpretEvent( new Event( eventData ), state.events().length );
              break;
            case 'ConversationRequalificationEvent':

              message = labels.CHATBOX_REQUALIFICATION;
              eventData = wrapIntoInfoMessage(eventData, message);
              break;

            /* NOT IMPLEMENTED YET */

            case 'FormRequestEvent': // NEW MEDIA | STANDALONE | ACC DEPENDENCY
              // it can only occur after the IUser explicitly requests a form to fill out or during a CHAT session at the AGENT initiative
              // IT'S UP TO THE BOX TO MANAGE THE DISPLAY OF SEVERAL MEDIA AT THE SAME TIME

              navigation.interpretEvent( new Event( eventData ), state.events().length );
              break;

            case 'SurveyEvent':
              navigation.interpretEvent( new Event( eventData ), state.events().length );
              break;

            case 'ChatInvitationEvent':
              navigation.interpretEvent( new Event( eventData ), state.events().length );
              break;

            case 'InvitationEvent':
              // status: accept | reject
              // formData => type
              var accepted = eventData.accepted;

              message = _.template( (accepted) ? labels.FORM_ACCEPTED : labels.FORM_REJECTED, {formName:labels.TITLE_MEDIA_WCB});

              eventData = wrapIntoLogMessage(eventData, message);

              break;

            default:
              window.alert( 'event name is unknown:' + eventData.type );
          }

          if (angular.isDefined(eventData)){
            var isExcluded =  eventType === 'OpenMediumResultEvent' && eventData.status === 'ALERTING';

            if ( !isExcluded ){ // TRICKY
              state.events( ).add( new Event( eventData ) );
            }
          }
        },

        getLastEventId=function( ){
          var events   = state.events(),
            lastEvent= (events.at(events.length-1) || {}).attributes;

          return (lastEvent || {}).eventId || 0;
        },

        getNickname=function(userType, nickname){
          return angular.isDefined(nickname) ? nickname : ( (userType === 'WEBUSER') ? /*state.get().get('userNickname')*/labels.CHATBOX_DEFAULT_USER_NICKNAME :  labels.DEFAULT_ASSISTANT_NICKNAME );
        };

      return {
        handleResponse       : handleResponse,
        injectMessage        : injectMessage,
        injectEvent          : injectEvent,
        injectInvitationEvent: injectInvitationEvent,
        injectLogMessage     : injectLogMessage,
        wrapIntoLogMessage   :  wrapIntoLogMessage
      };
    }
  ]);
