'use strict';

/**
 * responsability:
 *  - handle the application's current state
 */
angular.module('akio.webbox')
  .factory('navigation', ['state', 'trigger', 'Layout', 'MediumViewStatus', '$log',
    function(state, trigger, Layout, MediumViewStatus, $log){

      var _setRegion = function(options){
        if (!state.setRegion(options)){
          throw state.data.validationError;
        }
      };

      return {
        description: undefined,

        start: function(){
          this.restoreTrigger();
        },

        /**
         * restore any pending TRIGGER
         */
        restoreTrigger: function(){
          var oState     = state.get(),
            eventIndex = Number( oState.get('trigger') ),
            event;

          if (!isNaN(eventIndex) && eventIndex>-1){
            event = oState.get('events').at(eventIndex);
            trigger.register(event, eventIndex);
          }else{
            $log.info('navigation.start: no trigger to restore');
          }
        },

        //---------------------
        // navigate to a state
        //---------------------

        /**
         * quit the application
         * if the box is display as button.
         */
        quit: function(){
          state.removeRegion(); // go back to launch bar mode
        },

        /**
         * navigate to the new screen
         * called ONLY from:
         *  - the home screen
         *  - the navigation bar
         */
        goto: function( medium ){
          _setRegion({
            mediumType :medium.attributes.type,
            mediumUuid :medium.attributes.uuid,
            constraint:Layout.BORDER_CENTER
          });
        },

        enterMedia: function( mediumUuid, data ){
          $log.info('enterMedia:', mediumUuid, data);

          // clear events
          state.setMediaStatus(mediumUuid, MediumViewStatus.MEDIUM_STATUS_OPENED);
        },

        leaveMedia: function( mediumUuid, constraint ){
          if ( angular.isDefined( constraint ) && constraint === Layout.BORDER_WEST ){
            state.removeRegion( constraint );
          }else{
            if ( !angular.isDefined( mediumUuid ) ){
              // if medium is not precised, we assume it's the one loaded in the center region
              mediumUuid = state.getCenterRegion( ['mediumUuid'] );
            }
            //yth 17/02/15: add a security here to not display ChatBOX if communication is aborted by user manytimes.
            if(state.getMediaStatus() === 0 || state.getMediaStatus() === 1 ||state.getMediaStatus() === 3 ) {
              state.setMediaStatus(mediumUuid, MediumViewStatus.MEDIUM_STATUS_CLOSED);
            }
          }

        },

        interpretEvent: function( event, eventIndex ){
          var centerMediaOpened = state.isMediumOpened();

          if ( ( !state.isLaunchBarMode() && !centerMediaOpened && !event.isSurvey() && !event.isChatInvitation() && !event.isNetConnectionLazy( )  && !event.isServiceUnavailable())  ||
            ( state.isLaunchBarMode( ) && event.isForm( ) ) )
          { /* the new media is loaded into the CENTER REGION (PRIMARY MEDIA) */
            this.setRegionByEvent( event,  eventIndex );
          }
          else
          { /* display the invitation. */
            trigger.register(event, eventIndex);
          }
        },

        setRegionByEvent: function(event, eventIndex){

          var centerRegion        = state.getCenterRegion(),
            isCenterMediaOpened = state.isMediumOpened(centerRegion),
            constraint          = (isCenterMediaOpened) ? Layout.BORDER_WEST : Layout.BORDER_CENTER,
            mediumType          = event.get('formData').type,
            mediumUuid          = '8';

          _setRegion({
            mediumType  : mediumType,
            mediumUuid  : mediumUuid,
            constraint  : constraint,
            data        : eventIndex,
            mediumStatus: MediumViewStatus.MEDIUM_STATUS_OPENED
          });
        },

        /**
         * navigate to the search results screen
         * @param {String} input text input
         * UNPLUGGED FOR NOW
         */
        search: function(input){
          $log.info('search for text:'+input);
            //_setRegion({mediumType: state.STATE_MEDIA_NLS, constraint: Layout.BORDER_CENTER});
        },

        isLocked: function(){
          return state.getMediaStatus() === MediumViewStatus.MEDIUM_STATUS_CLOSING; // || state.isTrigger();
        }
      };

    }]);
