'use strict';

/**
 * reads, validates and saves the configuration file
 * If the process failed, an error is fired
 */
angular.module('akio.webbox')

/**
 * settings files
 */
  .value('boxParams', {

    /**
     * delay to retry after a failure has been detected
     */
    "HTTP_RETRY_ON_FAILURE_DELAY" : 10000,

    "ERROR_POLLING":20,


    "IS_DISPLAY_AS_BUTTON":"false",

    "CTC_BUTTON":{
      "BUTTON_DEFAULT":"#ctcButton",
      "BUTTON_NOT_AVAILABLE":"#ctcButtonNotAvailable",
      "BUTTON_IS_CHATTING":"#ctcButtonIsChatting"
    },

    "WCB_BUTTON":{
      "BUTTON_DEFAULT":"#wcbButton",
      "BUTTON_NOT_AVAILABLE":"#wcbButtonNotAvailable",
      "BUTTON_IS_CALLING":"#wcbButtonIsCalling"
    },

    "I18N"  : "webBrowser",

    /*
     * delay between polls
     */
    "POLLING_DELAY_MEDIA": 5000,
    "POLLING_DELAY_EVENT": 2000,
    "POLLING_DELAY_LOW":60000,

    "WEB_USER_INACTIVITY_DELAY":60000,

    /**
     * generic rules
     */
    "RULES": {
      "NL_AGGREGATION_ENABLED"  : false,
      "MEDIA_PRIORITY_LIST"     : []
    },
    "HELP_BUBBLE_VISIBLE":true,
    "BOX_COUNTRIES_LIST":[],

    "EMOJI_VISIBLE":false,

    "ATTACHMENT_VISIBLE":false,

    "ASK_QUESTION_BEFORE_CHAT":false,

    "CHAT_FORM":[],

    "IS_DISPLAY_LAUNCHBAR_AS_BUBBLE":false,

    "DELAYED_WCB": false

  })


  .factory('configuration', [ '$q', 'boxParams', 'i18nMessages', '$filter', '$window', '$log',
    function($q, boxParams, messages, $filter, $window, $log){

      /**
       * validates configuration file
       * @param result
       * @returns {*}
       */
      var validate=function( result ) {

          var errors                      = [],
            tokenRegexp                 = /^[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/i,
            urlRegexp                   = /^https?\:\/\/[^\/\s]+(\/.*)?$/,
            parameters                  = result.parameters,
            isTokenAuthorizationKeyValid= tokenRegexp.test( parameters.TOKEN_AUTHENTICATION ),
            isAIPServiceBaseUrlValid    = urlRegexp.test( parameters.AIP_SERVICE_BASE_URL );

          //check Token
          if( !isTokenAuthorizationKeyValid ){
            errors.push( "«TOKEN_AUTHORIZATION» key format is invalid! UUID format is required" );
          }

          //check url
          if( !isAIPServiceBaseUrlValid ){
            errors.push( "«AIP_SERVICE_BASE_URL» key format is invalid!" );
          }

          //check form chat
          if(!angular.isDefined(parameters.CHAT_FORM)||parameters.CHAT_FORM === null || parameters.CHAT_FORM.length ===0){
            parameters.ASK_QUESTION_BEFORE_CHAT = false;
            console.warn( "«CHAT_FORM» is undefined or not valid!" );

          }else{
            var titleIsDefine = false;
            var buttonChatIsDefine = false;
            var buttonWcbIsDefine = false;
            for(var i=0;i<parameters.CHAT_FORM.length;i++){
              if(parameters.CHAT_FORM[i].type === "title" && parameters.CHAT_FORM[i].textKey.length>0){
                titleIsDefine=true;
              }
              if(parameters.CHAT_FORM[i].type === "button" && parameters.CHAT_FORM[i].textKey && parameters.CHAT_FORM[i].textKey.length>0){
                buttonChatIsDefine=true;
              }
              if(parameters.CHAT_FORM[i].type === "buttonWCB" && parameters.CHAT_FORM[i].textKey && parameters.CHAT_FORM[i].textKey.length>0){
                buttonWcbIsDefine=true;
              }
            }
            if((!buttonChatIsDefine && !buttonWcbIsDefine)|| !titleIsDefine){
              parameters.ASK_QUESTION_BEFORE_CHAT = false;
              console.warn( "«CHAT_FORM» title or buttons are undefined or not valid!" );
            }
          }

          if ( parameters.BOX_CONTACT_EMAIL ) {
            result.messages.ERROR_NO_OPERATOR_AVAILABLE += messages.CONTACT_MAIL_MESSAGE +parameters.BOX_CONTACT_EMAIL + "\">" + parameters.BOX_CONTACT_EMAIL + "</a>.";
          }

          if ( errors.length>0 ){
            $log.error('@AWB: configuration file is invalid! ( #' + errors.length + ' errors )', errors);
            return $q.reject({
              text  : 'configuration file is invalid!',
              errors: errors
            });
          }else{
            return result;
          }
        },

        /**
         * saves configuration data
         * @param data {object}
         * @returns {*}
         */
        save=function( data ){
          // merge boxParams with data coming from the configuration file
          _.merge(boxParams, data.parameters);

          // merge messages with data coming from the configuration file
          _.assign(messages, data.messages);

          // construct the global info objects that holds the current configuration
          Akio.Webbox.info = {
            App: {
              version : '@@project.version',
              loadDate: $filter('date')(new Date( ), 'medium')
            }
          };
          return data;
        },

        /**
         * fires error as notification
         * @param event
         * @returns {Promise}
         */
        firesErrorAsNotification=function(event){
          return $q.reject( { text: event.data } );
        };

      return {
        read: function( data ){
          return $q.when( data )
            .then( validate, firesErrorAsNotification )
            .then( save );
        }
      };
    }
  ]);
