'use strict';

/**
 * responsability:
 *  - handle the application's current states
 */
angular.module('akio.webbox')
  .factory('state', ['media', 'Backbone', 'APP_STORAGE_NAME', 'Event', 'Layout', 'i18nMessages', '$q', '$log', '$interval', 'MediumModel', 'MediaCollection', 'EventCollection', 'MediumViewStatus', 'ViewState', 'boxParams',
    function (media, Backbone, APP_STORAGE_NAME, Event, Layout, messages, $q, $log, $interval, MediumModel, MediaCollection, EventCollection, MediumViewStatus, ViewState, boxParams) {

      var service = {

          data: undefined,

          setWcbdStatus: function( status ){
            this.data.save({wcbdStatus: status});
          },

          getWcbdStatus: function(){
            return this.data.get('wcbdStatus');
          },

          setWcbDate: function( selectedDate ){
            this.data.save({wcbDate: selectedDate});
          },

          getWcbDate: function (){
            return this.data.get('wcbDate');
          },

          setSelectedCountry: function (selectedCountry) {
            this.data.save({selectedCountry: selectedCountry});
          },

          getSelectedCountry: function(){
            return this.data.get('selectedCountry');
          },

          setPhoneNumber: function (phoneNumber) {
            this.data.save({phoneNumber: phoneNumber});
          },

          getPhoneNumber: function(){
            return this.data.get('phoneNumber');
          },

          setWcbState: function (state) {
            return $q.when(this.data.save({wcbState: state}));
          },

          getWcbState: function () {
            return this.data.get('wcbState');
          },

          setCountriesList: function (countriesList) {
            var checkCountry = function (i) {
              _.forEach(countriesListToSave, function (value) {
                if (value.alphaCode === boxParams.BOX_COUNTRIES_LIST[i]) {
                  updatedCountriesList.push(value);
                }
              });
            };

            var countriesListToSave = countriesList;
            var updatedCountriesList = [];
            if (boxParams.BOX_COUNTRIES_LIST.length !== 0) {
              for (var i = 0; i < boxParams.BOX_COUNTRIES_LIST.length; i++) {
                checkCountry(i);
              }
              if (updatedCountriesList.length > 0) {
                countriesListToSave = updatedCountriesList;
              }

            }
            this.data.save({countriesList: countriesListToSave});
          },

          getCountriesList: function () {
            return this.data.get('countriesList');
          },

          /**
           * up to date list of media
           */
          mediaList: new MediaCollection(),

          /**
           * returns all the data
           * @returns {Object}
           */
          get: function () {
            return this.data;
          },

          /**
           * returns the number of message sent inside a session
           * this value is required to send a message
           * @returns {number}
           */
          getMessageId: function () {
            var oldValue = Number(this.data.get('sendMessageCount'));
            this.data.save({sendMessageCount: oldValue + 1});
            return oldValue + 1;
          },

          isTrigger: function () {
            return angular.isDefined(this.data.get('trigger'));
          },

          getTitle: function () {
            return this._title;
          },

          setTitle: function (value) {
            this._title = value;
          },

          setCollapseIcon: function (value) {
            this.data.save({_collapseIcon: value});
          },

          getCollapseIcon: function () {
            return this.data.get('_collapseIcon');
          },

          setCurrentMediaType: function (value) {
            this.data.save({currentMediaType: value});
          },

          getCurrentMediaType: function () {
            return this.data.get('currentMediaType');
          },

          setCurrentMediaCondition: function (value) {
            this.data.save({currentMediaCondition: value});
          },

          getCurrentMediaCondition: function () {
            return this.data.get('currentMediaCondition');
          },

          isTargetingRulesMatches: function () {
            return this.data.get('targetingRules').display;
          },

          getResultForAllCondition: function () {
            var result = true;
            var conditions = this.data.get('targetingRules').targetingConditionUuid;
            for (var i = 0; i < conditions.length; i++) {
              if (conditions[i].conditionUuid.length > 0 && !conditions[i].result) {
                result = false;
              }
            }
            return result;
          },


          setDisplayAsButton: function (value) {
            this.data.save({displayAsButton: value});
          },

          getDisplayAsButton: function () {
            var isDisplay = this.data !== null ? this.data.get('displayAsButton') : false;
            return isDisplay;
          },

          regions: function () {
            return this.data.get('regions');
          },

          events: function () {
            return this.data.get('events');
          },

          isLaunchBarMode: function () {
            return this.getCenterRegion() === ViewState.STATE_NONE;
          },

          resetChatSessionUri: function () {
            this.data.save({sessionUri: null});
          },

          getChatSessionUri: function () {
            return this.data.get('sessionUri');
          },

          /**
           * returns the center region description
           * @param {Array|undefined} filters
           * @returns {*}
           */
          getCenterRegion: function (filters) {
            return this.getByRegion(Layout.BORDER_CENTER, filters);
          },

          getWestRegion: function (filters) {
            return this.getByRegion(Layout.BORDER_WEST, filters);
          },

          /**
           * returns a description of a given region - BORDER_CENTER | BORDER_NORTH | BORDER_WEST
           * @param constraint
           * @param filters {Array}
           * @returns {*}
           */
          getByRegion: function (constraint, filters) {

            var regions = this.regions();

            if (regions.length === 0) {
              /* LaunchBar mode */
              return ViewState.STATE_NONE;
            } else {
              /* Box opened */
              var region = _.find(regions, function (item) {
                return item.constraint === constraint;
              });
              return this._processRegion(region, filters);
            }
          },

          getRegionByMediumType: function (mediumType, filters) {
            var region = _.find(this.regions(), function (item) {
              return item.mediumType === mediumType;
            });
            return (angular.isDefined(region)) ? this._processRegion(region, filters) : region;
          },

          _processRegion: function (region, filters) {
            var keys = (filters || []).length > 0 ? _.intersection(_.keys(region), filters) : [];

            if (keys.length > 0) {
              region = _.pick(region, keys);
            }

            return ((filters || []).length === 1) ? region[filters[0]] : region;
          },

          removeRegion: function (constraint) {

            var regions = _.clone(this.regions());

            if (!angular.isDefined(constraint)) {
              constraint = Layout.BORDER_CENTER;
            }

            if (constraint === Layout.BORDER_CENTER) {
              this.data.save({regions: []});
            } else if (constraint === Layout.BORDER_WEST) {
              _.remove(regions, function (region) {
                return region.constraint === constraint;
              });
              this.data.save('regions', regions);
            }

            return this.data;
          },

          setRegion: function (options) {
            var region = {},
              regions;

            switch (options.constraint) {
              case Layout.BORDER_CENTER:
                // reasigning the center region, clear the surrounding regions that are all depending from the center region.
                regions = [];

                region = {mediumStatus: MediumViewStatus.MEDIUM_STATUS_PENDING};

                break;

              case Layout.BORDER_NORTH:
                // nothing for now
                break;

              case Layout.BORDER_WEST:

                // copy the list of regions but the one we want to reassign
                regions = _.filter(_.clone(this.regions()), function (region) {
                  return region.constraint !== options.constraint;
                });

                region = {mediumStatus: MediumViewStatus.MEDIUM_STATUS_OPENED};

                break;

              default:
                throw 'state.setRegion => region constraint not valid or not implemented yet';
            }

            _.assign(region, options);

            regions.push(region);

            return this.data.save('regions', regions);
          },

          /**
           * returns whether the current media is opened
           * @param value {String || Object}
           * @returns {boolean}
           */
          isMediumOpened: function (value) {
            return this.getMediaStatus(value) === MediumViewStatus.MEDIUM_STATUS_OPENED;
          },

          isMediumPending: function (region) {
            return region.mediumStatus === MediumViewStatus.MEDIUM_STATUS_PENDING;
          },

          isMediumCalendar: function (region) {
            return region.mediumStatus === MediumViewStatus.MEDIUM_STATUS_CALENDAR;
          },

          isMediumConfirm: function (region) {
            return region.mediumStatus === MediumViewStatus.MEDIUM_STATUS_CONFIRM;
          },

          getMediaStatus: function (value) {
            var status;
            if (!angular.isDefined(value)) {
              status = this.getCenterRegion(['mediumStatus']);
            } else if (angular.isObject(value)) {
              // value as an Object is supposed to be a region
              status = value.mediumStatus;
            } else {
              // value as a String is supposed to be a medium type
              status = this.getRegionByMediumType(value, ['mediumStatus']);
            }
            return status;
          },

          /**
           * set Media status. possible values are : MEDIA_STATUS_OPENED | MEDIA_STATUS_CLOSED
           */
          setMediaStatus: function (mediumUuid, status) {
            var regions = this.data.get('regions'),
              index = _.findIndex(regions, function (item) {
                return item.mediumUuid === mediumUuid;
              });

            try {
              regions[index].mediumStatus = status;
              this.data.set({regions: regions}).save();
            } catch (err) {
              $log.warn('state.setMediaStatus --> media is not active anymore');
            }
          },

          /**
           * restore session by fetching data from the browser session storage
           * @param params
           */
          sync: function (params) {
            var deferred = $q.defer();
            params.userNickname = messages.CHATBOX_DEFAULT_USER_NICKNAME;

            this.data = new Config(params);

            ////fires when the 'trigger' attributes changes
            ////TRIGGERS are not implemented right now. NOt the right place anyway
            //this.data.on('change:trigger', function( model, value, options ){
            //  var medium, title;
            //  if (angular.isDefined(value)){ /* trigger is removed */
            //    title = messages.TITLE_FORM_SURVEY; // TODO: need to be dynamically interpreted. Hardcoded to demonstrate the FORM SURVEY TRIGGER.
            //  }else{ /* trigger is displayed */
            //    title = messages.BOX_TITLE_DEFAULT;
            //  }
            //
            //  this.setTitle( title );
            //}, this);

            // fetches session data from the navigator's session storage
            this.data.fetch({
              success: function (model) {
                deferred.resolve(model);
              },
              error: function (model, xhr) {
                deferred.reject(xhr.responseText);
              }
            });

            return deferred.promise;
          },

          /**
           * dump session
           */
          dump: function () {
            var id = this.data.get('id'), /* session id related to the sessionStorage lib */
              data = _.clone(this.data.defaults);

            this.data.get('events').reset();

            _.extend(data, {id: id});

            this.data.save(data);
          }
        },

        /**
         * ui states
         */
        UISettings = Backbone.Model.extend({
          defaults: {
            collapse: false, // chatBoxState: tells whether the box is collapsed or expanded. Two values: OPEN | CLOSE
            zoom: 1 /* zoom coefficient. */
          }
        }),

        /**
         * collection of events received from AIP
         */
        Events = EventCollection,

        /**
         * holds the data save in the sesionStorage
         */
        Config = Backbone.Model.extend({
          defaults: {
            sessionUri: undefined,
            regions: [], // layout region list
            events: new Events(), // session event list
            sendMessageCount: 0,
            trigger: undefined,
            ui: new UISettings(),
            userNickname: messages.CHATBOX_DEFAULT_USER_NICKNAME,
            displayAsButton: false,
            openMediaChatRequestSent: false,
            closeMediaChatRequestSent: false,
            openMediaWcbRequestSent: false,
            closeMediaWcbRequestSent: false,
            // Box targeting rules
            targetingRules: {
              targetingConditionUuid: "",
              display: false
            },
            currentMediaCondition: '',
            currentMediaType: '',
            webboxRule: {},
            wcbState: '',
            countriesList: [],
            _collapseIcon: false,
            isFirstChatInSession: true
          },
          sessionStorage: new Backbone.SessionStorage(APP_STORAGE_NAME),
          parse: function (data) {
            if (data.events instanceof Array) {
              data.events = new Events(data.events);
            }
            return data;
          }
        });

      return service;
    }
  ]);
