'use strict';

angular.module('akio.webbox')
  .factory('timePickerUtils', ['boxParams', function(boxParams) {
    return {
      cloneDate: function (date) {
        return new Date(date);
      },

      formatDate: function (date) {
        return date.toLocaleDateString(languageDetection(boxParams.I18N), {
          weekday: 'long',
          year: 'numeric',
          month: 'short',
          day: 'numeric'
        });
      },

      formatTime: function (date) {
        return date.toLocaleTimeString(languageDetection(boxParams.I18N), {
          hour: 'numeric',
          minute: 'numeric'
        });
      },

      createTimeslot: function (timeslot) {
        var date = new Date(timeslot.fromDate);
        var endDate = new Date(timeslot.toDate);
        return {
          date: date.getTime(),
          endDate: endDate.getTime(),
          display: this.formatTime(date),
          opened: timeslot.opened,
          available: timeslot.available
        };
      },

      sortTimeslotsByDay: function(timeslots) {
        var sortedTimeslots = {};
        _.forEach(timeslots, function(timeslot) {
          var timePickerTimeslot = this.createTimeslot(timeslot);

          var timeslotDate = new Date(timeslot.fromDate);
          timeslotDate.setHours(0, 0, 0, 0);

          var timeslotDateTimestamp = timeslotDate.getTime();
          if (sortedTimeslots.hasOwnProperty(timeslotDateTimestamp)) {
            sortedTimeslots[timeslotDateTimestamp].push(timePickerTimeslot);
          } else {
            sortedTimeslots[timeslotDateTimestamp] = [timePickerTimeslot];
          }
        }, this);
        return sortedTimeslots;
      },

      getHours: function (sortedTimeslots) {
        var maxTimeslotNumber = 24;
        var filteredTimeslots = _.filter(sortedTimeslots, 'opened');

        // Can't display more timeslots !
        var dateTimeslotsLimit = Math.min(filteredTimeslots.length, maxTimeslotNumber);

        return filteredTimeslots.slice(0, dateTimeslotsLimit);
      },

      getFirstDisplayedDate: function(timeslots) {
        // Get first displayed timeslot
        var firstDisplayedTimeslot = _.find(timeslots, 'opened');
        if (_.isUndefined(firstDisplayedTimeslot)) {
          // If not displayed date available
          firstDisplayedTimeslot = timeslots[0];
        }
        var firstDisplayedTimeslotDate = new Date(firstDisplayedTimeslot.fromDate);
        // Set date at midnight
        firstDisplayedTimeslotDate.setHours(0, 0, 0, 0);
        return firstDisplayedTimeslotDate;
      },

      getMaxDate: function (timeslots) {
        var maxDate = _.last(timeslots);
        return new Date(maxDate.fromDate);
      },

      getMinDate: function (timeslots) {
        var minDate = _.first(timeslots);
        return new Date(minDate.fromDate);
      }
    };
  }]);
