'use strict';

angular.module('akio.webbox')
  .controller('widgetCtrl', ['$scope', 'session', 'externalInterface', 'navigation', '$injector', '$log', 'i18nMessages','state','determineClient',
    function ($scope, session, externalInterface, navigation, $injector, $log, labels, state, determineClient) {

      $scope.session= session;
      $scope.labels = labels;
      $scope.isDisplayOnMobile = determineClient.device === 'smartphone';
      $scope.isDisplayOnTablet = determineClient.device === 'tablet';
      $scope.isDisplayOnDesktop = determineClient.device === 'desktop';

      $scope.addDeviceClass = function(){
        if ($scope.isDisplayOnMobile) {
          return 'awbMobileDevice';
        } else if($scope.isDisplayOnTablet) {
          return 'awbTabletDevice';
        }else if($scope.isDisplayOnDesktop){
          return 'awbDesktopDevice';
        }else{
          return '';
        }
      };

      $scope.getLaunchBarAnimation = function() {
        if (determineClient.device === 'smartphone' || determineClient.device === 'tablet') {
          return 'awb-bubble-animation';
        }
        return 'awb-pane-animation';
      };

      /**
       * returns the current active card
       * @returns {number} active card number
       */
      $scope.getActiveCard = function(){
        var isBoxOpened  = $scope.session.isBoxOpened(),
          isActiveTrigger= $scope.session.isActiveTrigger();

        if (!isBoxOpened && isActiveTrigger) {
          return 2; /* TRIGGER */
        } else {
          return (isBoxOpened)? 1 /* WINDOW */:0 /* LAUNCHBAR */;
        }
      };

      $scope.$on('trigger:accept', function(e, message, messageIndex, messageData){
        $log.info('trigger:accept', message, messageIndex, messageData);

        var messageType = message.get('type');

        if ( messageType === 'NetworkConnectionLazyEvent' ) {
          // we try to resume the session
          session.fetch();
          session.fetchRetryCounter = 0;
          session.isTryingToReconnect = true;

        } else if ( messageType === 'ServiceInterruptedEvent' ) {
          // if there is an ongoing conversation, we stop it
          // then we start a new one
          session.renew( );

        } else if ( messageType === 'SurveyEvent' ) {
          // send result to aip
          $log.info('trigger:accept --> messageData', messageData);
        } else if ( messageType === 'ChatInvitationEvent' ) { /* open Media chat */
          var mediaList = session.getState().mediaList,
            medium      = mediaList.findWhere({'type': 'CHAT'});

          $injector.get('aip')
            .openMediaChat()
            .then( function(){
              navigation.goto( medium );
            } );

        } else if ( messageType === 'FormRequestEvent' ) {
          $injector.get('messageReader').injectInvitationEvent( true, message );
          navigation.setRegionByEvent(message, messageIndex);
        } else {
          navigation.setRegionByEvent(message, messageIndex);
        }
      });

      $scope.$on('trigger:reject', function(e, message){
        var messageType = message.get('type');

        if (messageType === 'NetworkConnectionLazyEvent' || messageType === 'ServiceInterruptedEvent') {
          session.fetchEventsIsRunning = false;
          session.isTryingToReconnect = false;
          state.events().reset();
          navigation.quit( ); // launch bar mode
        } else if ( messageType === 'FormRequestEvent' ) {
          $injector.get('messageReader').injectInvitationEvent( false, message );
        }
      });

      // ----------------------------
      // ExternalInterface: BEGIN
      // ----------------------------

      /**
       * navigate to a media
       * @param medium
       */
      $scope.gotoMedium = function(medium){
        // explicitly start the model-changes tracking process
        $scope.$apply(function(){
          externalInterface.navigateTo(medium);
        });
      };

      // DEBUG only

      $scope.sendFormEvent = function( ){
        $scope.$apply(function(){
          externalInterface.sendFormEvent( );
        });
      };

      $scope.sendChatInvitation = function( ){
        $scope.$apply(function(){
          externalInterface.sendChatInvitation( );
        });
      };

      $scope.sendSurveyInvitation = function( ){
        $scope.$apply(function(){
          externalInterface.sendSurveyInvitation( );
        });
      };

      $scope.sendErrorInvitation = function( ){
        $scope.$apply(function(){
          externalInterface.sendErrorInvitation( );
        });
      };

      // ----------------------------
      // ExternalInterface: END
      // ----------------------------

    }
  ]);
