'use strict';

/**
 * application's top container directive
 * @event widgetUI:loaded fires when the UI is loaded and ready.
 * @event widgetUI:error fires when the UI failed to load.
 */
angular.module('akio.webbox')

  .directive('awbWidget', ['$timeout', '$rootScope', '$window', '$http', 'determineClient','state',
    function( $timeout, $rootScope, $window, $http, determineClient, state ){

      /**
       * @inheritDoc
       */
      var link = function( scope, iElement ){
        var element= jQuery( iElement ),
          $iFrameElt,
          data = {},

          /**
           * fires when the widget succeeds to load
           */
          success=function( ){
            $rootScope.$emit( 'widgetUI:loaded', data );
          };


        var base = $window.AkioBaseUrl; // Set by the launcher

        $http.get(base+'/data/akio-webbox-settings.json')
          .then(function(result) {
            data.parameters = result.data.parameters;

            var language = languageDetection(data.parameters.I18N);
            $window.webBoxLanguage = language;
            //if language is fr we take the french messages else we take the english messages
            var jsonFileToIntegrate = base+'/data/i18n/messages_'+(language.indexOf('fr') > -1 ? 'fr' : 'en')+'.json';

            // Get language JSON
            return $http.get(jsonFileToIntegrate);
          })
          .then(function(result) {
            data.messages = result.data.messages;
            return data;
          })
          .then(function() {
            success( );
          }
        );

        function resizeForScrolling() {
            if($rootScope.session.isBoxOpened()&& !state.data.get('ui').collapse){
              resizeIFrame();
              window.parent.window.scrollTo(0,0);
            }
          }

        //resize if scroll
        if(determineClient.device === 'smartphone' && determineClient.os === 'iOS'){
          parent.window.addEventListener("scroll", resizeForScrolling);
        }


        var resizeIFrame = function(height) {
          var iFrameWrapper = jQuery('#akioWebBoxIFrame', window.parent.document).parent();
          if (determineClient.device === 'desktop' || determineClient.device === 'tablet') {
            iFrameWrapper.height(height);
          } else {
            if(state.isLaunchBarMode()){
              if(height !==null){
                iFrameWrapper.height(height);
              }else{
                iFrameWrapper.height(parent.window.innerHeight);
              }
            }else{
              if(state.data.get('ui').collapse && height!==null){
                iFrameWrapper.height(height);
              }else if(!state.data.get('ui').collapse){
                iFrameWrapper.height(parent.window.innerHeight);
              }else{
                iFrameWrapper.height(parent.window.innerHeight);
              }
            }
          }
        };

        // Resize dynamically the iframe when webbox content height is changing !
        scope.$on('webbox:animate', function(event, height) {
          if(_.isUndefined(height)) {
            setTimeout(function(){
              resizeIFrame(element.height());
            },100);
          } else {
            resizeIFrame(height);
          }
        });

        /**
         * listens when the component is removed from the DOM
         */
        scope.$on('$destroy', function onComponentDestroyed( ){
          $iFrameElt.off( );
        });
      };

      return {
        restrict: 'EA',
        replace: true,
        templateUrl: 'app/widget/widget.tpl.html',
        link: link
      };
    }
  ]);
