'use strict';

describe('test the differents filters', function() {

  beforeEach(module('akio.webbox.filters'));

  afterEach(function() {
    sessionStorage.clear();
  });

  it('should capitalize the text', inject(
    function ($filter) {
      expect($filter('capitalize')('hello')).toEqual('Hello');
    }
  ));

  it('should not be equal', inject(
    function ($filter) {
      expect($filter('capitalize')('hello')).not.toEqual('hello');
    }
  ));

  it('should unsafe the text space',inject(
    function ($filter) {
      var testFilter = $filter('unsafe')('<h1>there is a space</h1>');
      expect(testFilter).not.toEqual('<h1>there is a space</h1>');
    }
  ));

  it('should reverse',inject(
    function ($filter) {
      expect($filter('reverse')(['a','b','c'])).toEqual(['c','b','a']);
    }
  ));

  it('should reverse',inject(
    function ($filter) {
      expect($filter('reverse')(['a','b','c'])).not.toEqual(['a','b','c']);
    }
  ));

  it('should delete space',inject(
    function ($filter) {
      expect($filter('nospace')('there is a space')).toEqual('thereisaspace');
    }
  ));

  it('should delete space',inject(
    function ($filter) {
      expect($filter('nospace')('there is a space')).not.toEqual('there is a space');
    }
  ));

  it('should create an html link if word starting by www or http ',inject(
    function ($filter) {
      expect($filter('transformLink')('www.akio.com')).toEqual('<a name="awbChatLink" href="http://www.akio.com" target="_blank">www.akio.com</a>');
      expect($filter('transformLink')('http://www.akio.com')).toEqual('<a name="awbChatLink" href="http://www.akio.com" target="_blank">http://www.akio.com</a>');
    }
  ));

  it('should not create an html link if only 2 "w" or if two words are separate with a point "." ',inject(
    function ($filter) {
      expect($filter('transformLink')('ww.akio.com')).not.toEqual('<a name="awbChatLink" href="http://ww.akio.com" target="_blank">ww.akio.com</a>');
      expect($filter('transformLink')('mot.mot')).not.toEqual('<a name="awbChatLink" href="http://mot.mot" target="_blank">mot.mot</a>');
    }
  ));

  it('should interpret lineBreak',inject(
    function ($filter) {
      expect($filter('transformLineBreak')('passage à &lt;br/&gt; la ligne')).toEqual('passage à <br/> la ligne');
    }
  ));


  it('should transform html',inject(
    function ($filter) {
      var testFilter = $filter('noHTML')('<h1>titre</h1>');
      expect(testFilter).not.toEqual('<h1>titre</h1>');
      expect(testFilter).toEqual('&lt;h1&gt;titre&lt;/h1&gt;');
    }
  ));

  it('should transform duration',inject( //todo:how it works ?
    function ($filter) {
      var testFilter = $filter('duration')(375);
      console.log(testFilter);
      //expect(testFilter).not.toEqual('375');
      //expect(testFilter).toEqual('375');
    }
  ));

  it('should trust as html',inject(
    function ($filter) {
      var testFilter = $filter('to_trusted')('<h1>titre</h1>');
      expect(testFilter).not.toEqual('<h1>titre</h1>');
    }
  ));


});

