'use strict';

describe('test chatForm controller', function () {


  beforeEach(module('akio.webbox'));

  beforeEach(function () {
    if (!window.Akio) {
      window.Akio = {};
    }
    if (!window.Akio.Webbox) {
      window.Akio.Webbox = {};
    }
    if (!window.Akio.Webbox.settings) {
      window.Akio.Webbox.settings = {};
    }
    if (!window.Akio.Webbox.settings.parameters) {
      window.Akio.Webbox.settings.parameters = {};
    }
    if (!window.Akio.Webbox.settings.parameters.I18N) {
      window.Akio.Webbox.settings.parameters.I18N = '"fr"';
    }
  });

  var $controller, state, aip, $scope, boxParams, chatFormController, navigation;

  beforeEach(inject(function (_$controller_, _state_, _aip_, _$rootScope_, _boxParams_, _navigation_) {
    $controller = _$controller_;
    state = _state_;
    aip = _aip_;
    $scope = _$rootScope_;
    boxParams = _boxParams_;
    navigation = _navigation_;
    boxParams.CHAT_FORM = [
      {
        'type': 'title',
        'textKey': 'CHAT_FORM_TITLE'
      },
      {
        'type': 'button',
        'textKey': 'CHAT_FORM_VALID_BUTTON'
      },
      {
        'type': 'field',
        'mapping': 'frm_adresse_email_1'
      },
      {
        'type': 'buttonWCB',
        'textKey': 'WCB_FORM_VALID_BUTTON'
      }

    ];

    /* jshint ignore:start */
    var profileParameters = {
      frm_nom: 'Martin',
      frm_prenom: 'Flora',
      nickname: 'flo',
      frm_adresse_email_1: 'flora.martin@akio.com',
      frm_titre_de_la_page: document.title,
      frm_url_de_la_page: window.location.href,
      frm_reference_client: ''
    };
    window.parent.profileParameters = profileParameters;
    /* jshint ignore:end */

    //instantiate a new state
    state.sync({});
    state.mediaList.set([{
      'conditionUuid': '497a9c9f-1e1d-47f7-bed6-333ffd440f72',
      'uuid': '2',
      'name': 'Chat',
      'status': 'OPEN_AVAILABLE',
      'estimatedWaitTime': 25,
      'type': 'CHAT'
    }, {
      'conditionUuid': 'dedd5031-075a-451b-9e93-dfec6e2d1eff',
      'uuid': '8',
      'name': 'WebCallback',
      'status': 'OPEN_AVAILABLE',
      'estimatedWaitTime': 25,
      'type': 'VOICE'
    }]);
    $scope.labels = {
      WAITBOX_TITLE: 'title'
    };
    chatFormController = $controller('chatFormCtrl', {$scope: $scope});
  }));

  it('defined a controller and init buttons', function () {
    expect(chatFormController).toBeDefined();
    expect($scope.canStartChat()).toBeTruthy();
    expect($scope.canStartWcb()).toBeTruthy();
  });

  it('will disable change media buttons', function () {
    state.mediaList.set([{
      'conditionUuid': '497a9c9f-1e1d-47f7-bed6-333ffd440f72',
      'uuid': '2',
      'name': 'Chat',
      'status': 'CLOSED',
      'estimatedWaitTime': 25,
      'type': 'CHAT'
    }, {
      'conditionUuid': 'dedd5031-075a-451b-9e93-dfec6e2d1eff',
      'uuid': '8',
      'name': 'WebCallback',
      'status': 'CLOSED',
      'estimatedWaitTime': 25,
      'type': 'VOICE'
    }]);
    expect($scope.canStartChat()).toBeFalsy();
    expect($scope.canStartWcb()).toBeFalsy();
  });

  it('will change the box to collapse false to collapse true', function () {
    expect(state.getCollapseIcon()).toBeFalsy();
    $scope.startChat();
    expect(state.getCollapseIcon()).toBeTruthy();
  });

  it('will ask aipController to send an openMediaChat request', function () {
    spyOn(aip, 'openMediaChat').and.callThrough();
    spyOn(navigation, 'goto');
    $scope.startChat();
    expect(aip.openMediaChat).toHaveBeenCalled();
    expect(navigation.goto).toHaveBeenCalled();
  });

  it('will navigate to wcb screen', function () {
    spyOn(navigation, 'goto');
    $scope.startWcb();
    expect(navigation.goto).toHaveBeenCalled();
  });


});
