'use strict';

describe('test wait controller', function() {


  beforeEach(module('akio.webbox'));

  beforeEach(function() {
    if(!window.Akio) { window.Akio={}; }
    if(!window.Akio.Webbox) { window.Akio.Webbox={}; }
    if(!window.Akio.Webbox.settings) { window.Akio.Webbox.settings={}; }
    if(!window.Akio.Webbox.settings.parameters) { window.Akio.Webbox.settings.parameters={}; }
    if(!window.Akio.Webbox.settings.parameters.I18N) { window.Akio.Webbox.settings.parameters.I18N='"fr"'; }
  });

  var $controller,state,Event,notification;

  beforeEach(inject(function(_$controller_, _state_,_Event_,_notification_){
    $controller = _$controller_;
    state = _state_;
    Event = _Event_;
    notification=_notification_;

    //instantiate a new state
    state.sync({});

    //set a media result
    var mediaResult = {
      description : 'no description',
      estimatedWaitTime : 9,
      exposed :  true,
      icon : 'fa-comments-o',
      message : 'Ask us by Chat !',
      name : 'Chat',
      priority : 1,
      renderer : 'BUTTON',
      status : 'OPEN_AVAILABLE',
      textInput : true,
      title : 'Chat295',
      type : 'CHAT',
      uuid :'2'
    };

    //set the media in the state
    state.mediaList.set(mediaResult);

  }));

  it('defined a controller',function(){
    state.getCenterRegion = function(){return {mediumUuid:'2'};};
    var $scope = {};
    $scope.$watch=function(fake){return fake;};
    $scope.$on=function(fake){return fake;};
    $scope.labels={
      WAITBOX_TITLE:'title'
    };
    $scope.titleBar={};
    $scope.titleBar.setPanelAsClosable=function(boolean){console.log('setPanelAsClosable '+boolean);};
    $scope.titleBar.isCollapsed = function(){return true;};

    var waitController = $controller('waitCtrl', { $scope: $scope });
    expect(waitController).toBeDefined();

    expect($scope.info.canCancel()).toBe(true);
    expect($scope.info.isEstimatedWaitTime()).toBe(true);

    $scope.restart();
    expect($scope.restart).toBeDefined();

    $scope.info.cancelOpenMediumRequest();
    expect($scope.info.cancelOpenMediumRequest).toBeDefined();

    $scope.info.enterMedium();
    expect($scope.info.enterMedium).toBeDefined();

  });

  it('should resolve the viewstate ',function(){
    state.getCenterRegion = function(){return {mediumUuid:'2'};};
    var $scope = {};
    $scope.$watch=function(fake){return fake;};
    $scope.$on=function(fake){return fake;};
    $scope.labels={
      WAITBOX_TITLE:'title',
      WAITBOX_TITLE_INABILITY:'title2'
    };
    $scope.titleBar={};
    $scope.titleBar.setPanelAsClosable=function(boolean){console.log('setPanelAsClosable '+boolean);};
    $scope.titleBar.isCollapsed = function(){return true;};

    var waitController = $controller('waitCtrl', { $scope: $scope });
    expect(waitController).toBeDefined();

    //we create an event
    var e = new Event();
    e.type = 'FormRequestEvent';
    e.eventId =0 ;
    e.message = 'I am a fake event message';
    e.formData= {'type': 'VOICE'};
    e.isOpenMediumResultEvent=function(){return true;};
    $scope.resolveViewState(e);
    expect(state.getTitle()).toEqual('title2');

  });

  it('should notify the viewstate change',function(){
    state.getCenterRegion = function(){return {mediumUuid:'2'};};
    var $scope = {};
    $scope.$watch=function(fake){return fake;};
    $scope.$on=function(fake){return fake;};
    $scope.labels={
      WAITBOX_TITLE:'title'
    };
    $scope.titleBar={};
    $scope.titleBar.setPanelAsClosable=function(boolean){console.log('setPanelAsClosable '+boolean);};
    $scope.titleBar.isCollapsed = function(){return true;};
    var waitController = $controller('waitCtrl', { $scope: $scope });
    expect(waitController).toBeDefined();



    var e = new Event();
    e.type = 'FormRequestEvent';
    e.eventId =0 ;
    e.message = 'I am a fake event message';
    e.formData= {'type': 'VOICE'};
    e.isOpenMediumResultEvent=function(){return true;};

    var notificationLength = _.size(notification.list);
    $scope.notifyViewStatesChange();


    expect(notificationLength).toEqual(_.size(notification.list)-1);

    var f = new Event();
    f.type = 'OperatorJoinedEvent';
    f.eventId =0 ;
    f.message = 'I am a fake event message';
    f.formData= {'type': 'CHAT'};
    f.isOperatorJoinedEvent=function(){return true;};
    $scope.resolveViewState(f);

    var notificationLength2 = _.size(notification.list);

    $scope.notifyViewStatesChange();
    expect(notificationLength2).toEqual(_.size(notification.list));

  });

  it('should restart ',function(){
    state.getCenterRegion = function(){return {mediumUuid:'2'};};
    var $scope = {};
    $scope.$watch=function(fake){return fake;};
    $scope.$on=function(fake){return fake;};
    $scope.labels={
      WAITBOX_TITLE:'title'
    };
    $scope.titleBar={};
    $scope.titleBar.setPanelAsClosable=function(boolean){console.log('setPanelAsClosable '+boolean);};
    $scope.titleBar.isCollapsed = function(){return true;};
    var waitController = $controller('waitCtrl', { $scope: $scope });
    expect(waitController).toBeDefined();



    var e = new Event();
    e.type = 'FormRequestEvent';
    e.eventId =0 ;
    e.message = 'I am a fake event message';
    e.formData= {'type': 'VOICE'};
    e.isOpenMediumResultEvent=function(){return true;};
    $scope.info.viewState = '';

    $scope.restart();
    expect($scope.info.canCancel()).toEqual(false);

  });

});


describe('directive: wait', function() {
  var element, $compile, $scope, state, boxParams;

  beforeEach(module('akio.webbox'));
  beforeEach(module('app/layout/window/wait.tpl.html'));


  beforeEach(function () {
    inject(function ($rootScope, _$compile_, _state_, _boxParams_) {
      $scope = $rootScope.$new();
      $compile = _$compile_;
      state = _state_;
      boxParams = _boxParams_;
    });

    element = $compile(angular.element('<div awb-pane-wait ></div>'))($scope);


  });


  it('should create an html element', function () {
    expect(element).toBeDefined();
  });
});
