'use strict';

describe('test wcb controller', function () {

  beforeEach(module('akio.webbox'));
  beforeEach(module('app/layout/window/titlebar.tpl.html'));

  var wcbCtrl, $scope, state, ViewState, compile, labels, aip, session, boxParams;

  beforeEach(inject(function ($controller, $rootScope, $log, _aip_, $q, $compile, MediumViewStatus, notification,
                              MediumType, i18nMessages, _state_, _session_, _ViewState_, _boxParams_) {
    $scope = $rootScope.$new();
    ViewState = _ViewState_;
    state = _state_;
    compile = $compile;
    labels = i18nMessages;
    aip = _aip_;
    session = _session_;
    boxParams = _boxParams_;

    //instantiate a new state
    state.sync({});

    //set a media result
    var mediaResult = {
      description: 'no description',
      estimatedWaitTime: 9,
      exposed: true,
      icon: 'fa-comments-o',
      message: 'free call !',
      name: 'Wcb',
      priority: 1,
      renderer: 'BUTTON',
      status: 'OPEN_AVAILABLE',
      textInput: true,
      title: 'wcb295',
      type: 'WCB',
      uuid: '0'
    };

    //set the media in the state
    state.mediaList.set(mediaResult);
    state.setCountriesList([
      {
        indicator: '+33',
        countryCode: 'FR',
        countryName: 'france'
      },
      {
        indicator: '+41',
        countryCode: 'CH',
        countryName: 'suisse'
      },
      {
        indicator: '+377',
        countryCode: 'MC',
        countryName: 'monaco'
      },
      {
        indicator: '+32',
        countryCode: 'BE',
        countryName: 'belgique'
      },
      {
        indicator: '+32',
        countryCode: 'AU',
        countryName: 'belgique'
      },
      {
        indicator: '+32',
        countryCode: 'AS',
        countryName: 'belgique'
      }
    ]);
    //force ViewState for make injectMessage works
    ViewState.STATE_NONE = '2';

    //awbTitlebar needed for a scope function
    compile(angular.element('<div awb-titlebar></div>'))($scope);

    //set the scope.state
    $scope.state = state;

    wcbCtrl = $controller('wcbCtrl', {
      $scope: $scope,
      $log: $log,
      state: state,
      aip: aip,
      $q: $q,
      MediumViewStatus: MediumViewStatus,
      notification: notification,
      MediumType: MediumType,
      labels: i18nMessages
    });
  }));

  it('should be defined and initialized', function () {
    expect(wcbCtrl).toBeDefined();
    expect($scope).toBeDefined();
    expect($scope.order).toEqual(labels.MEDIUM_WCB_ORDER);
    expect($scope.buttonLabel).toEqual(labels.MEDIUM_WCB_BUTTON);
    expect($scope.phoneNumberPlaceHolder).toEqual(labels.MEDIUM_WCB_PHONE_NUMBER_PLACEHOLDER);
  });

  it('should check if number is a numberphone', function () {
    var isPhoneNumber = $scope.validatePhone('0296458745');
    expect(isPhoneNumber).toEqual(true);
  });

  it('should check if number is not a numberphone', function () {
    var isNotPhoneNumber = $scope.validatePhone('02aa-p8745');
    expect(isNotPhoneNumber).toEqual(false);
  });

  it('should return false if there is no instant webcallback available', function () {
    //set the media in the state
    var mediaResult = {
      conditionUuid: 'dedd5031-075a-451b-9e93-dfec6e2d1eff',
      uuid: '2',
      name: 'Chat',
      status: 'OPEN_AVAILABLE',
      estimatedWaitTime: 25,
      type: 'CHAT'
    };
    state.mediaList.set(mediaResult);
    expect($scope.isInstantWcbAvailable()).toBeFalsy();
  });

  it('should return true if there is instant webcallback available', function () {
    //set the media in the state
    var mediaResult = {
      conditionUuid: 'dedd5031-075a-451b-9e93-dfec6e2d1eff',
      uuid: '8',
      name: 'Voice',
      status: 'OPEN_AVAILABLE',
      estimatedWaitTime: 25,
      type: 'VOICE'
    };
    state.mediaList.set(mediaResult);
    expect($scope.isInstantWcbAvailable()).toBeTruthy();
  });

  it('should return false if there is no delayed webcallback available', function () {
    //set the media in the state
    var mediaResult = {
      conditionUuid: 'dedd5031-075a-451b-9e93-dfec6e2d1eff',
      uuid: '2',
      name: 'Chat',
      status: 'OPEN_AVAILABLE',
      estimatedWaitTime: 25,
      type: 'CHAT'
    };
    state.mediaList.set(mediaResult);
    expect($scope.isDelayedWcbAvailable()).toBeFalsy();
  });

  it('should return true if there is delayed webcallback available', function () {
    //set the media in the state
    var mediaResult = {
      conditionUuid: 'dedd5031-075a-451b-9e93-dfec6e2d1eff',
      uuid: '8',
      name: 'Voice',
      status: 'OPEN_DELAYED_WEBCALLBACK',
      estimatedWaitTime: 25,
      type: 'VOICE'
    };
    state.mediaList.set(mediaResult);
    expect($scope.isDelayedWcbAvailable()).toBeTruthy();
  });

});
