'use strict';

describe('test wcb controller', function () {

  beforeEach(module('akio.webbox'));
  beforeEach(module('app/layout/window/titlebar.tpl.html'));

  var wcbCtrl, $scope, state, ViewState, compile, labels, aip, session, boxParams, $timeout;

  beforeEach(inject(function ($controller, $rootScope, $log, _aip_, $q, $compile, MediumViewStatus, notification,
                              MediumType, i18nMessages, _state_, _session_, _ViewState_, _boxParams_, _$timeout_) {
    $scope = $rootScope.$new();
    ViewState = _ViewState_;
    state = _state_;
    compile = $compile;
    labels = i18nMessages;
    aip = _aip_;
    session = _session_;
    boxParams = _boxParams_;
    $timeout = _$timeout_;

    //instantiate a new state
    state.sync({});

    //set a media result
    var mediaResult = {
      description: 'no description',
      estimatedWaitTime: 9,
      exposed: true,
      icon: 'fa-comments-o',
      message: 'free call !',
      name: 'Wcb',
      priority: 1,
      renderer: 'BUTTON',
      status: 'OPEN_AVAILABLE',
      textInput: true,
      title: 'wcb295',
      type: 'WCB',
      uuid: '0'
    };

    //set the media in the state
    state.mediaList.set(mediaResult);
    state.setCountriesList([
      {
        indicator: '+33',
        countryCode: 'FR',
        countryName: 'france'
      },
      {
        indicator: '+41',
        countryCode: 'CH',
        countryName: 'suisse'
      },
      {
        indicator: '+377',
        countryCode: 'MC',
        countryName: 'monaco'
      },
      {
        indicator: '+32',
        countryCode: 'BE',
        countryName: 'belgique'
      },
      {
        indicator: '+32',
        countryCode: 'AU',
        countryName: 'belgique'
      },
      {
        indicator: '+32',
        countryCode: 'AS',
        countryName: 'belgique'
      }
    ]);
    //force ViewState for make injectMessage works
    ViewState.STATE_NONE = '2';

    //awbTitlebar needed for a scope function
    compile(angular.element('<div awb-titlebar></div>'))($scope);

    //set the scope.state
    $scope.state = state;

    wcbCtrl = $controller('wcbCtrl', {
      $scope: $scope,
      $log: $log,
      state: state,
      aip: aip,
      $q: $q,
      MediumViewStatus: MediumViewStatus,
      notification: notification,
      MediumType: MediumType,
      labels: i18nMessages
    });
  }));

  it('should set the selected Country', function () {
    var newCountry = {
      indicator: '+377',
      countryCode: 'mc',
      countryName: 'monaco'
    };
    $scope.changeCountry(newCountry);

    expect($scope.selectedCountry).toEqual(newCountry);
  });


  it('should send number on press enter key', function () {
    var event = {
      keyCode: 13
    };
    var indicator = '+33';
    $scope.phoneNumber = '0265656565';
    spyOn($scope, 'callNow');
    $scope.sendFormOnPressEnter(event, indicator);
    expect($scope.callNow).toHaveBeenCalled();
  });


  it('should send a wcb request', function () {
    spyOn(aip, 'openMediaWcb').and.callThrough();
    boxParams.ALLOW_SIP_NUMBER = true;
    $scope.openMediaWcb('+33', 'sip:10.22.84.87');
    var requestSent = state.get().get('openMediaWcbRequestSent');
    expect(requestSent).toEqual(true);
    expect(aip.openMediaWcb).toHaveBeenCalled();
  });


  it('should update the ui', function () {
    state.setWcbState('DIALING_IN_PROGRESS');
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual(labels.WCB_ONGOING_CALL);

    state.setWcbState('CUSTOMER_PHONE_RINGING');
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual(labels.WCB_ONGOING_CALL);

    state.setWcbState('WCB_IN_PREPARATION');
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual(labels.WCB_ONGOING_CALL);

    state.setWcbState('CALL_FAILED_NEW_ATTEMPT_IN_PROGRESS');
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual(labels.WCB_CALL_FAIL_TRY_AGAIN);

    state.setWcbState('CALL_FAILED_LAST_ATTEMPT');
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual(labels.WCB_CALL_FAIL_NO_TRY);

    state.setWcbState('CALL_PICKED_UP_BY_CUSTOMER');
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual('');

    state.setWcbState('');
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual('');

    state.setWcbState('WRONG_NUMBER');
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual(labels.WCB_WRONG_NUMBER);

    state.setWcbState('CONNECTION_ERROR');
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual(labels.WCB_NETWORK_CONNECTION_ERROR);

    state.setWcbState('INTERNAL_ERROR_DIALING_IMPOSSIBLE');
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual(labels.WCB_NETWORK_CONNECTION_ERROR);
  });

  it('should alert if net Connection Down', function () {
    session.isNetConnectionDown = true;
    session.fetchMediaIsRunning = true;
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual(labels.WCB_NETWORK_CONNECTION_ERROR);

    session.isNetConnectionDown = false;
    session.fetchMediaIsRunning = true;
    $scope.updateUI();
    $scope.$digest();
    expect($scope.information).toEqual('');
  });

});
