'use strict';

/**
 * Testing AIPConnector
 * Since tests are only run against PhantomJS, the connector will always be CORSConnector
 */
describe('Test AIPConnector',function() {

  /**
   * Testing fetch
   */
  describe('fetch', function () {

    beforeEach(module('akio.webbox'));

    var AIPConnector, $rootScope, state, session;
    beforeEach(inject(function (_aip_, _$rootScope_, _state_, _session_) {
      AIPConnector = _aip_;
      $rootScope = _$rootScope_;
      state = _state_;
      session = _session_;
    }));

    beforeEach(function () {
      jasmine.Ajax.install();
    });

    afterEach(function () {
      jasmine.Ajax.uninstall();
      AIPConnector = null;
      $rootScope = null;
      state = null;
      session = null;
      window.sessionStorage.clear();

    });


    it('should send a fetch request', function () {

      //make a new state
      state.sync({});

      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });

      // Start chat session
      AIPConnector.openMediaChat();

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 201,
        'content': 'application/json',
        'responseHeaders':{
          'Location':'http://uicdev/akio-cim-aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a'
        }
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      var mediaAndEvents = {
        'MediumAndEvents':{
          'events':{
            'event':[{
              'type':'OperatorJoinedEvent',
              'mediumUuid':'2',
              'sender':{
                'nickName':'Thirion',
                'userType':'OPERATOR'
              },
              'eventId':1
            }]
          },
          'medium':{
            'uuid':'2',
            'estimatedWaitTime':0,
            'status':'OPEN_AVAILABLE',
            'name':'Chat',
            'type':'CHAT'
          }
        }
      };
      // Start fetching media and event
      AIPConnector.fetch();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText' : JSON.stringify(mediaAndEvents)
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      expect(JSON.parse(jasmine.Ajax.requests.mostRecent().responseText)).toEqual(mediaAndEvents);
    });

    it('should send a fetch request', function () {

      //make a new state
      state.sync({});

      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });

      // Start chat session
      AIPConnector.openMediaChat();

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 201,
        'content': 'application/json',
        'responseHeaders':{
          'Location':'http://uicdev/akio-cim-aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a'
        }
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      var mediaAndEvents = {
        'MediumAndEvents':{
          'events':{
            'event':[{
              'type':'MessageEvent',
              'mediumUuid':'2',
              'sender':{
                'nickName':'Thirion',
              },
              'eventId':1
            }]
          },
          'medium':{
            'uuid':'2',
            'estimatedWaitTime':0,
            'status':'CLOSED',
            'name':'Chat',
            'type':'CHAT'
          }
        }
      };
      // Start fetching media and event
      AIPConnector.fetch();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText' : JSON.stringify(mediaAndEvents)
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      expect(JSON.parse(jasmine.Ajax.requests.mostRecent().responseText)).toEqual(mediaAndEvents);
    });




    it('should stop fetchEvents and start fetchMedia when dissuaded', function () {

      //make a new state
      state.sync({});

      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });


      // Start chat session
      AIPConnector.openMediaChat();

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 201,
        'content': 'application/json',
        'responseHeaders':{
          'Location':'http://uicdev/akio-cim-aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a'
        }
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      var mediaAndEvents = {


        'events':[{
          'type':'OpenMediumResultEvent',
          'status':'DISSUADED',
          'mediumUuid':'2',
          'eventId':1
        }],
        'medium':{
          'uuid':'2',
          'estimatedWaitTime':0,
          'status':'OPEN_AVAILABLE',
          'name':'Chat',
          'type':'CHAT'
        }

      };
      // Start fetching media and event
      AIPConnector.fetch();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText' : JSON.stringify(mediaAndEvents)
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      expect(session.fetchEventsIsRunning).toEqual(false);
      expect(session.fetchMediaIsRunning).toEqual(true);
    });

    it('should not stop fetchEvents and start fetchMedia when dissuaded', function () {

      //make a new state
      state.sync({});

      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });


      // Start chat session
      AIPConnector.openMediaChat();

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 201,
        'content': 'application/json',
        'responseHeaders':{
          'Location':'http://uicdev/akio-cim-aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a'
        }
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      var mediaAndEvents = {


        'events':[{
          'type':'MessageEvent',
          'status':'OK',
          'mediumUuid':'2',
          'eventId':1,
          'message':'au revoir'
        }],
        'medium':{
          'uuid':'2',
          'estimatedWaitTime':0,
          'status':'CLOSED',
          'name':'Chat',
          'type':'CHAT'
        }

      };
      // Start fetching media and event
      AIPConnector.fetch();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText' : JSON.stringify(mediaAndEvents)
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();
      $rootScope.$digest();

      expect(session.fetchEventsIsRunning).toEqual(true);
      expect(session.fetchMediaIsRunning).toEqual(false);
    });

    it('should work with MediumClosedEvent', function () {

      //make a new state
      state.sync({});

      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });


      // Start chat session
      AIPConnector.openMediaChat();

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 201,
        'content': 'application/json',
        'responseHeaders':{
          'Location':'http://uicdev/akio-cim-aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a'
        }
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      var mediaAndEvents = {


        'events':[{
          'type':'MediumClosedEvent',
          'status':'OK',
          'mediumUuid':'2',
          'eventId':1,
          'message':'au revoir'
        }],
        'medium':{
          'uuid':'4',
          'estimatedWaitTime':0,
          'status':'OPEN_AVAILABLE',
          'name':'Chat',
          'type':'CHAT'
        }

      };
      // Start fetching media and event
      AIPConnector.fetch();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText' : JSON.stringify(mediaAndEvents)
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();
      $rootScope.$digest();

      expect(session.fetchEventsIsRunning).toEqual(true);
      expect(session.fetchMediaIsRunning).toEqual(false);
    });

    it('should not fetch with a medium status CLOSED', function () {

      //make a new state
      state.sync({});

      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });


      // Start chat session
      AIPConnector.openMediaChat();

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 201,
        'content': 'application/json',
        'responseHeaders':{
          'Location':'http://uicdev/akio-cim-aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a'
        }
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      var mediaAndEvents = {


        'events':[{
          'type':'MediumClosedEvent',
          'status':'OK',
          'mediumUuid':'2',
          'eventId':1,
          'message':'au revoir'
        }],
        'medium':{
          'uuid':'4',
          'estimatedWaitTime':0,
          'status':'CLOSED',
          'name':'Chat',
          'type':'CHAT'
        }

      };
      // Start fetching media and event
      AIPConnector.fetch();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText' : JSON.stringify(mediaAndEvents)
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();
      $rootScope.$digest();

      expect(session.fetchEventsIsRunning).toEqual(false);
      expect(session.fetchMediaIsRunning).toEqual(true);
    });

    it('should work with OpenMediumResultEvent', function () {

      //make a new state
      state.sync({});

      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });


      // Start chat session
      AIPConnector.openMediaChat();

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 201,
        'content': 'application/json',
        'responseHeaders':{
          'Location':'http://uicdev/akio-cim-aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a'
        }
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      var mediaAndEvents = {


        'events':[{
          'type':'OpenMediumResultEvent',
          'status':'OK',
          'mediumUuid':'2',
          'eventId':1,
          'message':'au revoir'
        }],
        'medium':{
          'uuid':'4',
          'estimatedWaitTime':0,
          'status':'OPEN_AVAILABLE',
          'name':'Chat',
          'type':'CHAT'
        }

      };
      // Start fetching media and event
      AIPConnector.fetch();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText' : JSON.stringify(mediaAndEvents)
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();
      $rootScope.$digest();

      expect(session.fetchEventsIsRunning).toEqual(true);
      expect(session.fetchMediaIsRunning).toEqual(false);
    });


  });
});

