'use strict';

/**
 * Testing AIPConnector
 * Since tests are only run against PhantomJS, the connector will always be CORSConnector
 */
describe('Test AIPConnector', function () {

  /**
   * Testing getInitData on a new session
   */
  describe('getInitData on a new session', function () {

    beforeEach(module('akio.webbox'));

    var AIPConnector, $rootScope, state, session;
    beforeEach(inject(function (_aip_, _$rootScope_, _state_, _session_) {
      AIPConnector = _aip_;
      $rootScope = _$rootScope_;
      state = _state_;
      session = _session_;
    }));

    beforeEach(function () {
      // The webbox is calling window.parent.akioTargetingRun, so we copy the function to window.parent
      window.parent.akioTargetingRun = window.akioTargetingRun;
      spyOn(window.parent, 'akioTargetingRun').and.callThrough();
      jasmine.Ajax.install();
      jasmine.clock().install();
      state.sync({});

      // Mock isOngoinChatSession to run targeting rules every time
      spyOn(session, 'isOngoingChatSession').and.callFake(function() {
        return false;
      });
      // Mock start polling
      spyOn(session, 'startPolling');
    });

    afterEach(function () {
      jasmine.clock().uninstall();
      jasmine.Ajax.uninstall();
      // Reset spies
      window.parent.akioTargetingRun.calls.reset();
      session.isOngoingChatSession.calls.reset();
      session.startPolling.calls.reset();
      sessionStorage.clear();

      AIPConnector = null;
      $rootScope = null;
      state = null;
      session = null;
    });


    it('should evaluate correctly multiple rules with a timer which condition evaluation changes during the test - change after the end of the timer', function () {
      AIPConnector.getInitData();

      window.testVariable = 0;
      window.testVariable2 = 0;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ALL',
          'conditions': [
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334cf',
              'name': 'Javascript-DOM2',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable2 == 0;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }'
            },
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }'
            }
          ],
          'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279',
          'name':'1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(state.get().get('targetingRules').display).toBeFalsy();

      // Change value before the end of the timer
      jasmine.clock().tick(5000);

      // Change the condition in order to always have a falsy condition !
      window.testVariable2 = 10;
      window.testVariable = 10;

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(3001);

      expect(state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(state.get().get('targetingRules').targetingConditionUuid, '')).not.toBeTruthy();

      // Change value before the end of the timer
      jasmine.clock().tick(5000);

      window.testVariable2 = 0;

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(3001);

      expect(state.get().get('targetingRules').display).toBeTruthy();
      expect(state.get().get('targetingRules').targetingConditionUuid).not.toEqual([{ conditionUuid: '', calendarUuid: 'ad491898-22e9-491a-aa7e-79dcef391279' }]);
      expect(session.startPolling).toHaveBeenCalled();

      window.testVariable = undefined;
      window.testVariable2 = undefined;
    });

    it('should go to error function', function () {
      AIPConnector.getInitData();



      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 503
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      expect(window.parent.akioTargetingRun).not.toHaveBeenCalled();
    });

  });

  /**
   * Testing getInitData with an existing session
   */
  describe('getInitData with an existing session', function () {

    beforeEach(module('akio.webbox'));

    var AIPConnector, $rootScope, state, session;
    beforeEach(inject(function (_aip_, _$rootScope_, _state_, _session_) {
      AIPConnector = _aip_;
      $rootScope = _$rootScope_;
      state = _state_;
      session = _session_;
    }));

    beforeEach(function () {
      spyOn(window, 'akioTargetingRun').and.callThrough();
      jasmine.Ajax.install();
      jasmine.clock().install();

      // Mock isOngoinChatSession to run targeting rules every time
      spyOn(session, 'isOngoingChatSession').and.callFake(function () {
        return true;
      });
      // Mock start polling
      spyOn(session, 'startPolling');
    });

    afterEach(function () {
      jasmine.clock().uninstall();
      jasmine.Ajax.uninstall();
      // Reset spies
      window.akioTargetingRun.calls.reset();
      session.isOngoingChatSession.calls.reset();
      session.startPolling.calls.reset();
    });

    it('should start the polling without evaluating the targeting rules', function () {

      state.sync({
        targetingRules    : {
          targetingConditionUuid : '',
          display : true
        }
      });


      AIPConnector.getInitData();

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ANY',
          'conditions': [
            {
              'uuid': '813a6a95-cf1b-4ef7-bc69-ef01162ea978',
              'name': 'URL-http://localhost:10001/context.html',
              'script': 'function(callback) { callback(document.URL == "http://localhost:10001/context.html"); }'
            }
          ],
          'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279',
          'name':'1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      expect(window.akioTargetingRun).not.toHaveBeenCalled();

      expect(state.get().get('targetingRules').display).toBeTruthy();
      expect(_.isEqual(state.get().get('targetingRules').targetingConditionUuid, '')).toBeTruthy();
      expect(session.startPolling).not.toHaveBeenCalled();

    });

  });

});
