'use strict';

/**
 * Testing AIPConnector
 * Since tests are only run against PhantomJS, the connector will always be CORSConnector
 */
describe('Test AIPConnector', function () {

  /**
   * Testing getInitData on a new session
   */
  describe('getInitData on a new session', function () {

    beforeEach(module('akio.webbox'));

    //this.AIPConnector;
    //this.$rootScope;
    //this.state;
    //this.session;
    beforeEach(inject(function (_aip_, _$rootScope_, _state_, _session_) {
      this.AIPConnector = _aip_;
      this.$rootScope = _$rootScope_;
      this.state = _state_;
      this.session = _session_;
    }));

    beforeEach(function () {
      window.parent.akioTargetingRun = window.akioTargetingRun;
      spyOn(window.parent, 'akioTargetingRun').and.callThrough();
      jasmine.Ajax.install();
      jasmine.clock().install();
      this.state.sync({});

      // Mock isOngoinChatSession to run targeting rules every time
      spyOn(this.session, 'isOngoingChatSession').and.callFake(function() {
        return false;
      });
      // Mock start polling
      spyOn(this.session, 'startPolling');
    });

    afterEach(function () {
      jasmine.clock().uninstall();
      jasmine.Ajax.uninstall();
      // Reset spies
      window.parent.akioTargetingRun.calls.reset();
      this.session.isOngoingChatSession.calls.reset();
      this.session.startPolling.calls.reset();
      this.AIPConnector = null;
      this.$rootScope = null;
      this.state = null;
      this.session = null;
      window.sessionStorage.clear();

    });

    it('should evaluate correctly a truthy URL rule', function () {
      this.state.sync({});

      this.AIPConnector.getInitData();

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ANY',
          'conditions': [
            {
              'uuid': '813a6a95-cf1b-4ef7-bc69-ef01162ea978',
              'name': 'URL-http://localhost:10001/context.html',
              'script': 'function(callback) { callback(document.URL == "http://localhost:10001/context.html"); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'

            }
          ],
          'name': '1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeTruthy();

      expect(this.state.get().get('targetingRules').targetingConditionUuid.uuid).toEqual(undefined);

      expect(this.session.startPolling).toHaveBeenCalled();
    });


    it('should evaluate correctly an falsy URL rule', function () {
      this.state.sync({});

      this.AIPConnector.getInitData();

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ANY',
          'conditions': [
            {
              'uuid': '813a6a95-cf1b-4ef7-bc69-ef01162ea978',
              'name': 'URL-http://uicdev/tenant1/example/',
              'script': 'function(callback) { callback(document.URL == "http://uicdev/tenant1/example/"); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'

            }
          ],
          'name': '1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid, '')).not.toBeTruthy();

      expect(this.session.startPolling).not.toHaveBeenCalled();
    });


    it('should evaluate correctly a timer rule', function () {
      this.state.sync({});

      this.AIPConnector.getInitData();

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ANY',
          'conditions': [
            {
              'uuid': '6c4bed66-27db-4153-b358-a667c4b7d88a',
              'name': 'Temps-10',
              'script': 'function(callback) { window.setTimeout(function() { callback(true); }, 10 * 1000); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'

            }
          ],
          'name': '1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      expect(this.session.startPolling).not.toHaveBeenCalled();

      // Check 1ms before the timeout
      jasmine.clock().tick(10000);

      expect(this.state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid, '')).toBeTruthy();

      expect(this.session.startPolling).not.toHaveBeenCalled();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeTruthy();
      expect(this.state.get().get('targetingRules').targetingConditionUuid.uuid).toEqual(undefined);
      expect(this.session.startPolling).toHaveBeenCalled();
    });


    it('should evaluate correctly a truthy custom rule', function () {
      this.state.sync({});

      this.AIPConnector.getInitData();

      window.testVariable = 10;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ANY',
          'conditions': [
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'

            }
          ],
          'name': '1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeTruthy();
      expect(this.state.get().get('targetingRules').targetingConditionUuid.uuid).toEqual(undefined);
      expect(this.session.startPolling).toHaveBeenCalled();

      window.testVariable = undefined;
    });


    it('should evaluate correctly a falsy custom rule', function () {
      this.state.sync({});
      this.AIPConnector.getInitData();

      window.testVariable = 0;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ANY',
          'conditions': [
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'
            }
          ],
          'name': '1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid, '')).toBeTruthy();
      expect(this.session.startPolling).not.toHaveBeenCalled();

      window.testVariable = undefined;
    });


    it('should evaluate correctly multiple rules when validation is ANY and every condition is truthy', function () {

      this.state.sync({});

      this.AIPConnector.getInitData();

      window.testVariable = 10;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ANY',
          'conditions': [
            {
              'uuid': '813a6a95-cf1b-4ef7-bc69-ef01162ea978',
              'name': 'URL-http://localhost:10001/context.html',
              'script': 'function(callback) { callback(document.URL == "http://localhost:10001/context.html"); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'

            },
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'

            }
          ],
          'name': '1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeTruthy();
      expect(this.state.get().get('targetingRules').targetingConditionUuid.uuid).toEqual(undefined);
      expect(this.session.startPolling).toHaveBeenCalled();

      window.testVariable = undefined;
    });


    it('should evaluate correctly multiple rules when validation is ANY and at least one condition is truthy but not every condition', function () {
      this.state.sync({});


      this.AIPConnector.getInitData();

      window.testVariable = 10;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ANY',
          'conditions': [
            {
              'uuid': '813a6a95-cf1b-4ef7-bc69-ef01162ea978',
              'name': 'URL-http://uicdev/tenant1/example/',
              'script': 'function(callback) { callback(document.URL == "http://uicdev/tenant1/example/"); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'

            },
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'

            }
          ],
          'name': '1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeTruthy();
      expect(this.state.get().get('targetingRules').targetingConditionUuid.uuid).toEqual(undefined);
      expect(this.state.get().get('targetingRules').targetingConditionUuid.uuid).not.toEqual('813a6a95-cf1b-4ef7-bc69-ef01162ea978');
      expect(this.session.startPolling).toHaveBeenCalled();

      window.testVariable = undefined;
    });


    it('should evaluate correctly multiple rules when validation is ANY and every condition is falsy', function () {
      this.state.sync({});

      this.AIPConnector.getInitData();

      window.testVariable = 0;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ANY',
          'conditions': [
            {
              'uuid': '813a6a95-cf1b-4ef7-bc69-ef01162ea978',
              'name': 'URL-http://uicdev/tenant1/example/',
              'script': 'function(callback) { callback(document.URL == "http://uicdev/tenant1/example/"); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'

            },
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'

            }
          ],
          'name': '1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid, '')).not.toBeTruthy();
      expect(this.session.startPolling).not.toHaveBeenCalled();

      window.testVariable = undefined;
    });


    it('should evaluate correctly multiple rules when validation is ALL and every condition is truthy', function () {
      this.state.sync({});
      this.AIPConnector.getInitData();

      window.testVariable = 10;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ALL',
          'conditions': [
            {
              'uuid': '813a6a95-cf1b-4ef7-bc69-ef01162ea978',
              'name': 'URL-http://localhost:10001/context.html',
              'script': 'function(callback) { callback(document.URL == "http://localhost:10001/context.html"); }'
            },
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }'
            }
          ],
          'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279',
          'name':'1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeTruthy();
      expect(this.state.get().get('targetingRules').targetingConditionUuid).not.toEqual([{  conditionUuid: '', calendarUuid: 'ad491898-22e9-491a-aa7e-79dcef391279'  }]);
      expect(this.session.startPolling).toHaveBeenCalled();

      window.testVariable = undefined;
    });


    it('should evaluate correctly multiple rules when validation is ALL and at least one condition is truthy but not every condition', function () {
      this.state.sync({});
      this.AIPConnector.getInitData();

      window.testVariable = 10;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ALL',
          'conditions': [
            {
              'uuid': '813a6a95-cf1b-4ef7-bc69-ef01162ea978',
              'name': 'URL-http://uicdev/tenant1/example/',
              'script': 'function(callback) { callback(document.URL == "http://uicdev/tenant1/example/"); }'
            },
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }'
            }
          ],
          'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279',
          'name':'1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid, '')).not.toBeTruthy();

      expect(this.session.startPolling).not.toHaveBeenCalled();

      window.testVariable = undefined;
    });


    it('should evaluate correctly multiple rules when validation is ALL and every condition is falsy', function () {
      this.state.sync({});
      this.AIPConnector.getInitData();

      window.testVariable = 0;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ALL',
          'conditions': [
            {
              'uuid': '813a6a95-cf1b-4ef7-bc69-ef01162ea978',
              'name': 'URL-http://uicdev/tenant1/example/',
              'script': 'function(callback) { callback(document.URL == "http://uicdev/tenant1/example/"); }'
            },
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }'
            }
          ],
          'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279',
          'name':'1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid, '')).not.toBeTruthy();

      expect(this.session.startPolling).not.toHaveBeenCalled();

      window.testVariable = undefined;
    });


    it('should evaluate correctly a single rule which condition evaluation changes during the test', function () {
      this.state.sync({});
      this.AIPConnector.getInitData();

      window.testVariable = 0;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ANY',
          'conditions': [
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }',
              'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279'
            }
          ],
          'name': '1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Change value after 5s
      jasmine.clock().tick(5000);

      window.testVariable = 10;

      expect(this.state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid, '')).toBeTruthy();

      expect(this.session.startPolling).not.toHaveBeenCalled();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(3001);

      expect(this.state.get().get('targetingRules').display).toBeTruthy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid[0].conditionUuid, '264a9991-4d32-44ed-a0d9-8c1ea17334ce')).toBeTruthy();
      expect(this.session.startPolling).toHaveBeenCalled();

      window.testVariable = undefined;
    });

    it('should evaluate correctly multiple rules with a timer when condition evaluation changes during the test - change before the end of the timer', function () {
      this.state.sync({});
      this.AIPConnector.getInitData();

      window.testVariable = 0;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ALL',
          'conditions': [
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }'
            },
            {
              'uuid': '6c4bed66-27db-4153-b358-a667c4b7d88a',
              'name': 'Temps-10',
              'script': 'function(callback) { window.setTimeout(function() { callback(true); }, 10 * 1000); }'
            }
          ],
          'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279',
          'name':'1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Change value before the end of the timer
      jasmine.clock().tick(5000);

      window.testVariable = 10;

      expect(this.state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid, '')).toBeTruthy();

      jasmine.clock().tick(5000);

      expect(this.state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid, '')).not.toBeTruthy();

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(1);

      expect(this.state.get().get('targetingRules').display).toBeTruthy();
      expect(this.state.get().get('targetingRules').targetingConditionUuid).not.toEqual([{ conditionUuid: '', calendarUuid: 'ad491898-22e9-491a-aa7e-79dcef391279' }]);
      expect(this.session.startPolling).toHaveBeenCalled();

      window.testVariable = undefined;
    });


    it('should evaluate correctly multiple rules with a timer which condition evaluation changes during the test - change after the end of the timer', function () {
      this.state.sync({});
      this.AIPConnector.getInitData();

      window.testVariable = 0;

      var configuration = {
        'webboxRule': {
          'uuid': '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation': 'ALL',
          'conditions': [
            {
              'uuid': '264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name': 'Javascript-DOM',
              'script': 'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }'
            },
            {
              'uuid': '6c4bed66-27db-4153-b358-a667c4b7d88a',
              'name': 'Temps-10',
              'script': 'function(callback) { window.setTimeout(function() { callback(true); }, 10 * 1000); }'
            }
          ],
          'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279',
          'name':'1'
        }
      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText': JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      this.$rootScope.$apply();

      expect(window.parent.akioTargetingRun).toHaveBeenCalled();
      expect(_.isEqual(window.parent.akioTargetingRun.calls.mostRecent().args[0], configuration.webboxRule)).toBeTruthy();

      expect(this.state.get().get('targetingRules').display).toBeFalsy();

      // Change value before the end of the timer
      jasmine.clock().tick(15000);

      expect(this.state.get().get('targetingRules').display).toBeFalsy();
      expect(_.isEqual(this.state.get().get('targetingRules').targetingConditionUuid, '')).not.toBeTruthy();

      window.testVariable = 10;

      // Let akioTargetingRun run its callback
      jasmine.clock().tick(3001);

      expect(this.state.get().get('targetingRules').display).toBeTruthy();
      expect(this.state.get().get('targetingRules').targetingConditionUuid).not.toEqual([{ conditionUuid: '', calendarUuid: 'ad491898-22e9-491a-aa7e-79dcef391279'  }]);
      expect(this.session.startPolling).toHaveBeenCalled();

      window.testVariable = undefined;
    });

  });
});
