'use strict';

/**
 * Testing AIPConnector
 * Since tests are only run against PhantomJS, the connector will always be CORSConnector
 */
describe('Test AIPConnector', function () {

    /**
     * Testing getMedia
     */
    describe('getMedia', function () {

        beforeEach(module('akio.webbox'));

        var AIPConnector, $rootScope, state;
        beforeEach(inject(function (_aip_, _$rootScope_, _state_) {
          AIPConnector = _aip_;
          $rootScope = _$rootScope_;
          state = _state_;
        }));

        beforeEach(function () {
          jasmine.Ajax.install();
          state.sync({});
        });

        afterEach(function () {
          jasmine.Ajax.uninstall();
          AIPConnector = null;
          $rootScope = null;
          state = null;
          window.sessionStorage.clear();

        });

        it('should send a request to get Media Availability, and recept a response from aip', function () {


          AIPConnector.getMedia()
            .then(function(result){
              expect(result).not.toBe(null);
            }
          );



          var response = {
            'Medium': {
              'estimatedWaitTime': 30,
              'uuid': '2',
              'status': 'OPEN_AVAILABLE',
              'name': 'Chat',
              'type': 'CHAT'
            }
          };

          // Mock server response
          jasmine.Ajax.requests.mostRecent().respondWith({
            'status': 200,
            'content': 'application/json',
            'responseText': JSON.stringify(response)
          });

          $rootScope.$apply();

        });

        it('should recept media availability', function () {

          AIPConnector.getMedia()
            .then(function(result){
              state.mediaList.set( result.media );
            }
          );


          var response = [{media:{

            'uuid': '8',
            'name': 'WebCallback',
            'type': 'VOICE'

          }}];

          // Mock server response
          jasmine.Ajax.requests.mostRecent().respondWith({
            'status': 200,
            'content': 'application/json',
            'responseText': JSON.stringify(response)
          });

          // Force ajax response to be processed in AIPConnector
          $rootScope.$apply();

          expect(state.mediaList).not.toBe(null);
          expect(state.mediaList.getMediumByUuid( '8').attributes.name).toEqual('WebCallback');
          expect(state.mediaList.getMediumByUuid( '8').attributes.type).toEqual('VOICE');
        });

        it('should recept media availability', function () {

          AIPConnector.getMedia()
            .then(function(result){
              state.mediaList.set( result.media );
            }
          );


          var response = [{media:{

            'estimatedWaitTime': 25,
            'uuid': '2',
            'status': 'OPEN_AVAILABLE',
            'name': 'Chat',
            'type': 'CHAT'

          }}];

          // Mock server response
          jasmine.Ajax.requests.mostRecent().respondWith({
            'status': 200,
            'content': 'application/json',
            'responseText': JSON.stringify(response)
          });

          // Force ajax response to be processed in AIPConnector
          $rootScope.$apply();

          expect(state.mediaList).not.toBe(null);
          expect(state.mediaList.getMediumByUuid( '2').attributes.status).toEqual('OPEN_AVAILABLE');
          expect(state.mediaList.getMediumByUuid( '2').attributes.name).toEqual('Chat');
          expect(state.mediaList.getMediumByUuid( '2').attributes.type).toEqual('CHAT');
          expect(state.mediaList.getMediumByUuid( '2').attributes.estimatedWaitTime).toEqual(25);
        });

        it('should not recept media availability', function () {

          AIPConnector.getMedia()
            .then(function(result){
              expect(result.method).toEqual('getMedia');
              expect(result.success).toEqual(false);
              expect(result.code).toEqual(0);
            }
          );




          // Mock server response
          jasmine.Ajax.requests.mostRecent().respondWith({
            'status': 503
          });

          // Force ajax response to be processed in AIPConnector
          $rootScope.$apply();

        });

      }
    );
  }
);
