'use strict';

/**
 * Testing AIPConnector
 * Since tests are only run against PhantomJS, the connector will always be CORSConnector
 */
describe('Test AIPConnector',function() {

  /**
   * Testing openMediaChat
   */
  describe('openMediaChat', function () {

    beforeEach(module('akio.webbox'));

    var AIPConnector, $rootScope, state, session;
    beforeEach(inject(function (_aip_, _$rootScope_, _state_, _session_) {
      AIPConnector = _aip_;
      $rootScope = _$rootScope_;
      state = _state_;
      session = _session_;
    }));

    beforeEach(function () {
      state.targetingRule = {
        profileParameters: [],
        display: false
      };
      jasmine.Ajax.install();
      jasmine.clock().install();

      //make a new state
      state.sync({});

      // Mock isOngoinChatSession to run targeting rules every time
      spyOn(session, 'isOngoingChatSession').and.callFake(function() {
        return false;
      });
      // Mock start polling
      spyOn(session, 'startPolling');
    });

    afterEach(function () {
      jasmine.clock().uninstall();
      jasmine.Ajax.uninstall();
      // Reset spies
      session.isOngoingChatSession.calls.reset();
      session.startPolling.calls.reset();

      AIPConnector = null;
      $rootScope = null;
      state = null;
      session = null;
    });

    it('should send the targeting rules profile parameters when starting a chat session', function () {

      AIPConnector.getInitData();

      window.testVariable = 10;

      var configuration = {

        'webboxRule' : {
          'uuid' : '40e7419f-f62a-4937-a393-722d4e66f2eb',
          'validation' : 'ALL',
          'conditions' : [
            {
              'uuid' : '813a6a95-cf1b-4ef7-bc69-ef01162ea978',
              'name':'URL-http://localhost:10001/context.html',
              'script':'function(callback) { callback(document.URL == "http://localhost:10001/context.html"); }'
            },
            {
              'uuid':'264a9991-4d32-44ed-a0d9-8c1ea17334ce',
              'name':'Javascript-DOM',
              'script':'function(callback) { var lastResult = false; var check = function() { var result = eval("testVariable == 10;"); if (lastResult != result) { lastResult = result; callback(result); } }; check(); window.setInterval(check, 3000); }'
            },
            {
              'uuid':'6c4bed66-27db-4153-b358-a667c4b7d88a',
              'name':'Temps-10',
              'script':'function(callback) { window.setTimeout(function() { callback(true); }, 10 * 1000); }'
            }
          ],
          'calendarUUID':'ad491898-22e9-491a-aa7e-79dcef391279',
          'name':'1'
        }

      };

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status' : 200,
        'content' : 'application/json',
        'responseText' : JSON.stringify(configuration)
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      // Wait until the timeout
      jasmine.clock().tick(10001);

      // Check if we have profile parameters for the targeting rule
      var targetingRuleProfileParameters = state.get().get('targetingRules');
      var expectedResult = {
        targetingConditionUuid: [{
          conditionUuid: '',
          calendarUuid: 'ad491898-22e9-491a-aa7e-79dcef391279'
        }],
        display: true
      };

      expect(targetingRuleProfileParameters).not.toEqual(expectedResult);

      // Start chat session
      AIPConnector.openMediaChat();

      // Get ajax request parameters
      var ajaxRequestParams = JSON.parse(jasmine.Ajax.requests.mostRecent().params);
      console.log(ajaxRequestParams);
      expect(ajaxRequestParams.targetingConditionUuid).toEqual('');

      window.testVariable = undefined;
    });

    it('should stop the fetch media and start the fetch event', function () {
      // Start chat session
      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });


      AIPConnector.openMediaChat();


      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 201,
        'content': 'application/json',
        'responseHeaders':{
          'Location':'http://uicdev/aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a'
        }
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      expect(session.fetchMediaIsRunning).toBe(false);
      expect(session.fetchEventsIsRunning).toBe(true);
      expect(state.get( ).get( 'sessionUri' )).toEqual('http://uicdev/aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a');

    });

    it('should get the the sessionUri and save it in the state', function () {

      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });


      // Start chat session
      AIPConnector.openMediaChat();

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 201,
        'content': 'application/json',
        'responseHeaders':{
          'Location':'http://uicdev/aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a'
        }
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      var sessionUri =   state.get( ).get( 'sessionUri' );
      expect(sessionUri).toEqual('http://uicdev/aip/api/v1/chat/46c0b46b-6c81-4b2f-bdb3-267ce4f6581a');

    });

    it('should get an error if aip respond with 503', function () {
      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });


      AIPConnector.openMediaChat().then(function(response){
        expect(response.code).toEqual(0);
        expect(response.success).toEqual(false);
        expect(response.method).toEqual('openMediaChat');
      });

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 503

      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

    });
  });
});
