'use strict';

/**
 * Testing AIPConnector
 * Since tests are only run against PhantomJS, the connector will always be CORSConnector
 */
describe('Test AIPConnector',function() {

  /**
   * Testing openMediaWcb
   */
  describe('openMediaWcb', function () {

    beforeEach(module('akio.webbox'));

    var AIPConnector, $rootScope, state, session;
    beforeEach(inject(function (_aip_, _$rootScope_, _state_, _session_) {
      AIPConnector = _aip_;
      $rootScope = _$rootScope_;
      state = _state_;
      session = _session_;
    }));

    beforeEach(function () {
      state.targetingRule = {
        profileParameters: [],
        display: false
      };
      jasmine.Ajax.install();
      jasmine.clock().install();

      //make a new state
      state.sync({});

      // Mock isOngoinChatSession to run targeting rules every time
      spyOn(session, 'isOngoingChatSession').and.callFake(function() {
        return false;
      });
      // Mock start polling
      spyOn(session, 'startPolling');
    });

    afterEach(function () {
      jasmine.clock().uninstall();
      jasmine.Ajax.uninstall();
      // Reset spies
     // session.isOngoingWcbSession.calls.reset();
      session.startPolling.calls.reset();

      AIPConnector = null;
      $rootScope = null;
      state = null;
      session = null;
    });


    it('should get the the sessionUri and save it in the state', function () {

      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });


      // Start chat session
      AIPConnector.openMediaWcb('FR','0661764184');

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200,
        'content': 'application/json',
        'responseText':JSON.stringify({'sessionUuid':'123456789'})
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      var sessionUuid =   state.get( ).get( 'sessionUri' );
      console.log(sessionUuid);
      console.log(state);

      expect(sessionUuid).toEqual('http://uicdev/akio-cim-aip/api/v1/webcallback/status/123456789');

    });

    it('should get an error if aip respond with 503', function () {
      state.data.save({
        targetingRules : {
          targetingConditionUuid: [{conditionUuid:'',calendarUuid:''}] ,
          display : true
        }
      });


      AIPConnector.openMediaWcb().then(function(response){
        expect(response.code).toEqual(0);
        expect(response.success).toEqual(false);
        expect(response.method).toEqual('openMediaWcb');
      });

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 503

      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

    });
  });
});
