'use strict';

/**
 * Testing AIPConnector
 * Since tests are only run against PhantomJS, the connector will always be CORSConnector
 */
describe('Test AIPConnector',function() {

  /**
   * Testing sendMessage
   */
  describe('sendMessage', function () {

    beforeEach(module('akio.webbox'));

    var AIPConnector, $rootScope, state, session, ViewState;
    beforeEach(inject(function (_aip_, _$rootScope_, _state_, _session_, _ViewState_) {
      AIPConnector = _aip_;
      $rootScope = _$rootScope_;
      state = _state_;
      session = _session_;
      ViewState = _ViewState_;
    }));

    beforeEach(function () {
      jasmine.Ajax.install();

    });

    afterEach(function () {
      jasmine.Ajax.uninstall();
      sessionStorage.clear();
    });

    it('should send one message', function () {

      //make a new state
      state.sync({});

      var mediaResult = {
        description : 'no description',
        estimatedWaitTime : 9,
        exposed :  true,
        icon : 'fa-comments-o',
        message : 'Ask us by Chat !',
        name : 'Chat',
        priority : 1,
        renderer : 'BUTTON',
        status : 'OPEN_AVAILABLE',
        textInput : true,
        title : 'Chat295',
        type : 'CHAT',
        uuid :'2'
      };

      //set the media in the state
      state.mediaList.set(mediaResult);

      //force ViewState for make injectMessage works
      ViewState.STATE_NONE = '2';

      // Start chat session
      AIPConnector.sendMessage('this is a test');

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 204,
        'content': 'application/json'
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();

      expect(state.get().attributes.sendMessageCount).toEqual(1);
      expect(state.get().attributes.events.models[0].attributes.message).toEqual('this is a test');


    });

    it('should get an error if AIP respond with 503', function () {

      //make a new state
      state.sync({});

      var mediaResult = {
        description : 'no description',
        estimatedWaitTime : 9,
        exposed :  true,
        icon : 'fa-comments-o',
        message : 'Ask us by Chat !',
        name : 'Chat',
        priority : 1,
        renderer : 'BUTTON',
        status : 'OPEN_AVAILABLE',
        textInput : true,
        title : 'Chat295',
        type : 'CHAT',
        uuid :'2'
      };

      //set the media in the state
      state.mediaList.set(mediaResult);

      //force ViewState for make injectMessage works
      ViewState.STATE_NONE = '2';

       //Start chat session
      AIPConnector.sendMessage('this is a test').then(function(response){
        expect(response.method).toEqual('sendInputEvent');
        expect(response.success).toEqual(false);
        expect(response.code).toEqual(5);

      });

      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 503
      });

      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();


    });

  });
});
