'use strict';

describe('test message reader factory',function(){

  beforeEach(module('akio.webbox'));

  var messageReader,state, $rootScope, Event, notification, navigation;


  beforeEach(inject(function(_messageReader_, _state_, _$rootScope_,_Event_, _notification_, _navigation_){
    messageReader = _messageReader_;
    $rootScope = _$rootScope_;
    state = _state_;
    Event = _Event_;
    notification = _notification_;
    navigation = _navigation_;

    spyOn(notification,'error').and.callThrough();
    spyOn(navigation,'leaveMedia').and.callThrough();
    spyOn(navigation,'interpretEvent').and.callThrough();



    state.sync({});
    state.data.save('regions', [{
      mediumType:'CHAT',
      constraint:0,
      mediumStatus:1,
      mediumUuid:'2'
    }]);
    //set a media result
    var mediaResult = {
      description : 'no description',
      estimatedWaitTime : 9,
      exposed :  true,
      icon : 'fa-comments-o',
      message : 'Ask us by Chat !',
      name : 'Chat',
      priority : 1,
      renderer : 'BUTTON',
      status : 'OPEN_AVAILABLE',
      textInput : true,
      title : 'Chat295',
      type : 'CHAT',
      uuid :'2'
    };

    //set the media in the state
    state.mediaList.set(mediaResult);
  }));


  it('should define a factory', function(){
    expect(messageReader).not.toBe(undefined);

  });

  it('should wrap into log message', function(){
    var event = {event:'a fake event'};
    var text = 'a text for the test';
    var result = messageReader.wrapIntoLogMessage(event,text);

    var expectedResult = {
      'type'     : 'LogMessageEvent',
      'original' : {event:'a fake event'},
      'message'  : 'a text for the test',
      'sender'  : {userType: 'SYSTEM', nickName:''},
      'eventId'  : event.eventId,
      'mediumUuid': event.mediumUuid
    };

    expect(result).toEqual(expectedResult);

  });

  it('should handle Response', function(){

    var event0 = new Event();
    event0.type = 'OpenMediumResultEvent';
    event0.mediumUuid = '2';
    event0.status = 'DISSUADED';

    var event1 = new Event();
    event1.type = 'OpenMediumResultEvent';
    event1.mediumUuid = '2';
    event1.status = 'BAD_SERVICE';

    var event2 = new Event();
    event2.type = 'OperatorJoinedEvent';
    event2.mediumUuid = '2';
    event2.status = 'DISSUADED';

    var event3 = new Event();
    event3.type = 'ConversationInactivityEvent';
    event3.mediumUuid = '2';
    event3.status = 'ALERTING';

    var event4 = new Event();
    event4.type = 'OperatorLeftEvent';
    event4.mediumUuid = '2';
    event4.status = 'ALERTING';
    event4.sender = { nickName : 'fakeUser', userType: 'Agent' };

    var event5 = new Event();
    event5.type = 'MediumClosedEvent';
    event5.mediumUuid = '2';
    event5.status = 'ALERTING';
    event5.sender = { nickName : 'fakeUser', userType: 'Agent' };

    var event6 = new Event();
    event6.type = 'TypingStartedEvent';
    event6.mediumUuid = '2';
    event6.status = 'ALERTING';
    event6.sender = { nickName : 'fakeUser', userType: 'Agent' };

    var event7 = new Event();
    event7.type = 'TypingStartedEvent';
    event7.mediumUuid = '2';
    event7.status = 'ALERTING';
    event7.sender = { nickName : 'fakeUser', userType: 'Agent' };

    var event8 = new Event();
    event8.type = 'TypingStoppedEvent';
    event8.mediumUuid = '2';
    event8.status = 'ALERTING';
    event8.sender = { nickName : 'fakeUser', userType: 'Agent' };

    var event9 = new Event();
    event9.type = 'NetworkConnectionLazyEvent';
    event9.mediumUuid = '2';
    event9.status = 'ALERTING';
    event9.sender = { nickName : 'fakeUser', userType: 'Agent' };

    var event10 = new Event();
    event10.type = 'ServiceUnavailableEvent';
    event10.mediumUuid = '2';
    event10.status = 'ALERTING';
    event10.sender = { nickName : 'fakeUser', userType: 'Agent' };

    var event11 = new Event();
    event11.type = 'ConversationRequalificationEvent';
    event11.mediumUuid = '2';
    event11.status = 'ALERTING';
    event11.sender = { nickName : 'fakeUser', userType: 'Agent' };

    var Result = {
      media     : 'CHAT',
      events : [event0, event1, event2, event3, event4, event5, event6, event7, event8, event9, event10, event11]
    };
    messageReader.handleResponse(Result);



    expect(notification.error).toHaveBeenCalled();
    expect(navigation.leaveMedia).toHaveBeenCalled();
    expect(navigation.interpretEvent).toHaveBeenCalled();

  });


});
