'use strict';

describe('test state factory',function(){

  beforeEach(module('akio.webbox'));

  var state, $rootScope, boxParams;


  beforeEach(inject(function(_state_, _$rootScope_,_boxParams_){
    state = _state_;
    $rootScope = _$rootScope_;
    boxParams=_boxParams_;

  }));

  it('should get the title', function() {
    state.setTitle('mon Titre');
    expect(state._title).toEqual('mon Titre');
    var myTitle = state.getTitle();
    expect(myTitle).toEqual('mon Titre');
  });

  it('should create a new State', function(){
    expect(state.data).toBe(undefined);
    state.sync({});
    expect(state.data).not.toBe(undefined);
  });

  it('should tells if box is display as trigger', function(){
    expect(state.isTrigger).not.toBe(undefined);
    state.sync({});
    var isTrigger = state.isTrigger();
    expect(isTrigger).toEqual(false);
  });

  it('should returns the west region description', function(){
    state.sync({});
    var westRegion = state.getWestRegion();
    expect(westRegion).toEqual('2');
  });

  it('should returns a region ', function(){
    state.sync({});
    state.data.save('regions', [{
      mediumType:'CHAT',
      constraint:0,
      mediumStatus:1,
      mediumUuid:'2'
    }]);
    var filter = ['mediumStatus'];
    var result = state.getRegionByMediumType('CHAT',filter);
    expect(result).toEqual(1);
  });

  it('should set region with constraint 0', function(){
    state.sync({});
    var options={constraint:0};
    state.setRegion(options);
    expect(state.data.get('regions')).toEqual([{ mediumStatus: 0, constraint: 0 }]);
  });

  it('should set region with constraint 2', function(){
    state.sync({});
    var options={constraint:2};
    state.setRegion(options);
    expect(state.data.get('regions')).toEqual([{ mediumStatus: 1, constraint: 2 }]);
  });

  it('should set the countries list', function(){
    state.sync({});
    var countriesListToSet=[
      {
        internationalCode:'+33',
        alphaCode:'FR',
        name:'france'
      },
      {
        internationalCode:'+41',
        alphaCode:'CH',
        name:'suisse'
      },
      {
        internationalCode:'+377',
        alphaCode:'MC',
        name:'monaco'
      }
    ];
    state.setCountriesList(countriesListToSet);
    var getCountriesList = state.getCountriesList();
    expect(getCountriesList).toEqual(countriesListToSet);

    boxParams.BOX_COUNTRIES_LIST=['FR','CH'];
    state.setCountriesList(countriesListToSet);
    var getCountriesList2 = state.getCountriesList();
    expect(getCountriesList2).not.toEqual(countriesListToSet);
  });

  it('should start media polling if there is no chat conversation started', function() {
    state.sync({});
    state.setCurrentMediaType('VOICE');

    expect(state.getCurrentMediaType()).toEqual('VOICE');

  });

});
