'use strict';

describe('test session provider', function () {

  describe('session service initialization', function () {

    beforeEach(module('akio.webbox'));

    var $rootScope, state, boxParams, session;
    beforeEach(inject(function (_aip_, _$rootScope_, _state_, _session_, _$injector_, _boxParams_) {
      $rootScope = _$rootScope_;
      state = _state_;
      session = _session_;
      boxParams = _boxParams_;
    }));

    beforeEach(function () {
      jasmine.Ajax.install();
    });

    afterEach(function () {
      jasmine.Ajax.uninstall();
    });

    it('should initialize isNetConnectionDown to false', function () {
      expect(session.isNetConnectionDown).toBeFalsy();
    });

    it('should initialize fetchTimer to null', function () {
      expect(session.fetchTimer).toBeNull();
    });

    it('should initialized the fetchMediaIsRunning to false', function () {
      expect(session.fetchMediaIsRunning).toBeFalsy();
    });

    it('should initialized the fetchEventsIsRunning to false', function () {
      expect(session.fetchEventsIsRunning).toBeFalsy();
    });

    it('should initialize box visible to false', function () {
      expect(session.isBoxVisible()).toBeFalsy();
    });

    it('boxParams should be defined', function () {
      expect(boxParams).toBeDefined();
    });

    it('should interpret error on fetch', function () {
      //make a new state
      state.sync({});

      session.errorPolling = 20;

      expect(session.isNetConnectionDown).toEqual(false);
      expect(session.fetchRetryCounter).toEqual(0);

      // Start fetching media and event
      session.fetch();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 503
      });
      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 503
      });

      session.isTryingToReconnect = true;
      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();
      expect(session.isNetConnectionDown).toEqual(true);
      expect(session.fetchRetryCounter).not.toEqual(0);
    });

    it('should scheduled next fetch', function () {
      //make a new state
      state.sync({});

      session.errorPolling = 20;

      expect(session.isNetConnectionDown).toEqual(false);
      expect(session.fetchRetryCounter).toEqual(0);

      // Start fetching media and event
      session.fetch();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200
      });
      session.isTryingToReconnect = true;
      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();
      expect(session.isNetConnectionDown).toEqual(false);
      expect(session.fetchRetryCounter).toEqual(0);
    });

    it('should interpret error on fetchWcbState', function () {
      //make a new state
      state.sync({});

      session.errorPolling = 20;

      expect(session.isNetConnectionDown).toEqual(false);
      expect(session.fetchRetryCounter).toEqual(0);

      // Start fetching media and event
      session.fetchWcbState();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 503
      });
      session.isTryingToReconnect = true;
      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();
      expect(session.isNetConnectionDown).toEqual(true);
      expect(session.fetchRetryCounter).not.toEqual(0);
    });

    it('should schedule next fetch on fetchWcbState', function () {
      //make a new state
      state.sync({});

      session.errorPolling = 20;

      expect(session.isNetConnectionDown).toEqual(false);
      expect(session.fetchRetryCounter).toEqual(0);

      // Start fetching media and event
      session.fetchWcbState();

      // Mock server response
      jasmine.Ajax.requests.mostRecent().respondWith({
        'status': 200
      });
      session.isTryingToReconnect = true;
      // Force ajax response to be processed in AIPConnector
      $rootScope.$apply();
      expect(session.isNetConnectionDown).toEqual(false);
      expect(session.fetchRetryCounter).toEqual(0);
    });


    it('should set the current media type', function () {
      state.sync({});

      expect(session.isDisplayAsButton()).toEqual(false);
    });

    /*it('should display chat as button', function() {
      boxParams.IS_DISPLAY_AS_BUTTON = 'true';

      state.sync({});
      //AIPConnector.getInitData();
      $rootScope.$apply();

      expect(session.isDisplayAsButton()).toEqual(true);
    });*/
  });

  describe('box polling', function () {

    beforeEach(module('akio.webbox'));

    var session, state, EventCollection, $q, $rootScope;
    beforeEach(inject(function (_session_, _state_, _EventCollection_, _$q_, _$rootScope_) {
      session = _session_;
      state = _state_;
      EventCollection = _EventCollection_;
      $q = _$q_;
      $rootScope = _$rootScope_;
    }));

    beforeEach(function () {
      // Mock fetch request
      spyOn(session, 'fetch').and.callFake(function () {
        var deferred = $q.defer();
        return deferred.promise;
      });
      // Mock fetchMedia request
      spyOn(session, 'fetchMedia').and.callFake(function () {
        var deferred = $q.defer();
        return deferred.promise;
      });

      state.sync({});
    });

    afterEach(function () {
      session.fetch.calls.reset();
      session.fetchMedia.calls.reset();
    });


    it('should start media polling if there is no chat conversation started', function () {
      session.startPolling();

      expect(session.fetchMedia).toHaveBeenCalled();
      expect(session.fetchMediaIsRunning).not.toBeFalsy();
      expect(session.fetch).not.toHaveBeenCalled();
      expect(session.fetchEventsIsRunning).toBeFalsy();
    });

    it('should start event polling if there is already a started conversation', function () {
      // Mock events to simulate an existing session
      spyOn(state, 'events').and.callFake(function () {
        return new EventCollection([{
          'type': 'LogMessageEvent',
          'original': {
            'type': 'OperatorJoinedEvent',
            'mediumUuid': '2',
            'sender': {'nickName': 'nick', 'userType': 'OPERATOR'},
            'eventId': 1
          },
          'message': 'Message',
          'sender': {'userType': 'SYSTEM', 'nickName': ''},
          'eventId': 1,
          'mediumUuid': '2'
        }]);
      });

      session.startPolling();

      expect(session.fetchMedia).not.toHaveBeenCalled();
      expect(session.fetchMediaIsRunning).toBeFalsy();

      expect(session.fetch).toHaveBeenCalled();
      expect(session.fetchEventsIsRunning).not.toBeFalsy();
    });

    it('should start a session', function () {
      var result = session.start();
      expect(result).not.toBe(null);
      //expect(result.$$state.status).toEqual(0);
    });

    it('should get the state', function () {
      var result = session.getState();
      expect(result.data).toBeDefined();
    });

    it('should tells whether the box is authorized on a given page', function () {
      var businessRule = session.businessRule();
      expect(businessRule).toEqual(false);
    });

    it('should tells media chat is not available', function () {
      var mediaOpenAndAvailable = session.isMediaChatOpenAndAvailable();
      expect(mediaOpenAndAvailable).toEqual(false);
    });

    it('should tells if media chat is open and available', function () {
      //set the media in the state
      var mediaResult = {
        conditionUuid:'497a9c9f-1e1d-47f7-bed6-333ffd440f72',
        uuid:'2',
        name:'Chat',
        status:'OPEN_AVAILABLE',
        estimatedWaitTime:25,
        type:'CHAT'
      };
      state.mediaList.set(mediaResult);

      var mediaOpenAndAvailable = session.isMediaChatOpenAndAvailable();
      expect(mediaOpenAndAvailable).toEqual(true);
    });

    it('should tells if media wcb is not available', function () {
      var mediaOpenAndAvailable = session.isMediaWcbOpenAndAvailable();
      expect(mediaOpenAndAvailable).toEqual(false);
    });

    it('should tells if media wcb is open and available for instant webcallback', function () {
      //set the media in the state
      var mediaResult = {
        conditionUuid:'dedd5031-075a-451b-9e93-dfec6e2d1eff',
        uuid:'8',
        name:'Voice',
        status:'OPEN_AVAILABLE',
        estimatedWaitTime:25,
        type:'VOICE'
      };
      state.mediaList.set(mediaResult);

      var mediaOpenAndAvailable = session.isMediaWcbOpenAndAvailable();
      expect(mediaOpenAndAvailable).toEqual(true);
    });

    it('should tells if media wcb is open and available for delayed webcallback', function () {
      //set the media in the state
      var mediaResult = {
        conditionUuid:'dedd5031-075a-451b-9e93-dfec6e2d1eff',
        uuid:'8',
        name:'Voice',
        status:'OPEN_DELAYED_WEBCALLBACK',
        estimatedWaitTime:25,
        type:'VOICE'
      };
      state.mediaList.set(mediaResult);

      var mediaOpenAndAvailable = session.isMediaWcbOpenAndAvailable();
      expect(mediaOpenAndAvailable).toEqual(true);
    });

    it('should add blur/focus events function to manage polling', function () {
      session.addFocusBlurListener();
      $rootScope.$digest();
      window.onblur();
      window.onfocus();

      expect(window.parent.window.onblur).toBeDefined();
      expect(window.parent.window.onfocus).toBeDefined();
      expect(window.onblur).toBeDefined();
      expect(window.onblur).toBeDefined();
    });

    it('should tells if conditions are reunited to display the box', function () {
      var targetRule = session.targetRule();
      expect(targetRule).toEqual(false);
    });

    it('should tells if conditions are reunited to display the box', function () {
      session.isInitialized = true;
      var isBoxVisible = session.isBoxVisible();
      expect(isBoxVisible).toEqual(false);
    });

    it('should tells whether there is an ongoing session meanings that the box is opened', function () {
      var isBoxOpened = session.isBoxOpened();
      expect(isBoxOpened).toEqual(false);
    });

    it('should tells whether there is currently an ongoing chat session', function () {
      var result = session.isOngoingChatSession();
      expect(result).toEqual(false);
    });

    it('should tells whether there is currently an ongoing wcb session', function () {
      var result = session.isOngoingWcbSession();
      $rootScope.$apply();
      expect(result).toEqual(false);
    });

    it('should try to catchup the interrupted session', function () {
      session.catchup();
      expect(session.catchup()).toBeDefined();
    });

    it('should create a new session', function () {
      session.renew();
      expect(session.renew).toBeDefined();
    });

    it('should stop a session', function () {
      session.stop();
      expect(session.stop).toBeDefined();
    });

    it('should add delay to original delay', function () {
      var newDelay = session.addDelay(2);
      expect(newDelay).toEqual(1);
    });
  });
});
