'use strict';

describe('test time picker utils functions',function(){

  var exampleTimestamp = 1588171056000; // 29/04/2020 16:37:36

  beforeEach(module('akio.webbox'));

  var timePickerUtils, boxParams;
  beforeEach(inject(function(_timePickerUtils_, _boxParams_){
    timePickerUtils = _timePickerUtils_;
    boxParams = _boxParams_;
    boxParams.CHAT_FORM = [
      { type: 'title', textKey: 'CHAT_FORM_TITLE' },
      { type: 'button', textKey: 'CHAT_FORM_VALID_BUTTON'},
      { type:'field', mapping: 'frm_adresse_email_1'},
      { type:'buttonWCB', textKey : 'WCB_FORM_VALID_BUTTON'}
    ];
  }));

  it('should return an object', function() {
    expect(timePickerUtils).toEqual((jasmine.any(Object)));
  });

  it('should clone a date', function() {
    var date = new Date();
    expect(timePickerUtils.cloneDate(date)).not.toBe(date);
  });

  // PhantomJS doesn't support short names for months !
  it('should format date for locale FR', function() {
    boxParams.I18N = '"fr-FR"';
    var date = new Date(exampleTimestamp);
    expect(timePickerUtils.formatDate(date)).toEqual('mercredi 29 avril 2020');
  });

  // PhantomJS doesn't support short names for months !
  it('should format date for locale US', function() {
    boxParams.I18N = '"en-US"';
    var date = new Date(exampleTimestamp);
    expect(timePickerUtils.formatDate(date)).toEqual('Wednesday, April 29, 2020');
  });

  it('should format time for locale FR', function() {
    boxParams.I18N = '"fr-FR"';
    var date = new Date(exampleTimestamp);
    expect(timePickerUtils.formatTime(date)).toEqual('16:37');
  });

  it('should format time for locale US', function() {
    boxParams.I18N = '"en-US"';
    var date = new Date(exampleTimestamp);
    expect(timePickerUtils.formatTime(date)).toEqual('4:37 PM');
  });

  it('should create a timeslot', function() {
    boxParams.I18N = '"fr-FR"';
    var timeslot = {
      fromDate: exampleTimestamp,
      toDate: exampleTimestamp + 1000000,
      opened: true,
      available: true
    };
    expect(timePickerUtils.createTimeslot(timeslot)).toEqual({
      date: exampleTimestamp,
      endDate: exampleTimestamp + 1000000,
      display: '16:37',
      opened: true,
      available: true
    });
  });

  it('should sort timeslots by day', function() {
    var timeslots = [
      { fromDate: 1592949700000, toDate: 1592949800000, opened: true, available: true }, // 24/06/2020
      { fromDate: 1593036100000, toDate: 1593036200000, opened: true, available: true }, // 25/06/2020
      { fromDate: 1593036200000, toDate: 1593036300000, opened: true, available: true }, // 25/06/2020
      { fromDate: 1593122500000, toDate: 1593122600000, opened: true, available: true }, // 26/06/2020
      { fromDate: 1593208900000, toDate: 1593209000000, opened: true, available: true } // 27/06/2020
    ];
    var sortedTimeslots = timePickerUtils.sortTimeslotsByDay(timeslots);

    var dates = Object.keys(sortedTimeslots);
    expect(dates.length).toEqual(4);
    timeslots.forEach(function(timeslot) {
      var expectedValue = new Date(timeslot.fromDate);
      expectedValue.setHours(0, 0, 0, 0);
      var dayTimestamp = expectedValue.getTime().toString();
      var dateIndex = dates.indexOf(dayTimestamp);
      var createdTimeslot = timePickerUtils.createTimeslot(timeslot);

      // Timeslot is in its day array
      expect(dateIndex).not.toEqual(-1);
      expect(_.some(sortedTimeslots[dayTimestamp], createdTimeslot)).toBeTruthy();
    });
  });

  it('should return the first 24 opened timeslots for the current day', function() {
    var timeslots = [
      { fromDate: 1593036200000, toDate: 1593036200001, opened: true, available: true },
      { fromDate: 1593036200001, toDate: 1593036300002, opened: true, available: true },
      { fromDate: 1593036200002, toDate: 1593036300003, opened: true, available: true },
      { fromDate: 1593036200003, toDate: 1593036300004, opened: true, available: true },
      { fromDate: 1593036200004, toDate: 1593036300005, opened: true, available: true },
      { fromDate: 1593036200005, toDate: 1593036300006, opened: true, available: true },
      { fromDate: 1593036200006, toDate: 1593036300007, opened: true, available: true },
      { fromDate: 1593036200007, toDate: 1593036300008, opened: true, available: true },
      { fromDate: 1593036200008, toDate: 1593036300009, opened: true, available: true },
      { fromDate: 1593036200009, toDate: 1593036300010, opened: true, available: true },
      { fromDate: 1593036200010, toDate: 1593036300011, opened: true, available: true },
      { fromDate: 1593036200011, toDate: 1593036300012, opened: true, available: true },
      { fromDate: 1593036200012, toDate: 1593036300013, opened: true, available: true },
      { fromDate: 1593036200013, toDate: 1593036300014, opened: true, available: true },
      { fromDate: 1593036200014, toDate: 1593036300015, opened: true, available: true },
      { fromDate: 1593036200015, toDate: 1593036300016, opened: true, available: true },
      { fromDate: 1593036200016, toDate: 1593036300017, opened: true, available: true },
      { fromDate: 1593036200017, toDate: 1593036300018, opened: true, available: true },
      { fromDate: 1593036200018, toDate: 1593036300019, opened: true, available: true },
      { fromDate: 1593036200019, toDate: 1593036300020, opened: true, available: true },
      { fromDate: 1593036200020, toDate: 1593036300021, opened: true, available: true },
      { fromDate: 1593036200021, toDate: 1593036300022, opened: true, available: true },
      { fromDate: 1593036200022, toDate: 1593036300023, opened: true, available: true },
      { fromDate: 1593036200023, toDate: 1593036300024, opened: true, available: true },
      { fromDate: 1593036200024, toDate: 1593036300025, opened: true, available: true },
      { fromDate: 1593036200025, toDate: 1593036300026, opened: true, available: true },
      { fromDate: 1593036200026, toDate: 1593036300027, opened: true, available: true },
      { fromDate: 1593036200027, toDate: 1593036300028, opened: true, available: true },
      { fromDate: 1593036200028, toDate: 1593036300029, opened: true, available: true },
      { fromDate: 1593036200029, toDate: 1593036300030, opened: true, available: true }
    ];
    var sortedTimeslots = timePickerUtils.sortTimeslotsByDay(timeslots);
    var hours = timePickerUtils.getHours(sortedTimeslots[Object.keys(sortedTimeslots)[0]]);

    expect(hours.length).toBeLessThanOrEqual(24);
  });

  it('should not return closed timeslots for the current day', function() {
    var timeslots = [
      { fromDate: 1593036200000, toDate: 1593036200001, opened: false, available: true }
    ];
    var sortedTimeslots = timePickerUtils.sortTimeslotsByDay(timeslots);
    var hours = timePickerUtils.getHours(sortedTimeslots[Object.keys(sortedTimeslots)[0]]);

    expect(hours.length).toBe(0);
  });

  it('should return the first opened timeslot if there is one', function() {
    var timeslots = [
      { fromDate: 1592949700000, toDate: 1592949800000, opened: false, available: true }, // 24/06/2020
      { fromDate: 1593036100000, toDate: 1593036200000, opened: true, available: true } // 25/06/2020
    ];
    var result = timePickerUtils.getFirstDisplayedDate(timeslots);
    var expectedResult = new Date(1593036100000);
    expectedResult.setHours(0, 0, 0, 0);

    expect(result).toEqual(expectedResult);
  });

  it('should return the first timeslot if there is no opened timeslot', function() {
    var timeslots = [
      { fromDate: 1592949700000, toDate: 1592949800000, opened: false, available: true }, // 24/06/2020
      { fromDate: 1593036100000, toDate: 1593036200000, opened: false, available: true } // 25/06/2020
    ];
    var result = timePickerUtils.getFirstDisplayedDate(timeslots);
    var expectedResult = new Date(1592949700000);
    expectedResult.setHours(0, 0, 0, 0);

    expect(result).toEqual(expectedResult);
  });

  it('should return the first timeslot', function() {
    var timeslots = [
      { fromDate: 1592949700000, toDate: 1592949800000, opened: true, available: true },
      { fromDate: 1593036100000, toDate: 1593036200000, opened: true, available: true }
    ];
    var result = timePickerUtils.getMinDate(timeslots);
    expect(result).toEqual(new Date(1592949700000));
  });

  it('should return the last timeslot', function() {
    var timeslots = [
      { fromDate: 1592949700000, toDate: 1592949800000, opened: true, available: true },
      { fromDate: 1593036100000, toDate: 1593036200000, opened: true, available: true }
    ];
    var result = timePickerUtils.getMaxDate(timeslots);
    expect(result).toEqual(new Date(1593036100000));
  });

});
